@@ -26,18 +26,20 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
-#include "APL_types.hh"
 #include "Archive.hh"
-#include "ComplexCell.hh"
+#include "Common.hh"
 #include "CharCell.hh"
+#include "ComplexCell.hh"
 #include "Executable.hh"
 #include "IntCell.hh"
 #include "FloatCell.hh"
 #include "Function.hh"
 #include "IndexExpr.hh"
 #include "LvalCell.hh"
+#include "NativeFunction.hh"
 #include "Output.hh"
 #include "PointerCell.hh"
+#include "PrintOperator.hh"
 #include "Symbol.hh"
 #include "Token.hh"
 #include "UCS_string.hh"
@@ -84,9 +86,9 @@ XML_Saving_Archive::find_owner(const Cell * cell)
 {
    loop(v, values.size())
       {
-        Value_P val = values[v]._val;
-        if (cell < &val->get_ravel(0))      continue;
-        if (cell >= val->get_ravel_end())   continue;
+        const Value & val = values[v]._val;
+        if (cell < &val.get_ravel(0))      continue;
+        if (cell >= val.get_ravel_end())   continue;
         return v;
       }
 
@@ -95,14 +97,13 @@ XML_Saving_Archive::find_owner(const Cell * cell)
 }
 //-----------------------------------------------------------------------------
 int
-XML_Saving_Archive::find_vid(Value_P val)
+XML_Saving_Archive::find_vid(const Value & val)
 {
    loop(v, values.size())
       {
-         if (val == values[v]._val)   return v;
+         if (&val == &values[v]._val)   return v;
       }
 
-   FIXME;
    return -1;
 }
 //-----------------------------------------------------------------------------
@@ -159,30 +160,29 @@ bool char_mode = false;
 }
 //-----------------------------------------------------------------------------
 XML_Saving_Archive &
-XML_Saving_Archive::operator <<(Value_P v)
+XML_Saving_Archive::save_shape(const Value & v)
 {
 char cc[80];
 
    do_indent();
    snprintf(cc, sizeof(cc), "<Value flg=\"%X\" vid=\"%d\"",
-                 v->get_flags(), vid);
+                 v.get_flags(), vid);
    out << cc;
 
-   if (v->is_nested())
-      {
-        const unsigned int sub_vid = find_vid(v);
-        Assert(sub_vid < values.size());
-        unsigned int parent_vid = -1;
-        if (values[sub_vid]._par)   parent_vid = find_vid(values[sub_vid]._par);
-        snprintf(cc, sizeof(cc), " parent=\"%d\"", parent_vid);
-        out << cc;
-      }
-   snprintf(cc, sizeof(cc), " rk=\"%u\"", v->get_rank());
+const unsigned int sub_vid = find_vid(v);
+   Assert(sub_vid < values.size());
+unsigned int parent_vid = -1;
+
+   if (&values[sub_vid]._par)   parent_vid = find_vid(values[sub_vid]._par);
+
+   snprintf(cc, sizeof(cc), " parent=\"%d\"", parent_vid);
+   out << cc;
+   snprintf(cc, sizeof(cc), " rk=\"%u\"", v.get_rank());
    out << cc;
 
-   loop (r, v->get_rank())
+   loop (r, v.get_rank())
       {
-        snprintf(cc, sizeof(cc), " sh-%llu=\"%llu\"", r, v->get_shape_item(r));
+        snprintf(cc, sizeof(cc), " sh-%llu=\"%llu\"", r, v.get_shape_item(r));
         out << cc;
       }
 
@@ -191,7 +191,7 @@ char cc[80];
 }
 //-----------------------------------------------------------------------------
 XML_Saving_Archive &
-XML_Saving_Archive::operator <<(Value & v)
+XML_Saving_Archive::save_ravel(const Value & v)
 {
 int space = do_indent();
 bool char_mode = false;
@@ -252,7 +252,7 @@ char cc[80];
 
             case CT_POINTER:
                  {
-                   const int vid = find_vid(cell.get_pointer_value());
+                   const int vid = find_vid(*cell.get_pointer_value());
                    snprintf(cc, sizeof(cc), "%d", vid);
                    NEED(1 + strlen(cc), false) << UNI_PAD_U6 << decr(space, cc);
                    space--;   // PAD_U6
@@ -263,8 +263,8 @@ char cc[80];
                  {
                    Cell * cp = cell.get_lval_value();
                    const int vid = find_owner(cp);
-                   Value_P val = values[vid]._val;
-                   const ShapeItem offset = val->get_offset(cp);
+                   const Value & val = values[vid]._val;
+                   const ShapeItem offset = val.get_offset(cp);
                    snprintf(cc, sizeof(cc), "%d[%lld]", vid, offset);
                    NEED(1 + strlen(cc), false) << UNI_PAD_U7 << decr(space, cc);
                    space--;   // PAD_U6
@@ -277,7 +277,9 @@ XML_Saving_Archive &
 XML_Saving_Archive::operator <<(const Function & fun)
 {
    do_indent();
-   out << "<Function>" << endl;
+   out << "<Function";
+   if (fun.is_native())   out << " native=\"1\"";
+   out << ">" << endl;
    ++indent;
 
    *this <<  fun.canonical(false);
@@ -293,7 +295,7 @@ XML_Saving_Archive &
 XML_Saving_Archive::operator <<(const Prefix & prefix)
 {
    do_indent();
-   out << "<Parser assign-pending=\"" << prefix.get_assign_pending()
+   out << "<Parser assign-pending=\"" << prefix.get_assign_state()
        << "\" lookahead-high=\""      << prefix.get_lookahead_high()
        << "\">" << endl;
 
@@ -377,9 +379,9 @@ XML_Saving_Archive::operator <<(const StateIndicator & si)
 
    if (si.eval_arg_F)   // function pending
       {
-        if (si.eval_arg_A)   // function has a left arg
+        if (!!si.eval_arg_A)   // function has a left arg
            {
-             const unsigned int vid_A = find_vid(si.eval_arg_A);
+             const unsigned int vid_A = find_vid(*si.eval_arg_A.get());
              out << " vid_arg_A=\"" << vid_A << "\"";
            }
 
@@ -389,14 +391,14 @@ XML_Saving_Archive::operator <<(const StateIndicator & si)
           out << cc;
         }
 
-        if (si.eval_arg_B)   // function has a right arg
+        if (!!si.eval_arg_B)   // function has a right arg
            {
-             const int vid_B = find_vid(si.eval_arg_B);
+             const int vid_B = find_vid(*si.eval_arg_B.get());
              out << " vid_arg_B=\"" << vid_B << "\"";
            }
       }
 
-   out <<"/>" << endl;
+   out <<">" << endl;
 
    ++indent;
    do_indent();
@@ -404,7 +406,7 @@ XML_Saving_Archive::operator <<(const StateIndicator & si)
       {
         case PM_FUNCTION:
              {
-               Symbol * sym = workspace.lookup_symbol(exec.get_name());
+               Symbol * sym = Workspace::lookup_symbol(exec.get_name());
                Assert(sym);
                const UserFunction * ufun = exec.get_ufun();
                Assert(ufun);
@@ -525,7 +527,7 @@ XML_Saving_Archive::emit_token_val(const Token & tok)
         case TV_LIN:   out << " line=\"" << tok.get_fun_line() << "\"/";
                        break;
 
-        case TV_VAL:   { const int vid = find_vid(tok.get_apl_val());
+        case TV_VAL:   { const int vid = find_vid(*tok.get_apl_val());
                          out << " vid=\"" << vid << "\"/";
                        }
                        break;
@@ -536,8 +538,8 @@ XML_Saving_Archive::emit_token_val(const Token & tok)
                          loop(i, rank)
                              {
                                if (i)   out << ",";
-                               Value_P val = idx.values[i];
-                               if (val)   out << "vid_" << find_vid(val);
+                               const Value & val = *idx.values[i].get();
+                               if (&val)   out << "vid_" << find_vid(val);
                                else       out << "-";
                                 out << "\"/>";
                              }
@@ -551,7 +553,7 @@ XML_Saving_Archive::emit_token_val(const Token & tok)
                          if (ufun)   // user defined function
                             {
                               const UCS_string & fname = ufun->get_name();
-                              Symbol * sym = workspace.lookup_symbol(fname);
+                              Symbol * sym = Workspace::lookup_symbol(fname);
                               Assert(sym);
                               const int sym_depth = sym->get_ufun_depth(ufun);
                               out << " ufun-name=\"" << fname
@@ -590,7 +592,7 @@ XML_Saving_Archive::operator <<(const ValueStackItem & vsi)
 
         case NC_VARIABLE:
              do_indent();
-             out << "<Variable vid=\"" << find_vid(vsi.sym_val.value)
+             out << "<Variable vid=\"" << find_vid(*vsi.apl_val.get())
                  << "\"/>" << endl;
              break;
 
@@ -611,7 +613,7 @@ XML_Saving_Archive::operator <<(const ValueStackItem & vsi)
 }
 //-----------------------------------------------------------------------------
 XML_Saving_Archive &
-XML_Saving_Archive::operator <<(const Workspace & ws)
+XML_Saving_Archive::save()
 {
 tm * t;
    {
@@ -722,14 +724,14 @@ tm * t;
 "    <!-- hour/minute/second is )SAVE time in UTC (aka. GMT).\n"
 "         timezone is offset to UTC in seconds.\n"
 "         local time is UTC + offset -->\n"
-"<Workspace wsid=\""     << workspace.WS_name
+"<Workspace wsid=\""     << Workspace::get_WS_name()
      << "\" year=\""     << (t->tm_year + 1900)
      << "\" month=\""    << (t->tm_mon  + 1)
      << "\" day=\""      <<  t->tm_mday << "\"" << endl <<
 "           hour=\""     <<  t->tm_hour
      << "\" minute=\""   <<  t->tm_min
      << "\" second=\""   <<  t->tm_sec
-     << "\" timezone=\"" << ws.v_quad_TZ.get_offset()
+     << "\" timezone=\"" << Workspace::get_v_quad_TZ().get_offset()
      << "\">\n" << endl;
 
    ++indent;
@@ -738,37 +740,37 @@ tm * t;
    // saving of stale values and unmark the used values
    //
    Value::mark_all_dynamic_values();
-   Workspace::the_workspace->unmark_all_values();
+   Workspace::unmark_all_values();
 
    for (const DynamicObject * obj = DynamicObject::get_all_values()->get_next();
         obj != DynamicObject::get_all_values(); obj = obj->get_next())
        {
-         Value * val = (Value *)obj;
+         const Value & val(*(const Value *)obj);
 
-         if (val->is_forever())   continue;   // e.g. ⍞
-         if (val->is_marked())    continue;
+         if (val.is_forever())   continue;   // e.g. ⍞
+         if (val.is_marked())    continue;   // stale
 
-         val->clear_marked();
-         values.push_back(val);
+         val.clear_marked();
+         values.push_back(_val_par(val));
        }
 
    // set up parents of values
    //
    loop(p, values.size())
       {
-        Value_P parent = values[p]._val;
-        const ShapeItem ec = parent->nz_element_count();
-        const Cell * cP = &parent->get_ravel(0);
+        const Value & parent = values[p]._val;
+        const ShapeItem ec = parent.nz_element_count();
+        const Cell * cP = &parent.get_ravel(0);
         loop(e, ec)
             {
               if (cP->is_pointer_cell())
                  {
-                   Value_P sub = cP->get_pointer_value();
-                   Assert(sub);
+                   const Value & sub = *cP->get_pointer_value().get();
+                   Assert1(&sub);
                    unsigned int sub_idx = find_vid(sub);
                    Assert(sub_idx < values.size());
-                   Assert(values[sub_idx]._par == 0);
-                   values[sub_idx]._par = parent;
+                   Assert(!&values[sub_idx]._par);
+                   values[sub_idx] = _val_par(values[sub_idx]._val,  parent);
                  }
               else if (cP->is_lval_cell())
                  {
@@ -780,26 +782,26 @@ CERR << "LVAL CELL in " << p << " at " LOC << endl;
 
    // save all values (without their ravel)
    //
-   for (vid = 0; vid < values.size(); ++vid)   *this << values[vid]._val;
+   for (vid = 0; vid < values.size(); ++vid)   save_shape(values[vid]._val);
 
    // save ravels of all values
    //
-   for (vid = 0; vid < values.size(); ++vid)   *this << *values[vid]._val;
+   for (vid = 0; vid < values.size(); ++vid)   save_ravel(values[vid]._val);
 
    // save user defined symbols
    //
-   *this << ws.symbol_table;
+   *this << Workspace::get_symbol_table();
 
    // save certain system variables
    //
-#define rw_sv_def(x) *this << ws.v_quad_ ## x;
-#define ro_sv_def(x) *this << ws.v_quad_ ## x;
+#define rw_sv_def(x) *this << Workspace::get_v_quad_ ## x();
+#define ro_sv_def(x) *this << Workspace::get_v_quad_ ## x();
 #include "SystemVariable.def"
 
    // save state indicator
    //
    {
-     const int levels = ws.SI_entry_count();
+     const int levels = Workspace::SI_entry_count();
      do_indent();
      out << "<StateIndicator levels=\"" << levels << "\">" << endl;
 
@@ -807,7 +809,7 @@ CERR << "LVAL CELL in " << p << " at " LOC << endl;
 
      loop(l, levels)
         {
-          for (const StateIndicator * si = ws.SI_top();
+          for (const StateIndicator * si = Workspace::SI_top();
                si; si = si->get_parent())
               {
                 if (si->get_level() == l)
@@ -835,11 +837,10 @@ CERR << "LVAL CELL in " << p << " at " LOC << endl;
        << char(0) << char(0) <<char(0) <<char(0) << endl;
 }
 //=============================================================================
-XML_Loading_Archive::XML_Loading_Archive(const char * filename, Workspace & ws)
+XML_Loading_Archive::XML_Loading_Archive(const char * filename)
    : line(1),
      data(0),
      end(0),
-     workspace(ws),
      copying(false),
      protection(false),
      reading_vids(false)
@@ -1106,6 +1107,11 @@ const char ** tag_pos = tag_order;
          else    /* complain */               expect_tag("UNEXPECTED", LOC);
        }
 
+   // loaded workspace can contain stale variables, e.g. shared vars.
+   // remove them.
+   //
+   Value::erase_stale(LOC);
+
 const char * tz_sign = (tzone < 0) ? "" : "+";
    COUT << "SAVED " << year << "-" << month << "-" << day
         << "  " << hour << ":" << minute << ":" << second
@@ -1114,7 +1120,7 @@ const char * tz_sign = (tzone < 0) ? "" : "+";
 const UTF8 * end = wsid;
    while (*end != '"')   ++end;
 
-   workspace.WS_name = UCS_string(UTF8_string(wsid, end - wsid));
+   Workspace::set_WS_name(UCS_string(UTF8_string(wsid, end - wsid)));
 }
 //-----------------------------------------------------------------------------
 void
@@ -1172,13 +1178,13 @@ bool no_copy = false;   // assume the value is needed
 
    if (no_copy)
       {
-        values.push_back(0);
+        values.push_back(Value_P());
       }
    else
       {
         Assert(vid == values.size());
 
-        Value_P val = new Value(sh_value, LOC);
+        Value_P val(new Value(sh_value, LOC));
         values.push_back(val);
       }
 }
@@ -1282,7 +1288,7 @@ XML_Loading_Archive::read_chars(UCS_string & ucs, const UTF8 * & utf)
 
           if (char_mode && *utf < 0x80)
              {
-               ucs += Unicode(*utf++);
+               ucs.append(Unicode(*utf++));
                continue;
              }
 
@@ -1302,7 +1308,7 @@ XML_Loading_Archive::read_chars(UCS_string & ucs, const UTF8 * & utf)
               char_mode = false;
               char * end = 0;
               const int hex = strtol((const char *)utf, &end, 16);
-              ucs += Unicode(hex);
+              ucs.append(Unicode(hex));
               utf = (const UTF8 *)end;
               continue;
             }
@@ -1331,7 +1337,7 @@ const UTF8 * cells = find_attr("cells", false);
    Assert(vid < values.size());
 Value_P val = values[vid];
 
-   if (val == 0)   // )COPY with vids_COPY or static value
+   if (!val)   // )COPY with vids_COPY or static value
       {
         return;
       }
@@ -1353,6 +1359,8 @@ Cell * end = C + count;
      const Unicode next = UTF8_string::toUni(cells, len);
      Assert(next == '"');
    }
+
+   val->check_value(LOC);
 }
 //-----------------------------------------------------------------------------
 void
@@ -1375,19 +1383,16 @@ const unsigned int vid = find_int_attr("vid", false, 10);
    // inform the user, but continue.
    try
       {
-        symbol.assign(values[vid]);
+        symbol.assign(values[vid], LOC);
       }
    catch (...)
       {
-        CERR << "*** Could not assign value " << *values[vid]
-             << "    to variable " << symbol.get_name() << " ***" << endl;
+        if (symbol.get_name() != id_name(ID_QUAD_SYL))
+           {
+             CERR << "*** Could not assign value " << *values[vid]
+                  << "    to variable " << symbol.get_name() << " ***" << endl;
+           }
       }
-  
-   // if assign() was unsuccessful, e.g. because symbol.is_readonly() then
-   // the assigned flag is not set and erase() succeeeds. Otherwise erase()
-   // does nothing.
-   //
-   values[vid]->erase(LOC);
 }
 //-----------------------------------------------------------------------------
 void
@@ -1401,6 +1406,8 @@ const int value = find_int_attr("value", false, 10);
 void
 XML_Loading_Archive::read_Function(int d, Symbol & symbol)
 {
+const int native     = find_int_attr("native", true, 10);
+
    next_tag(LOC);
    expect_tag("UCS", LOC);
 const UTF8 * uni = find_attr("uni", false);
@@ -1411,11 +1418,30 @@ UCS_string text;
    while (*uni != '"')   read_chars(text, uni);
 
 int err = 0;
-UserFunction * ufun = new UserFunction(text, err, LOC);
-   Assert(err == -1);
+   if (native == 1)
+      {
+        NativeFunction * nfun = NativeFunction::fix(text, symbol.get_name());
+        if (nfun)   // fix succeeded
+           {
+             if (d == 0)   symbol.pop(false);
+             symbol.push_function(nfun);
+           }
+        else        // fix failed
+           {
+             CERR << "   *** loading of native function " << text
+                  << " failed" << endl << endl;;
+             if (d == 0)   symbol.pop(false);
+             symbol.push();
+           }
+      }
+   else
+      {
+        UserFunction * ufun = new UserFunction(text, err, false, LOC);
+        Assert(err == -1);
 
-   if (d == 0)   symbol.pop(false);
-   symbol.push_function(ufun);
+        if (d == 0)   symbol.pop(false);
+        symbol.push_function(ufun);
+      }
 }
 //-----------------------------------------------------------------------------
 void
@@ -1460,7 +1486,7 @@ const int depth = find_int_attr("stack-size", false, 10);
 
    // lookup symbol, trying ⎕xx first
    //
-Symbol * symbol = workspace.lookup_existing_symbol(name_ucs);
+Symbol * symbol = Workspace::lookup_existing_symbol(name_ucs);
 
    // we do NOT copy if:
    //
@@ -1503,7 +1529,7 @@ const bool no_copy = (symbol && protection) || ! is_allowed(name_ucs);
            }
       }
 
-   if (symbol == 0)   symbol = workspace.symbol_table.lookup_symbol(name_ucs);
+   if (symbol == 0)   symbol = Workspace::lookup_symbol(name_ucs);
    Assert(symbol);
 
    loop(d, depth)
@@ -1565,8 +1591,8 @@ const Executable * exec = 0;
    Assert(lev == level);
    Assert(exec);
 
-   workspace.push_SI(exec, LOC);
-StateIndicator * si = workspace.SI_top();
+   Workspace::push_SI(exec, LOC);
+StateIndicator * si = Workspace::SI_top();
    Assert(si);
    if (Id_arg_F != -1)
       {
@@ -1619,7 +1645,7 @@ const UTF8 * n  = name;
 UTF8_string name_utf(name, n - name);
 UCS_string name_ucs(name_utf);
 
-Symbol * symbol = workspace.symbol_table.lookup_symbol(name_ucs);
+Symbol * symbol = Workspace::lookup_symbol(name_ucs);
    Assert(symbol);
    Assert(level >= 0);
    Assert(level < symbol->value_stack_size());
@@ -1639,12 +1665,12 @@ XML_Loading_Archive::read_Parsers(StateIndicator & si)
    next_tag(LOC);
    expect_tag("Parser", LOC);
 
-const int ass_pending = find_int_attr("assign-pending", false, 10);
+const int ass_state = find_int_attr("assign-pending", false, 10);
 const int lah_high = find_int_attr("lookahead-high", false, 10);
 
 Prefix & parser = si.current_stack;
 
-   parser.assign_pending = ass_pending != 0;
+   parser.set_assign_state((Prefix::Assign_state)ass_state);
    parser.set_lookahead_high(Function_PC(lah_high));
 
    for (;;)
@@ -1712,7 +1738,7 @@ const TokenTag tag = TokenTag(find_int_attr("tag", false, 16));
                while (*end != '"')   ++end;
                UTF8_string name_utf(sym_name, end - sym_name);
                UCS_string name_ucs(name_utf);
-               Symbol * symbol = workspace.symbol_table.lookup_symbol(name_ucs);
+               Symbol * symbol = Workspace::lookup_symbol(name_ucs);
                new (&tloc.tok) Token(tag, symbol);
              }
              break;
@@ -1736,13 +1762,13 @@ const TokenTag tag = TokenTag(find_int_attr("tag", false, 16));
              {
                const unsigned int rank = find_int_attr("rank", false, 10);
                char * vids = (char *)find_attr("index", false);
-               IndexExpr * idx = new IndexExpr(LOC);
+               IndexExpr & idx = *new IndexExpr(false, LOC);
                while (*vids != '"')
                   {
                     if (*vids == ',')   ++vids;
                     if (*vids == '-')   // elided index
                        {
-                         idx->add(0);
+                         idx.add(Value_P());
                        }
                     else                // value
                        {
@@ -1753,7 +1779,7 @@ const TokenTag tag = TokenTag(find_int_attr("tag", false, 16));
                          char * end;
                          const unsigned int vid = strtol(vids, &vids, 10);
                          Assert(vid < values.size());
-                         idx->add(values[vid]);
+                         idx.add(values[vid]);
                        }
                   }
                new (&tloc.tok) Token(tag, idx);
@@ -1778,8 +1804,7 @@ const TokenTag tag = TokenTag(find_int_attr("tag", false, 16));
                     while (*end != '"')   ++end;
                     UTF8_string name_utf(fun_name, end - fun_name);
                     UCS_string ucs(name_utf);
-                    const Symbol & symbol =
-                                   *workspace.symbol_table.lookup_symbol(ucs);
+                    const Symbol & symbol = *Workspace::lookup_symbol(ucs);
                     Assert(level >= 0);
                     Assert(level < symbol.value_stack_size());
                     const ValueStackItem & vsi = symbol[level];
