@@ -59,20 +59,19 @@ vector<ShapeItem> rep_counts;
 
 Shape shape_Z(shape_B);
    shape_Z.set_shape_item(axis, ec_A);
-Value_P Z = new Value(shape_Z, LOC);
+Value_P Z(new Value(shape_Z, LOC));
 
    if (ec_A == 0)   // (⍳0)/B : 
       {
         if (shape_B.get_shape_item(axis) > 1)   LENGTH_ERROR;
 
-        Z->set_default(B);
-        return CHECK(Z, LOC);
+        Z->set_default(*B.get());
+        Z->check_value(LOC);
+        return Token(TOK_APL_VALUE1, Z);
       }
 
-
 const Shape3 shape_Z3(shape_Z, axis);
 
-Cell * cZ = &Z->get_ravel(0);
 const Cell * cB = &B->get_ravel(0);
 const bool lval = cB->is_lval_cell();
 
@@ -94,18 +93,18 @@ ShapeItem inc_2 = 0;              // increment after result m*l items
            {
              if (rep_counts[m] == 1)   // copy items from B
                 {
-                  loop(l, shape_Z3.l())   cZ++->init(cB[l]);
+                  loop(l, shape_Z3.l())   Z->next_ravel()->init(cB[l]);
                   cB += inc_1;
                 }
              else                      // init items
                 {
                   if (lval)
                      {
-                       loop(l, shape_Z3.l())   new (cZ++) LvalCell(0);
+                       loop(l, shape_Z3.l())   new (Z->next_ravel()) LvalCell(0);
                      }
                   else
                      {
-                       loop(l, shape_Z3.l())   cZ++->init_type(fill[l]);
+                       loop(l, shape_Z3.l())   Z->next_ravel()->init_type(fill[l]);
                      }
                 }
            }
@@ -113,9 +112,10 @@ ShapeItem inc_2 = 0;              // increment after result m*l items
         cB += inc_2;
       }
 
-   Z->set_default(B);
+   Z->set_default(*B.get());
 
-   return CHECK(Z, LOC);
+   Z->check_value(LOC);
+   return Token(TOK_APL_VALUE1, Z);
 }
 //-----------------------------------------------------------------------------
 Token
@@ -134,18 +134,19 @@ const ShapeItem m_len = B->get_shape_item(axis);
 
    if (m_len == 1)
       {
-        Shape shape_Z(B->get_shape());
-        shape_Z.remove_shape_item(axis);
+        const Shape shape_Z = B->get_shape().without_axis(axis);
         return Bif_F12_RHO::do_reshape(shape_Z, B);
       }
 
 const Shape3 Z3(B->get_shape(), axis);
-_EOC_arg arg;
-Value_P Z = new Value(B->get_shape(), LOC);
-   arg._reduce_beam().init(Z, Z3, LO, B, m_len, 1, 1);
+
+Value_P Z(new Value(B->get_shape(), LOC));
+EOC_arg arg(Z, B);
+REDUCTION & _arg = arg.u.u_REDUCTION;
+
+   _arg.init(Z3, LO, &B->get_ravel(0), m_len, 1, 1);
 
 Token tok(TOK_FIRST_TIME);
-   B->set_eoc();   // keep B
    Bif_REDUCE::eoc_beam(tok, arg);
    return tok;
 }
