@@ -40,10 +40,10 @@ Cell::operator new(std::size_t s, void * pos)
 void
 Cell::init(const Cell & other)
 {
+   Assert(&other);
    switch(other.get_cell_type())
       {
-        case CT_NONE:
-        case CT_BASE:
+        default:
              Assert(0);
 
         case CT_CHAR:
@@ -73,7 +73,6 @@ Cell::init(const Cell & other)
         case CT_CELLREF:
              new (this) LvalCell(other.get_lval_value());
              return;
-
       }
 
    Assert(0 && "Bad cell type");
@@ -85,7 +84,6 @@ Cell::init_from_value(Value_P value, const char * loc)
    if (value->is_skalar())
       {
         init(value->get_ravel(0));
-        value->erase(loc);
       }
    else
       {
@@ -103,11 +101,11 @@ Value_P ret;
       }
    else
       {
-        ret = new Value(loc);
+        ret = Value_P(new Value(loc));
         ret->get_ravel(0).init(*this);
+        ret->check_value(LOC);
       }
 
-   ret->set_arg();
    return ret;
 }
 //-----------------------------------------------------------------------------
@@ -117,10 +115,11 @@ Cell::init_type(const Cell & other)
    if (other.is_pointer_cell())
       {
         Value_P B = other.get_pointer_value();
-        Value_P Z = new Value(B->get_shape(), LOC);
+        Value_P Z(new Value(B->get_shape(), LOC));
 
         const ShapeItem len = B->nz_element_count();
         loop(l, len)   Z->get_ravel(l).init_type(B->get_ravel(l));
+        Z->check_value(LOC);
 
         new (this) PointerCell(Z);
       }
@@ -134,6 +133,16 @@ Cell::init_type(const Cell & other)
       }
 }
 //-----------------------------------------------------------------------------
+void
+Cell::copy(Value & val, const Cell * & src, ShapeItem count)
+{
+   loop(c, count)
+      {
+        Assert1(val.more());
+        val.next_ravel()->init(*src++);
+      }
+}
+//-----------------------------------------------------------------------------
 bool
 Cell::greater(const Cell * other, bool ascending) const
 {
