@@ -69,7 +69,6 @@ Command::process_line(UCS_string & line)
          case UNI_ASCII_R_PARENT:      // regular command, e.g. )SI
          case UNI_ASCII_R_BRACK:       // debug command, e.g. ]LOG
               {
-                Workspace & w = *Workspace::the_workspace;
                 ostream & out = (line[0] == UNI_ASCII_R_PARENT) ? COUT : CERR;
 
                 // split line into command and arguments
@@ -156,14 +155,14 @@ const Executable * statements = 0;
           //
           for (bool goon = true; goon;)
               {
-                StateIndicator * si = Workspace::the_workspace->SI_top();
+                StateIndicator * si = Workspace::SI_top();
                 if (si == 0)   break;   // SI empty
 
                 const Executable * exec = si->get_executable();
                 Assert(exec);
                 goon = exec->get_parse_mode() != PM_STATEMENT_LIST;
                 si->escape();   // pop local vars of user defined functions
-                Workspace::the_workspace->pop_SI(LOC);
+                Workspace::pop_SI(LOC);
               }
           return;
         }
@@ -173,7 +172,7 @@ const Executable * statements = 0;
 
    // push a new context for the statements.
    //
-   Workspace::the_workspace->push_SI(statements, LOC);
+   Workspace::push_SI(statements, LOC);
 
    for (;;)
        {
@@ -181,17 +180,18 @@ const Executable * statements = 0;
          // NOTE that the entire SI may change while executing this loop.
          // We should therefore avoid references to SI entries.
          //
-         Token token = Workspace::the_workspace->SI_top()
+         Token token = Workspace::SI_top()
                                   ->get_executable()->execute_body();
 
 // Q(token)
+
          // start over if execution has pushed a new context
          //
          if (token.get_tag() == TOK_SI_PUSHED)   continue;
 
          // maybe call EOC handler and repeat if true returned
          //
-         if (Workspace::the_workspace->SI_top()->call_eoc_handler(token))
+         if (Workspace::SI_top()->call_eoc_handler(token))
             continue;
 
          // the far most frequent cases are TC_VALUE and TOK_VOID
@@ -199,7 +199,7 @@ const Executable * statements = 0;
          //
          if (token.get_Class() == TC_VALUE || token.get_tag() == TOK_VOID )
             {
-              if (Workspace::the_workspace->SI_top()->get_executable()
+              if (Workspace::SI_top()->get_executable()
                              ->get_parse_mode() == PM_STATEMENT_LIST)
                  {
                    if (attention_raised)
@@ -212,7 +212,7 @@ const Executable * statements = 0;
                    break;   // will return to calling context
                  }
 
-              Workspace::the_workspace->pop_SI(LOC);
+              Workspace::pop_SI(LOC);
 
               // we are back in the calling SI. There should be a TOK_SI_PUSHED
               // token at the top of stack. Replace it with the result from
@@ -220,10 +220,10 @@ const Executable * statements = 0;
               //
               {
                 Prefix & prefix =
-                         Workspace::the_workspace->SI_top()->get_prefix();
+                         Workspace::SI_top()->get_prefix();
                 Assert(prefix.at0().get_tag() == TOK_SI_PUSHED);
 
-                prefix.tos().tok = token;
+                copy_1(prefix.tos().tok, token, LOC);
               }
               if (attention_raised)
                  {
@@ -237,19 +237,19 @@ const Executable * statements = 0;
 
          if (token.get_tag() == TOK_BRANCH)
             {
-              StateIndicator * si = Workspace::the_workspace->SI_top_fun();
+              StateIndicator * si = Workspace::SI_top_fun();
 
               if (si == 0)
                  {
-                    Workspace::the_workspace->more_error = UCS_string(
+                    Workspace::more_error() = UCS_string(
                           _("empty branch (→⍳0) without suspended function"));
                     SYNTAX_ERROR;   // →N without function,
                  }
 
               // pop contexts above defined function
               //
-              while (si != Workspace::the_workspace->SI_top())
-                    Workspace::the_workspace->pop_SI(LOC);
+              while (si != Workspace::SI_top())
+                    Workspace::pop_SI(LOC);
 
               const Function_Line line = Function_Line(token.get_int_val());
 
@@ -265,14 +265,14 @@ const Executable * statements = 0;
               //
               for (bool goon = true; goon;)
                   {
-                    StateIndicator * si = Workspace::the_workspace->SI_top();
+                    StateIndicator * si = Workspace::SI_top();
                     if (si == 0)   break;   // SI empty
 
                     const Executable * exec = si->get_executable();
                     Assert(exec);
                     goon = exec->get_parse_mode() != PM_STATEMENT_LIST;
                     si->escape();   // pop local vars of user defined functions
-                    Workspace::the_workspace->pop_SI(LOC);
+                    Workspace::pop_SI(LOC);
               }
               return;
 
@@ -286,8 +286,8 @@ const Executable * statements = 0;
               attention_raised = false;
               interrupt_raised = false;
 
-              Workspace::the_workspace->SI_top()->get_error().print(CERR);
-              if (Workspace::the_workspace->SI_top()->get_level() == 0)
+              Workspace::get_error()->print(CERR);
+              if (Workspace::SI_top()->get_level() == 0)
                  {
                    Value::erase_stale(LOC);
                    IndexExpr::erase_stale(LOC);
@@ -302,7 +302,7 @@ const Executable * statements = 0;
 
    // pop the context for the statements
    //
-   Workspace::the_workspace->pop_SI(LOC);
+   Workspace::pop_SI(LOC);
 }
 //-----------------------------------------------------------------------------
 void 
@@ -312,7 +312,7 @@ Command::cmd_CHECK(ostream & out)
    //
    {
      bool erased = false;
-     const int stale = Workspace::the_workspace->cleanup_expunged(CERR, erased);
+     const int stale = Workspace::cleanup_expunged(CERR, erased);
      if (stale)
         {
           char cc[200];
@@ -348,10 +348,10 @@ Command::cmd_CHECK(ostream & out)
 void 
 Command::cmd_CONTINUE(ostream & out)
 {
-   Workspace::the_workspace->wsid(out, UCS_string("CONTINUE"));
+   Workspace::wsid(out, UCS_string("CONTINUE"));
 
 vector<UCS_string> vcont;
-   Workspace::the_workspace->save_WS(out, vcont);
+   Workspace::save_WS(out, vcont);
    cmd_OFF(0);
 }
 //-----------------------------------------------------------------------------
@@ -368,7 +368,7 @@ Command::cmd_DROP(ostream & out, const vector<UCS_string> & lib_ws)
       {
         out << _("BAD COMMAND") << endl;
 
-        UCS_string & t4 = Workspace::the_workspace->more_error;
+        UCS_string & t4 = Workspace::more_error();
         t4 = UCS_string(_("missing workspace name in command )DROP"));
         return;
       }
@@ -376,7 +376,7 @@ Command::cmd_DROP(ostream & out, const vector<UCS_string> & lib_ws)
    if (lib_ws.size() > 2)   // too many arguments
       {
         out << _("BAD COMMAND") << endl;
-        UCS_string & t4 = Workspace::the_workspace->more_error;
+        UCS_string & t4 = Workspace::more_error();
 
         t4 = UCS_string(_("too many parameters in command )DROP"));
         return;
@@ -390,21 +390,21 @@ UCS_string wname = lib_ws.back();
    if (lib_ws.size() == 2)   libref = (LibRef)(lib_ws.front().atoi());
 UTF8_string filename = LibPaths::get_lib_filename(libref, wname, true, "xml");
 
-int result = unlink((const char *)filename.c_str());
+int result = unlink(filename.c_str());
    if (result)
       {
         out << wname << _(" NOT DROPPED: ") << strerror(errno) << endl;
-        UCS_string & t4 = Workspace::the_workspace->more_error;
+        UCS_string & t4 = Workspace::more_error();
         t4.clear();
-        t4.app(_("could not unlink file "));
-        t4.app(filename.c_str());
+        t4.append_ascii(_("could not unlink file "));
+        t4.append_utf8(filename.c_str());
       }
 }
 //-----------------------------------------------------------------------------
 void
 Command::cmd_ERASE(ostream & out, vector<UCS_string> & args)
 {
-   Workspace::the_workspace->erase_symbols(CERR, args);
+   Workspace::erase_symbols(CERR, args);
 }
 //-----------------------------------------------------------------------------
 void 
@@ -442,7 +442,7 @@ void
 Command::cmd_HOST(ostream & out, const UCS_string & arg)
 {
 UTF8_string host_cmd(arg);
-FILE * pipe = popen((const char *)host_cmd.c_str(), "r");
+FILE * pipe = popen(host_cmd.c_str(), "r");
    if (pipe == 0)   // popen failed
       {
         out << _(")HOST command failed: ") << strerror(errno) << endl;
@@ -466,7 +466,7 @@ Command::cmd_IN(ostream & out, vector<UCS_string> & args, bool protection)
    if (args.size() == 0)
       {
         out << _("BAD COMMAND") << endl;
-        Workspace::the_workspace->more_error =
+        Workspace::more_error() =
                    UCS_string(_("missing filename in command )IN"));
         return;
       }
@@ -476,16 +476,18 @@ UCS_string fname = args.front();
 
 UTF8_string filename = LibPaths::get_lib_filename(LIB_NONE, fname, true, "atf");
 
-FILE * in = fopen((const char *)(filename.c_str()), "r");
+FILE * in = fopen(filename.c_str(), "r");
    if (in == 0)   // open failed: try filename.atf unless already .atf
       {
-        CERR << ")IN " << fname << _(" failed: ") << strerror(errno) << endl;
+        UTF8_string fname_utf8(fname);
+        CERR << ")IN " << fname_utf8.c_str()
+             << _(" failed: ") << strerror(errno) << endl;
 
         char cc[200];
         snprintf(cc, sizeof(cc),
                  _("command )IN: could not open file %s for reading: %s"),
-                 fname.c_str(), strerror(errno));
-        Workspace::the_workspace->more_error = UCS_string(cc);
+                 fname_utf8.c_str(), strerror(errno));
+        Workspace::more_error() = UCS_string(cc);
         return;
       }
 
@@ -541,7 +543,7 @@ Command::cmd_LIBS(ostream & out, const vector<UCS_string> & lib_ref)
    if (lib_ref.size() > 0)   // set path
       {
         UTF8_string utf(lib_ref[0]);
-        LibPaths::set_APL_lib_root((const char *)utf.c_str());
+        LibPaths::set_APL_lib_root(utf.c_str());
         out << "LIBRARY ROOT SET TO " << lib_ref[0] << endl;
         return;
       }
@@ -571,7 +573,7 @@ Command::cmd_LIBS(ostream & out, const vector<UCS_string> & lib_ref)
              }
 
         out << left << setw(52) << path.c_str();
-        DIR * dir = opendir((const char *)path.c_str());
+        DIR * dir = opendir(path.c_str());
         if (dir)   { out << " present" << endl;   closedir(dir); }
         else       { out << " missing (" << errno << ")" << endl; }
       }
@@ -585,7 +587,7 @@ Command::cmd_LIB(ostream & out, const UCS_string & arg)
 {
 UTF8_string path = LibPaths::get_lib_dir((LibRef)(arg.atoi()));
 
-DIR * dir = opendir((const char *)path.c_str());
+DIR * dir = opendir(path.c_str());
    if (dir == 0)
       {
         out << _("IMPROPER LIBRARY REFERENCE") << ": " << path << "/ : "
@@ -595,7 +597,7 @@ DIR * dir = opendir((const char *)path.c_str());
           snprintf(cc, sizeof(cc),
                    _("path %s: could not be openend as directory: %s"),
                    path.c_str(), strerror(errno));
-        Workspace::the_workspace->more_error = UCS_string(cc);
+        Workspace::more_error() = UCS_string(cc);
         return;
       }
 
@@ -658,13 +660,13 @@ Command::cmd_LOG(ostream & out, const UCS_string & arg)
 void 
 Command::cmd_MORE(ostream & out)
 {
-   if (Workspace::the_workspace->more_error.size() == 0)
+   if (Workspace::more_error().size() == 0)
       {
         out << _("NO MORE ERROR INFO") << endl;
         return;
       }
 
-   out << Workspace::the_workspace->more_error << endl;
+   out << Workspace::more_error() << endl;
    return;
 }
 //-----------------------------------------------------------------------------
@@ -683,7 +685,7 @@ Command::cmd_OUT(ostream & out, vector<UCS_string> & args)
    if (args.size() == 0)
       {
         out << _("BAD COMMAND") << endl;
-        Workspace::the_workspace->more_error =
+        Workspace::more_error() =
                    UCS_string(_("missing filename in command )OUT"));
         return;
       }
@@ -693,20 +695,21 @@ UCS_string fname = args.front();
 
 UTF8_string filename = LibPaths::get_lib_filename(LIB_NONE, fname, false,"atf");
    
-FILE * atf = fopen((const char *)(filename.c_str()), "w");
+FILE * atf = fopen(filename.c_str(), "w");
    if (atf == 0)
       {
+        UTF8_string fname_utf8(fname);
         out << ")OUT " << fname << _(" failed: ") << strerror(errno) << endl;
         char cc[200];
         snprintf(cc, sizeof(cc),
                  _("command )OUT: could not open file %s for writing: %s"),
-                 fname.c_str(), strerror(errno));
-        Workspace::the_workspace->more_error = UCS_string(cc);
+                 fname_utf8.c_str(), strerror(errno));
+        Workspace::more_error() = UCS_string(cc);
         return;
       }
 
 uint64_t seq = 1;   // sequence number for records written
-   Workspace::the_workspace->write_OUT(atf, seq, args);
+   Workspace::write_OUT(atf, seq, args);
 
    fclose(atf);
 }
@@ -797,7 +800,7 @@ const char sub_type = record[1];
                 {
                   const char * stype = _(" *** bad sub-record of X");
 
-                          " -------------------------------------";
+//                          " -------------------------------------";
                   switch(sub_type)
                      {
                        case 'A': stype = " 2 ⎕TF array ";           break;
@@ -841,7 +844,7 @@ int idx = 1;
    // data + 1 is: NAME RK SHAPE RAVEL...
    //
    while (idx < data.size() && data[idx] != UNI_ASCII_SPACE)
-         name += data[idx++];
+         name.append(data[idx++]);
    ++idx;   // skip space after the name
 
 int rank = 0;
@@ -881,16 +884,16 @@ int idx = get_nrs(var_name, shape);
    if (objects.size() && !var_name.contained_in(objects))   return;
 
 Symbol * sym = 0;
-   if (var_name[0] == UNI_QUAD_QUAD)   // system variable.
+   if (Avec::is_quad(var_name[0]))   // system variable.
       {
         int len = 0;
-        const Token t = Workspace::the_workspace->get_quad(var_name, len);
+        const Token t = Workspace::get_quad(var_name, len);
         if (t.get_ValueType() == TV_SYM)   sym = t.get_sym_ptr();
         else                               Assert(0 && "Bad system variable");
       }
    else                            // user defined variable
       {
-        sym = Workspace::the_workspace-> symbol_table.lookup_symbol(var_name);
+        sym = Workspace::lookup_symbol(var_name);
         Assert(sym);
       }
    
@@ -908,9 +911,9 @@ Token_string tos;
      if (tokenizer.tokenize(data1, tos) != E_NO_ERROR)   return;
    }
  
-   if (tos.size() != shape.element_count())   return;
+   if (tos.size() != shape.get_volume())   return;
 
-Value_P val = new Value(shape, LOC);
+Value_P val(new Value(shape, LOC));
    new (&val->get_ravel(0)) IntCell(0);   // prototype
 
 const ShapeItem ec = val->element_count();
@@ -927,7 +930,7 @@ const ShapeItem ec = val->element_count();
       }
 
    Assert(sym);
-   sym->assign(val);
+   sym->assign(val, LOC);
 }
 //-----------------------------------------------------------------------------
 void
@@ -940,16 +943,16 @@ int idx = get_nrs(var_name, shape);
    if (objects.size() && !var_name.contained_in(objects))   return;
 
 Symbol * sym = 0;
-   if (var_name[0] == UNI_QUAD_QUAD)   // system variable.
+   if (Avec::is_quad(var_name[0]))   // system variable.
       {
         int len = 0;
-        const Token t = Workspace::the_workspace->get_quad(var_name, len);
+        const Token t = Workspace::get_quad(var_name, len);
         if (t.get_ValueType() == TV_SYM)   sym = t.get_sym_ptr();
         else                               Assert(0 && "Bad system variable");
       }
    else                            // user defined variable
       {
-        sym = Workspace::the_workspace-> symbol_table.lookup_symbol(var_name);
+        sym = Workspace::lookup_symbol(var_name);
         Assert(sym);
       }
    
@@ -960,14 +963,14 @@ Symbol * sym = 0;
         CERR << "'" << endl;
       }
 
-Value_P val = new Value(shape, LOC);
+Value_P val(new Value(shape, LOC));
    new (&val->get_ravel(0)) CharCell(UNI_ASCII_SPACE);   // prototype
 
 const ShapeItem ec = val->element_count();
    loop(e, ec)   new (&val->get_ravel(e)) CharCell(data[idx + e]);
 
    Assert(sym);
-   sym->assign(val);
+   sym->assign(val, LOC);
 }
 //-----------------------------------------------------------------------------
 void
@@ -988,7 +991,7 @@ UCS_string var_or_fun;
            {
              const Unicode uni = data1[d];
              if (uni == UNI_LEFT_ARROW)   break;
-             var_name += uni;
+             var_name.append(uni);
            }
 
         if (!var_name.contained_in(objects))   return;
@@ -1008,19 +1011,19 @@ UCS_string fun_name;
 
    /// chars 1...' ' are the function name
    while ((idx < data.size()) && (data[idx] != UNI_ASCII_SPACE))
-        fun_name += data[idx++];
+        fun_name.append(data[idx++]);
    ++idx;
 
    if (objects.size() && !fun_name.contained_in(objects))   return;
 
 UCS_string statement;
-   while (idx < data.size())   statement += data[idx++];
-   statement += UNI_ASCII_LF;
+   while (idx < data.size())   statement.append(data[idx++]);
+   statement.append(UNI_ASCII_LF);
 
 UCS_string fun_name1;
    Quad_TF::tf2_parse(statement, fun_name1);
 
-Symbol * sym1 = Workspace::the_workspace->lookup_existing_symbol(fun_name1);
+Symbol * sym1 = Workspace::lookup_existing_symbol(fun_name1);
    Assert(sym1);
 Function * fun1 = sym1->get_function();
    Assert(fun1);
@@ -1123,13 +1126,13 @@ Command::transfer_context::add(const UTF8 * str, int len)
              case ('*' + 1) ... ('^' - 1):   // < '^'
              case ('^' + 1) ... ('~' - 1):   // < '~'
              case ('~' + 1):                 // i.e. ASCII except * ^ ~
-                  data += Unicode(utf);         break;
+                  data.append(Unicode(utf));         break;
 
-             case '^': data += UNI_AND;              break;   // ~ → ∼
-             case '*': data += UNI_STAR_OPERATOR;    break;   // * → ⋆
-             case '~': data += UNI_TILDE_OPERATOR;   break;   // ~ → ∼
+             case '^': data.append(UNI_AND);              break;   // ~ → ∼
+             case '*': data.append(UNI_STAR_OPERATOR);    break;   // * → ⋆
+             case '~': data.append(UNI_TILDE_OPERATOR);   break;   // ~ → ∼
              
-             default:  data += Unicode(cp_to_uni_map[utf - 128]);
+             default:  data.append(Unicode(cp_to_uni_map[utf - 128]));
            }
       }
 }
@@ -1178,14 +1181,15 @@ bool got_minus = false;
      while (s < user_arg.size() && user_arg[s] <= ' ') ++s;
      if (s == user_arg.size())   return false;   // empty argument: OK
 
-     while (s < user_arg.size() && user_arg[s] > ' ' &&
-                                   user_arg[s] != '-')  from += user_arg[s++];
+     while (s < user_arg.size()   &&
+                user_arg[s] > ' ' &&
+                user_arg[s] != '-')  from.append(user_arg[s++]);
      while (s < user_arg.size() && user_arg[s] <= ' ') ++s;
 
      if (s < user_arg.size() && user_arg[s] == '-') { ++s;   got_minus = true; }
 
      while (s < user_arg.size() && user_arg[s] <= ' ') ++s;
-     while (s < user_arg.size() && user_arg[s] > ' ')  to += user_arg[s++];
+     while (s < user_arg.size() && user_arg[s] > ' ')  to.append(user_arg[s++]);
      while (s < user_arg.size() && user_arg[s] <= ' ') ++s;
 
 const bool got_rest = (s < user_arg.size());
