@@ -27,9 +27,9 @@
 
 #include "Common.hh"
 #include "Output.hh"
+#include "Value.hh"
 
 uint64_t total_memory = 0;
-uint64_t used_memory = 0;
 
 const char * scriptname = 0;
 
@@ -94,8 +94,20 @@ APL_time ret =  mktime(&t);
    return ret;
 }
 //-----------------------------------------------------------------------------
-ostream & operator << (ostream & out, const Function_PC2 & ft)
+ostream &
+operator << (ostream & out, const Function_PC2 & ft)
 {
    return out << ft.low << ":" << ft.high;
 }
 //-----------------------------------------------------------------------------
+ostream &
+print_flags (ostream & out, ValueFlags flags)
+{
+   return out << ((flags & VF_marked)   ?  "M" : "-")
+              << ((flags & VF_complete) ?  "C" : "-")
+              << ((flags & VF_left)     ?  "&" : "-")
+              << ((flags & VF_dirty)    ?  "?" : "-")
+              << ((flags & VF_forever)  ?  "∞" : "-");
+}
+//-----------------------------------------------------------------------------
+
