@@ -21,7 +21,7 @@
 #include <unistd.h>
 
 #include "Command.hh"
-#include "Common.hh"
+#include "Common.hh"   // for HAVE_LIBREADLINE etc.
 #include "Input.hh"
 #include "main.hh"
 #include "Output.hh"
@@ -31,22 +31,44 @@
 #include "UTF8_string.hh"
 #include "Workspace.hh"
 
-#include "../config.h"   // for HAVE_LIBREADLINE
+/// an optional function called before printing a prompt and reading input
+extern void (*start_input)();
 
+/// an optional function called after having read a line of input
+extern void (*end_input)();
+
+void (*start_input)() = 0;
+void (*end_input)() = 0;
+
+#if HAVE_LIBREADLINE
+
+bool Input::use_readline = true;
 
 namespace readline_lib
 {
-#if HAVE_LIBREADLINE
-
 #include <readline/readline.h>
 #include <readline/history.h>
+};   // namespace readline_lib
 
 #else // ! HAVE_LIBREADLINE
 
-static void add_history(const char * line) {}
-static char * no_readline(const UCS_string * prompt);
+bool Input::use_readline = false;
 
-char *
+namespace readline_lib
+{
+static void add_history(const char * line) {}
+static void rl_initialize() {}
+static void stifle_history(int) {}
+static void read_history(const char *) {}
+static void rl_stuff_char(char) {}
+static char * readline(const char *) {}
+};   // namespace readline_lib
+
+#endif //  HAVE_LIBREADLINE
+
+// a function to be used if readline_lib::readline() does not exist or
+// is not wanted
+static char *
 no_readline(const UCS_string * prompt)
 {
    if (prompt)
@@ -60,7 +82,7 @@ no_readline(const UCS_string * prompt)
            }
       }
 
-static char buffer[2000];   // returned as result
+static char buffer[MAX_INPUT_LEN];   // returned as result
 const char * s = fgets(buffer, sizeof(buffer) - 1, stdin);
    if (s == 0)   return 0;
 
@@ -70,26 +92,24 @@ int len = strlen(buffer);
    return buffer;
 }
 
-#endif //  HAVE_LIBREADLINE
-};   // namespace readline_lib
 
 const char * Input::input_file_name = 0;
 FILE * Input::input_file_FILE = 0;
 
-Input Input::the_input;
-
 bool Input::stdin_from_file = false;
 
 //-----------------------------------------------------------------------------
-Input::Input()
+void
+Input::init()
 {
-#if HAVE_LIBREADLINE
-   readline_lib::rl_initialize();
-   readline_lib::stifle_history(500);
-   readline_lib::read_history(".apl.history");
+   if (use_readline)
+      {
+        readline_lib::rl_initialize();
+        readline_lib::stifle_history(500);
+        readline_lib::read_history(".apl.history");
 
-//   readline_lib::rl_function_dumper(1);
-#endif
+//      readline_lib::rl_function_dumper(1);
+      }
 }
 //-----------------------------------------------------------------------------
 int
@@ -113,13 +133,13 @@ const UTF8 * buf = TestFiles::get_testcase_line();
    if (buf)   // we got a line from a test file.
       {
         input_type = "-T";
-        CIN << Workspace::the_workspace->prompt << buf << endl;
+        CIN << Workspace::get_prompt() << buf << endl;
       }
    else if (input_file_FILE)   // a -f file is open
       {
         input_type = "-f";
         buf = get_f_line(input_file_FILE);
-        if (buf)   CIN << Workspace::the_workspace->prompt << buf << endl;
+        if (buf)   CIN << Workspace::get_prompt() << buf << endl;
       }
    else if (input_file_name)   // -f present, but not opened yet
       {
@@ -155,7 +175,7 @@ const UTF8 * buf = TestFiles::get_testcase_line();
    if (buf == 0)   // all reads from files failed
       {
         input_type = "U";
-        buf = get_user_line(&Workspace::the_workspace->prompt);
+        buf = get_user_line(&Workspace::get_prompt(), false);
       }
 
    if (buf == 0)   // ^D or end of file
@@ -188,7 +208,7 @@ UTF8_string utf(buf1, len);
 UTF8 *
 Input::get_f_line(FILE * file)
 {
-static char buffer[2000];
+static char buffer[MAX_INPUT_LEN];
 
 const char * s = fgets(buffer, sizeof(buffer) - 1, file);
    if (s == 0)   // end of file
@@ -216,50 +236,54 @@ Input::exit_readline()
 }
 //-----------------------------------------------------------------------------
 const unsigned char *
-Input::get_user_line(const UCS_string * prompt)
+Input::get_user_line(const UCS_string * prompt, bool from_nabla)
 {
-   Output::set_color_mode(Output::COLM_INPUT);
+   if (start_input)   (*start_input)();
+
+   if (!from_nabla)   Output::set_color_mode(Output::COLM_INPUT);
 
 const APL_time from = now();
-#if HAVE_LIBREADLINE
 char * line;
-   if (prompt)
+   if (!use_readline)
+      {
+        line = no_readline(prompt);
+      }
+   else if (prompt)
       {
         UTF8_string prompt_utf(*prompt);
-        line = readline_lib::readline((const char*)prompt_utf.c_str());
+        line = readline_lib::readline(prompt_utf.c_str());
       }
    else
       {
         line = readline_lib::readline(0);
       }
-#else
-char * line = readline_lib::no_readline(prompt);
-#endif
+
+   if (end_input)   (*end_input)();
 
 const APL_time to = now();
-   Workspace::the_workspace->add_wait(to - from);
+   Workspace::add_wait(to - from);
 
 UTF8 * l = (UTF8 *)line;
    if (l)
       {
         while (*l && *l <= ' ')   ++l;   // skip leading whitespace
 
-        if (*l)   readline_lib::add_history(line);
+        if (use_readline && *l)   readline_lib::add_history(line);
       }
 
    return l;
 }
 //-----------------------------------------------------------------------------
-const unsigned char *
-Input::get_user_line_1(const UCS_string * prompt)
+const char *
+Input::get_user_line_nabla(const UCS_string * prompt)
 {
 const UTF8 * line = TestFiles::get_testcase_line();
-   if (line)   return line;
+   if (line)   return (const char *)line;
 
    if (input_file_FILE)   // a -f file is open
       {
         line = get_f_line(input_file_FILE);
-        if (line)   return line;
+        if (line)   return (const char *)line;
       }
 
    if (input_file_name)   // -f present, but not opened yet
@@ -283,7 +307,7 @@ const UTF8 * line = TestFiles::get_testcase_line();
         if (input_file_FILE)
            {
              line = get_f_line(input_file_FILE);
-             if (line)   return line;
+             if (line)   return (const char *)line;
            }
         else
            {
@@ -295,21 +319,32 @@ const UTF8 * line = TestFiles::get_testcase_line();
 
    // all reads from files failed.
    //
-#if HAVE_LIBREADLINE
-   if (prompt)
+   return (const char *)get_user_line(prompt, true);
+
+
+const APL_time from = now();
+   if (!use_readline)
       {
-        UTF8_string prompt_utf(*prompt);
-        loop(p, prompt_utf.size())
-           readline_lib::rl_stuff_char(prompt_utf[p] & 0x00FF);
+        line = (UTF8 *)no_readline(prompt);
       }
-   line = (UTF8 *)readline_lib::readline(0);
-#else // ! HAVE_LIBREADLINE
-   line = (UTF8 *)readline_lib::no_readline(prompt);
-#endif // HAVE_LIBREADLINE
+   else
+     {
+       if (prompt)
+          {
+            UTF8_string prompt_utf(*prompt);
+            loop(p, prompt_utf.size())
+            readline_lib::rl_stuff_char(prompt_utf[p] & 0x00FF);
+          }
+
+       line = (UTF8 *)readline_lib::readline(0);
+     }
 
    while (*line && *line <= ' ')   ++line;
 
-   return line;
+const APL_time to = now();
+   Workspace::add_wait(to - from);
+
+   return (const char *)line;
 }
 //-----------------------------------------------------------------------------
 UCS_string
@@ -330,7 +365,7 @@ Input::get_quad_cr_line(UCS_string ucs_prompt)
                 }
 
              UCS_string ret(ucs_prompt);
-             ret.app(line);
+             ret.append_utf8(line);
              return ret;
            }
 
@@ -339,16 +374,19 @@ Input::get_quad_cr_line(UCS_string ucs_prompt)
 
 UTF8_string utf_prompt(ucs_prompt);
 
-#if HAVE_LIBREADLINE
-   loop(s, utf_prompt.size())
-      readline_lib::rl_stuff_char(utf_prompt[s] & 0x00FF);
+const char * line;
+   if (!use_readline)
+      {
+        line = no_readline(&ucs_prompt);
+      }
+   else
+      {
+        loop(s, utf_prompt.size())
+            readline_lib::rl_stuff_char(utf_prompt[s] & 0x00FF);
 
-   cout << '\r';
-const char * line = readline_lib::readline(0);
-
-#else
-const char * line = readline_lib::no_readline(&ucs_prompt);
-#endif
+        cout << '\r';
+        line = readline_lib::readline(0);
+      }
 
 const UTF8_string uline(line);
    return UCS_string(uline);
