@@ -18,10 +18,9 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "../config.h"   // for HAVE_LIBLAPACK
 #include <iostream>
 
-#include "APL_types.hh"
+#include "Common.hh"
 #include "ComplexCell.hh"
 #include "FloatCell.hh"
 #include "Output.hh"
@@ -167,7 +166,7 @@ divide_matrix(ShapeItem rows, ShapeItem cols_A, Value_P A, ShapeItem cols_B,
 {
 const bool need_complex = A->is_complex(qct) || B->is_complex(qct);
 
-Value_P Z = new Value(shape_Z, LOC);
+Value_P Z(new Value(shape_Z, LOC));
 
    loop(c, cols_A)
        {
@@ -192,11 +191,10 @@ Value_P Z = new Value(shape_Z, LOC);
               const int result = complex_matrix_divide(rows, cols_B, b, a);
               if (result)
                  {
-                   Z->erase(LOC);
                    DOMAIN_ERROR;
                  }
 
-              loop(r, shape_Z.get_shape_item(0))
+              loop(r, shape_Z.get_cols())
                   new (&Z->get_ravel(r*cols_A + c)) ComplexCell(a[r].r, a[r].i);
             }
          else   // real
@@ -218,11 +216,10 @@ Value_P Z = new Value(shape_Z, LOC);
               const int result = real_matrix_divide(rows, cols_B, b, a);
               if (result)
                  {
-                   Z->erase(LOC);
                    DOMAIN_ERROR;
                  }
 
-              loop(r, shape_Z.get_shape_item(0))
+              loop(r, shape_Z.get_cols())
                   new (&Z->get_ravel(r*cols_A + c)) FloatCell(a[r]);
             }
        }
@@ -361,7 +358,7 @@ divide_matrix(ShapeItem rows, ShapeItem cols_A, Value_P A, ShapeItem cols_B,
 {
 const bool need_complex = A->is_complex(qct) || B->is_complex(qct);
 
-Value_P Z = new Value(shape_Z, LOC);
+Value_P Z(new Value(shape_Z, LOC));
 
    if (need_complex)
       {
