@@ -25,6 +25,7 @@
 
 #include "Error.hh"
 #include "LibPaths.hh"
+#include "PrintOperator.hh"
 
 #ifndef PATH_MAX
 #define PATH_MAX 4096
@@ -200,11 +201,11 @@ LibPaths::get_lib_dir(LibRef libref)
 UCS_string ret(APL_lib_root);
    if (libref == LIB0)
       {
-        ret.app("/workspaces");
+        ret.append_utf8("/workspaces");
       }
    else
       {
-        ret.app("/wslib");
+        ret.append_utf8("/wslib");
         ret.append_number(libref);
       }
 
@@ -215,8 +216,16 @@ UTF8_string
 LibPaths::get_lib_filename(LibRef lib, const UCS_string & name, 
                            bool existing, const char * extension)
 {
+   if (name[0] == UNI_ASCII_SLASH)   // absolute path
+      {
+         // absolute paths are fallbacks and never modified
+         // but taken as they are
+         //
+         return UTF8_string(name);
+      }
+
 UCS_string ret = get_lib_dir(lib);
-   ret.app("/");
+   ret.append_utf8("/");
    ret.append(name);
 
 bool has_extension = false;
@@ -227,7 +236,7 @@ bool has_extension = false;
         // If it does return filename otherwise filename.extension.
         //
         UTF8_string filename(ret);
-        has_extension = access((const char *)filename.c_str(), F_OK) == 0;
+        has_extension = access(filename.c_str(), F_OK) == 0;
       }
    else
       {
@@ -249,8 +258,8 @@ bool has_extension = false;
 
    if (!has_extension)
       {
-        ret.app(".");
-        ret.app(extension);
+        ret.append_utf8(".");
+        ret.append_utf8(extension);
       }
 
    return UTF8_string(ret);
