@@ -79,7 +79,7 @@ Nabla::edit()
    while (!do_close)
        {
          const UCS_string prompt = current_line.print_prompt();
-         const char * line = (const char *)Input::get_user_line_1(&prompt);
+         const char * line = Input::get_user_line_nabla(&prompt);
          const UTF8_string utf(line);
          const UCS_string ucs(utf);
          if (const char * loc = parse_oper(ucs, false))
@@ -98,13 +98,14 @@ Nabla::edit()
 UCS_string fun_text;
    loop(l, lines.size())
       {
-        fun_text += lines[l].text;
-        fun_text += UNI_ASCII_LF;
+        fun_text.append(lines[l].text);
+        fun_text.append(UNI_ASCII_LF);
       }
 
 int error_line = 0;
-UserFunction * ufun = UserFunction::fix(fun_text, error_line, LOC);
+UserFunction * ufun = UserFunction::fix(fun_text, error_line, false, LOC);
    if (ufun == 0)   NABLA_ERROR;
+
    if (locked)
       {
          const int exec_properties[4] = { 1, 1, 1, 1 };
@@ -134,7 +135,7 @@ UCS_string::iterator c(first_command.begin());
 
    // function name or header.
    //
-   while (c.more() && c.get() != UNI_ASCII_L_BRACK)   fun_name += c.next();
+   while (c.more() && c.get() != UNI_ASCII_L_BRACK)   fun_name.append(c.next());
 
    // at this point there could be an axis specification [X] that
    // could be confused with an operation like [⎕]. We take the first char
@@ -147,9 +148,9 @@ UCS_string::iterator c(first_command.begin());
               if (c.get(off) <= UNI_ASCII_SPACE)   continue; 
               if (Avec::is_first_symbol_char(c.get(off)))   // axis
                  {
-                   fun_name += c.next();   //  copy the [
+                   fun_name.append(c.next());   //  copy the [
                    while (c.more() && c.get() != UNI_ASCII_L_BRACK)
-                         fun_name += c.next();
+                         fun_name.append(c.next());
                  }
               break;
             }
@@ -163,7 +164,7 @@ UCS_string::iterator c(first_command.begin());
       {
         UCS_string oper;
         Unicode cc;
-        do oper += cc = c.next(); while (cc != UNI_ASCII_R_BRACK);
+        do oper.append(cc = c.next());   while (cc != UNI_ASCII_R_BRACK);
         if (const char * loc = parse_oper(oper, true))   return loc;   // error
       }
 
@@ -225,7 +226,7 @@ Unicode cc = c.next();
       {
         ecmd = ECMD_EDIT;
         edit_from = current_line;
-        for (; cc != Invalid_Unicode; cc = c.next())   current_text += cc;
+        for (; cc != Invalid_Unicode; cc = c.next())   current_text.append(cc);
         return 0;
       }
 
@@ -255,7 +256,8 @@ command_loop:
    // [→   abandon
    switch (c.get())
       {
-        case UNI_QUAD_QUAD:     ecmd = ECMD_SHOW;     c.next();   break;
+        case UNI_QUAD_QUAD:
+        case UNI_QUAD_QUAD1:    ecmd = ECMD_SHOW;     c.next();   break;
         case UNI_ASCII_R_BRACK: ecmd = ECMD_EDIT;                 break;
         case UNI_DELTA:         ecmd = ECMD_DELETE;   c.next();   break;
         case UNI_RIGHT_ARROW:   ecmd = ECMD_ESCAPE;   c.next();   break;
@@ -302,27 +304,77 @@ again:
          goto command_loop;
       }
 
-   // copy the rest to current_text and set do_close if the first character
-   // is ∇ or ⍫
+   // copy the rest to current_text. Set do_close if ∇ or ⍫ is seen
+   // unless inside strings.
    //
-   while ((cc = c.next()) != Invalid_Unicode)   current_text += cc;
-
-   if (current_text.size())
+   for (;;)
       {
-        if (current_text[0] == UNI_NABLA)
+        switch(cc = c.next())
            {
-             current_text.clear();
-             do_close = true;
-           }
-        else if (current_text[0] == UNI_DEL_TILDE)
-           {
-             current_text.clear();
-             locked = true;
-             do_close = true;
+             case Invalid_Unicode:     // regular end of input
+                  return 0;
+
+             case UNI_NABLA:           // ∇
+                  do_close = true;
+                  return 0;
+
+             case UNI_DEL_TILDE:       // ⍫
+                  locked = true;
+                  do_close = true;
+                  return 0;
+
+             case UNI_ASCII_DOUBLE_QUOTE:  // "
+                  current_text.append(cc);
+                  for (;;)
+                      {
+                        cc = c.next();
+                        if (cc == Invalid_Unicode)   // premature end of input
+                           {
+                             current_text.append(UNI_ASCII_DOUBLE_QUOTE);
+                             return 0;
+                           }
+
+                        current_text.append(cc);
+                        if (cc == UNI_ASCII_DOUBLE_QUOTE)   break; // string end
+                        if (cc == UNI_ASCII_BACKSLASH)      // \x
+                           {
+                             cc = c.next();
+                             if (cc == Invalid_Unicode)   // premature input end
+                                {
+                                  current_text.append(UNI_ASCII_BACKSLASH);
+                                  current_text.append(UNI_ASCII_DOUBLE_QUOTE);
+                                  return 0;
+                                }
+                             current_text.append(cc);
+                           }
+                      }
+                  break;
+
+             case UNI_SINGLE_QUOTE:    // '
+                  current_text.append(cc);
+                  for (;;)
+                      {
+                        // no need to care for ''. Since we only copy, we can
+                        // handle ' ... '' ... ' like two adjacent strings
+                        // instead of a string containing a (doubled) quote.
+                        //
+                        cc = c.next();
+                        if (cc == Invalid_Unicode)   // premature end of input
+                           {
+                             current_text.append(UNI_SINGLE_QUOTE);
+                             return 0;
+                           }
+
+                        current_text.append(cc);
+                        if (cc == UNI_SINGLE_QUOTE)      break;   // string end
+                      }
+                  break;
+
+             default:
+                current_text.append(cc);
            }
       }
 
-
    return 0;
 }
 //-----------------------------------------------------------------------------
@@ -340,7 +392,7 @@ LineLabel ret(0);
    if (c.get() == UNI_ASCII_FULLSTOP)
       {
         c.next();   // eat the .
-        while (Avec::is_digit(c.get()))   ret.ln_minor += c.next();
+        while (Avec::is_digit(c.get()))   ret.ln_minor.append(c.next());
       }
 
    return ret;
@@ -373,19 +425,19 @@ Nabla::open_function()
    // all chars are symbol chars.
    // They may or may not relate to an existing symbol.
    //
-Symbol * fsym = Workspace::the_workspace->symbol_table
-                                         .lookup_existing_symbol(fun_name);
-
+Symbol * fsym = Workspace::lookup_existing_symbol(fun_name);
    if (fsym == 0)   return open_new_function();
 
    // existing symbol: check that function is not on the SI stack.
    //
-   if (Workspace::the_workspace->is_called(fun_name))
+   if (Workspace::is_called(fun_name))
       {
-        UCS_string & t4 = Workspace::the_workspace->more_error;
+        UCS_string & t4 = Workspace::more_error();
         t4.clear();
-        t4.app("function ");   t4 += fun_name;
-        t4.app(" could not be edited, since it is used on the )SI stack.");
+        t4.append_utf8("function ");
+        t4.append(fun_name);
+        t4.append_utf8(" could not be edited, since "
+                       "it is used on the )SI stack.");
         return LOC;
       }
 
@@ -434,7 +486,7 @@ Nabla::open_new_function()
       CERR << "creating new function '" << fun_name << "'" << endl;
 
 int error_line = 0;
-UserFunction * ufun = UserFunction::fix(fun_name, error_line, LOC);
+UserFunction * ufun = UserFunction::fix(fun_name, error_line, true, LOC);
 
    if (ufun == 0)   return "Bad function header at " LOC;
    fun_name = ufun->get_name();
@@ -604,12 +656,12 @@ UCS_string ret("[");
 
    if (ln_minor.size())
       {
-        ret += Unicode('.');
-        loop(s, ln_minor.size())   ret += Unicode(char(ln_minor[s]));
+        ret.append(Unicode('.'));
+        loop(s, ln_minor.size())   ret.append(Unicode(char(ln_minor[s])));
       }
 
-   ret += Unicode(']');
-   ret += Unicode(' ');
+   ret.append(Unicode(']'));
+   ret.append(Unicode(' '));
    return ret;
 }
 //-----------------------------------------------------------------------------
@@ -626,7 +678,7 @@ LineLabel::next()
    //
 const Unicode cc = ln_minor[ln_minor.size() - 1];
    if (cc != UNI_ASCII_9)   ln_minor[ln_minor.size() - 1] = Unicode(cc + 1);
-   else                     ln_minor += UNI_ASCII_1;
+   else                     ln_minor.append(UNI_ASCII_1);
 }
 //-----------------------------------------------------------------------------
 bool
