@@ -22,6 +22,7 @@
 
 #include "CharCell.hh"
 #include "ComplexCell.hh"
+#include "Common.hh"
 #include "FloatCell.hh"
 #include "IntCell.hh"
 #include "Output.hh"
@@ -42,6 +43,7 @@ Parser::parse(const UCS_string & input, Token_string & tos)
    // convert input characters into token
    //
 Token_string tos1;
+
    {
      Tokenizer tokenizer(pmode, LOC);
      ErrorCode ec = tokenizer.tokenize(input, tos1);
@@ -57,21 +59,21 @@ Token_string tos1;
 
    // split tos1 into statements.
    //
-vector<Token_string> statements;
+vector<Token_string *> statements;
    {
      Source<Token> src(tos1);
-     Token_string stat;
+     Token_string  * stat = new Token_string();
      while (src.rest())
         {
           const Token tok = src.get();
           if (tok.get_tag() == TOK_DIAMOND)
              {
                statements.push_back(stat);
-               stat = Token_string();
+               stat = new Token_string();
              }
           else
              {
-               stat += tok;
+               stat->append(tok, LOC);
              }
         }
      statements.push_back(stat);
@@ -79,13 +81,18 @@ vector<Token_string> statements;
 
    loop(s, statements.size())
       {
-        Token_string & stat = statements[s];
-        ErrorCode err = parse_statement(stat);
+        Token_string * stat = statements[s];
+        ErrorCode err = parse_statement(*stat);
         if (err)   return err;
 
-        if (s)   tos += Token(TOK_DIAMOND);
+        if (s)   tos.append(Token(TOK_DIAMOND));
 
-        loop(t, stat.size())   tos += stat[t];
+        loop(t, stat->size())
+           {
+             tos.append(Token());
+             move_1(tos[tos.size() - 1], (*stat)[t], LOC);
+           }
+        delete stat;
       }
 
    return E_NO_ERROR;
@@ -153,6 +160,8 @@ Parser::collect_constants(Token_string & tos)
         print_token_list(CERR, tos, 0);
       }
 
+   // convert several items in vector notation into a single APL value
+   //
    loop (t, tos.size())
       {
         int32_t to;
@@ -162,21 +171,47 @@ Parser::collect_constants(Token_string & tos)
              case TOK_INTEGER:
              case TOK_REAL:
              case TOK_COMPLEX:
-             case TOK_APL_VALUE:
              case TOK_APL_VALUE1:
-                  for (to = t + 1; to < tos.size(); ++to)
-                      {
-                        if (tos[to].get_tag() == TOK_CHARACTER)    continue;
-                        if (tos[to].get_tag() == TOK_INTEGER)      continue;
-                        if (tos[to].get_tag() == TOK_REAL)         continue;
-                        if (tos[to].get_tag() == TOK_COMPLEX)      continue;
-                        if (tos[to].get_tag() == TOK_APL_VALUE)    continue;
-                        if (tos[to].get_tag() == TOK_APL_VALUE1)   continue;
-                        break;
-                      }
-                  create_value(tos, t, to - t);
-                  break;
+             case TOK_APL_VALUE3:
+                  break;          // continue below
+
+             default: continue;   // nex token
            }
+
+        // at this point, t is the first item. Collect subsequenct items.
+        //
+        for (to = t + 1; to < tos.size(); ++to)
+            {
+#if 1
+              // Note: ISO 13751 gives an example with 1 2 3[2] ↔ 2
+              // 
+              // In contrast, the binding rules stated in IBM's apl2lrm.pdf
+              // would give 3[2] ↔ RANK ERROR.
+              // 
+              // We follow apl2lrm; to enable IBM behavior write #if 0 above.
+              // 
+
+              // if tos[to + 1] is [ then [ binds stronger than
+              // vector notation and we stop collecting.
+              //
+              if ((to + 1) < tos.size() && tos[to + 1].get_tag() == TOK_L_BRACK)
+                 break;
+#endif
+              const TokenTag tag = tos[to].get_tag();
+
+              if (tag == TOK_CHARACTER)    continue;
+              if (tag == TOK_INTEGER)      continue;
+              if (tag == TOK_REAL)         continue;
+              if (tag == TOK_COMPLEX)      continue;
+              if (tag == TOK_APL_VALUE1)   continue;
+              if (tag == TOK_APL_VALUE3)   continue;
+
+              // no more values: stop collecting
+              //
+              break;
+            }
+
+        create_value(tos, t, to - t);
       }
 
    Log(LOG_collect_constants)
@@ -204,8 +239,8 @@ int opening = -1;
               case TOK_INTEGER:
               case TOK_REAL:
               case TOK_COMPLEX:
-              case TOK_APL_VALUE:
               case TOK_APL_VALUE1:
+              case TOK_APL_VALUE3:
                    continue;
 
               case TOK_L_PARENT:
@@ -218,14 +253,14 @@ int opening = -1;
               case TOK_R_PARENT:
                    if (opening == -1)   continue;   // ')' without ')'
 
-                   tos[opening] = Token();   // invalidate '('
-                   tos[t] = Token();         // invalidate ')'
+                   tos[opening].clear(LOC);  // invalidate '('
+                   tos[t].clear(LOC);        // invalidate ')'
                    create_value(tos, opening + 1, t - opening - 1);
 
                    // we removed parantheses, so we close the value.
                    //
-                   Assert(tos[opening + 1].get_tag() == TOK_APL_VALUE);
-                   tos[opening + 1].ChangeTag(TOK_APL_VALUE1);
+                   if (tos[opening + 1].get_tag() == TOK_APL_VALUE3)
+                      tos[opening + 1].ChangeTag(TOK_APL_VALUE1);
 
                    Log(LOG_collect_constants)
                       {
@@ -366,8 +401,8 @@ Parser::remove_nongrouping_parantheses(Token_string & tos)
                          // ((...)) are not "not separating"
                          //
                          progress = true;
-                         tos[t] = Token();
-                         tos[closing] = Token();
+                         tos[t].clear(LOC);
+                         tos[closing].clear(LOC);
                          continue;
                        }
                   }
@@ -383,9 +418,9 @@ Parser::remove_nongrouping_parantheses(Token_string & tos)
                // If X is non-skalar, enclose it
                //
                progress = true;
-               tos[t + 2] = tos[t + 1];
-               tos[t + 1] = Token();
-               tos[t]     = Token();
+               move_1(tos[t + 2], tos[t + 1], LOC);
+               tos[t + 1].clear(LOC);
+               tos[t].clear(LOC);
                ++t;   // skip tos[t + 1]
              }
        }
@@ -399,7 +434,7 @@ uint32_t dst = 0;
    loop(src, tos.size())
        {
          if (tos[src].get_tag() == TOK_VOID)   continue;
-         if (src != dst)   tos[dst] = tos[src];
+         if (src != dst)   move_1(tos[dst], tos[src], LOC);
          ++dst;
        }
 
@@ -430,46 +465,57 @@ Parser::create_value(Token_string & tos, uint32_t pos, uint32_t count)
 void
 Parser::create_skalar_value(Token & output)
 {
-Value_P skalar = 0;
    switch(output.get_tag())
-          {
-            case TOK_CHARACTER:
-                 skalar = new Value(LOC);
+      {
+        case TOK_CHARACTER:
+             {
+               Value_P skalar(new Value(LOC));
 
-                 new (&skalar->get_ravel(0))
-                     CharCell(output.get_char_val());
-                 output = Token(TOK_APL_VALUE, skalar);
-                 return;
+               new (&skalar->get_ravel(0))  CharCell(output.get_char_val());
+               skalar->check_value(LOC);
+               Token tok(TOK_APL_VALUE3, skalar);
+               move_1(output, tok, LOC);
+             }
+             return;
 
-            case TOK_INTEGER:
-                 skalar = new Value(create_loc);
+        case TOK_INTEGER:
+             {
+               Value_P skalar(new Value(LOC));
 
-                 new (&skalar->get_ravel(0))
-                     IntCell(output.get_int_val());
-                 output = Token(TOK_APL_VALUE, skalar);
-                 return;
+               new (&skalar->get_ravel(0))   IntCell(output.get_int_val());
+               skalar->check_value(LOC);
+               Token tok(TOK_APL_VALUE3, skalar);
+               move_1(output, tok, LOC);
+             }
+             return;
 
-            case TOK_REAL:
-                 skalar = new Value(LOC);
+        case TOK_REAL:
+             {
+               Value_P skalar(new Value(LOC));
 
-                 new (&skalar->get_ravel(0))
-                     FloatCell(output.get_flt_val());
-                 output = Token(TOK_APL_VALUE, skalar);
-                 return;
+               new (&skalar->get_ravel(0))  FloatCell(output.get_flt_val());
+               skalar->check_value(LOC);
+               Token tok(TOK_APL_VALUE3, skalar);
+               move_1(output, tok, LOC);
+             }
+             return;
 
-            case TOK_COMPLEX:
-                 skalar = new Value(LOC);
+        case TOK_COMPLEX:
+             {
+               Value_P skalar(new Value(LOC));
 
-                 new (&skalar->get_ravel(0))
-                     ComplexCell(output.get_cpx_real(),
-                                 output.get_cpx_imag());
-                 output = Token(TOK_APL_VALUE, skalar);
-                 return;
+               new (&skalar->get_ravel(0))   ComplexCell(output.get_cpx_real(),
+                                                         output.get_cpx_imag());
+               skalar->check_value(LOC);
+               Token tok(TOK_APL_VALUE3, skalar);
+               move_1(output, tok, LOC);
+             }
+             return;
 
-            case TOK_APL_VALUE:
-            case TOK_APL_VALUE1:
-                 return;
-          }
+        case TOK_APL_VALUE1:
+        case TOK_APL_VALUE3:
+             return;
+      }
 
    CERR << "Unexpected token " << output.get_tag() << ": " << output << endl;
    Assert(0 && "Unexpected token");
@@ -478,7 +524,7 @@ Value_P skalar = 0;
 void
 Parser::create_vector_value(Token_string & tos, uint32_t pos, uint32_t count)
 {
-Value_P vector = new Value(count, LOC);
+Value_P vector(new Value(count, LOC));
 
    loop(l, count)
        {
@@ -489,34 +535,37 @@ Value_P vector = new Value(count, LOC);
             {
               case TOK_CHARACTER:
                    new (addr) CharCell(tok.get_char_val());
-                   tok = Token();   // invalidate token
+                   tok.clear(LOC);   // invalidate token
                    break;
 
               case TOK_INTEGER:
                    new (addr) IntCell(tok.get_int_val());
-                   tok = Token();   // invalidate token
+                   tok.clear(LOC);   // invalidate token
                    break;
 
               case TOK_REAL:
                    new (addr) FloatCell(tok.get_flt_val());
-                   tok = Token();   // invalidate token
+                   tok.clear(LOC);   // invalidate token
                    break;
 
               case TOK_COMPLEX:
                    new (addr) ComplexCell(tok.get_cpx_real(),
                                           tok.get_cpx_imag());
-                   tok = Token();   // invalidate token
+                   tok.clear(LOC);   // invalidate token
                    break;
 
-            case TOK_APL_VALUE:
             case TOK_APL_VALUE1:
+            case TOK_APL_VALUE3:
                  new (addr) PointerCell(tok.get_apl_val());
-                 tok = Token();   // invalidate token
+                 tok.clear(LOC);   // invalidate token
                  break;
             }
        }
 
-   tos[pos] = Token(TOK_APL_VALUE, vector);
+   vector->check_value(LOC);
+Token tok(TOK_APL_VALUE3, vector);
+
+   move_1(tos[pos], tok, LOC);
 
    Log(LOG_create_value)
       {
