@@ -27,8 +27,7 @@
 //-----------------------------------------------------------------------------
 PointerCell::PointerCell(Value_P val)
 {
-   value.valp = val;
-   val->set_nested();
+   new (&value._valp()) Value_P(val);
 }
 //-----------------------------------------------------------------------------
 bool
@@ -58,16 +57,15 @@ PointerCell::greater(const Cell * other, bool ascending) const
 void
 PointerCell::release(const char * loc)
 {
-   value.valp->clear_nested();
-   value.valp->erase(loc);
-   value.valp = 0;
+   ptr_clear(value._valp(), loc);
    new (this) Cell;
 }
 //-----------------------------------------------------------------------------
 Value_P
 PointerCell::get_pointer_value() const
 {
-   return value.valp;
+Value_P ret = value._valp();
+   return ret;
 }
 //-----------------------------------------------------------------------------
 CellType
@@ -89,27 +87,27 @@ Value_P val = get_pointer_value();
 
         if (val->is_char_vector())
            {
-             ucs += UNI_SINGLE_QUOTE;
+             ucs.append(UNI_SINGLE_QUOTE);
              loop(e, ec)
                 {
                   const Unicode uni = val->get_ravel(e).get_char_value();
-                  ucs += uni;
-                  if (uni == UNI_SINGLE_QUOTE)   ucs += uni;   // ' -> ''
+                  ucs.append(uni);
+                  if (uni == UNI_SINGLE_QUOTE)   ucs.append(uni);   // ' -> ''
                 }
-             ucs += UNI_SINGLE_QUOTE;
+             ucs.append(UNI_SINGLE_QUOTE);
            }
         else
            {
-             ucs += UNI_ASCII_L_PARENT;
+             ucs.append(UNI_ASCII_L_PARENT);
              loop(e, ec)
                 {
                   PrintBuffer pb = val->get_ravel(e).
                         character_representation(pctx);
-                  ucs += UCS_string(pb, 0);
+                  ucs.append(UCS_string(pb, 0));
 
-                  if (e < ec - 1)   ucs += UNI_ASCII_SPACE;
+                  if (e < ec - 1)   ucs.append(UNI_ASCII_SPACE);
                 }
-             ucs += UNI_ASCII_R_PARENT;
+             ucs.append(UNI_ASCII_R_PARENT);
            }
 
         ColInfo ci;
@@ -120,7 +118,7 @@ PrintBuffer ret(*val, pctx);
    ret.get_info().flags &= ~CT_MASK;
    ret.get_info().flags |= CT_POINTER;
 
-   if ((pctx.get_style() & PST_CS_INNER) != PST_CS_NONE)
+   if (pctx.get_style() & PST_CS_INNER)
       {
         const ShapeItem ec = val->element_count();
         if (ec == 0)   // empty value
@@ -135,11 +133,39 @@ PrintBuffer ret(*val, pctx);
                      }
                 }
 
+             // do A←(1⌈⍴A)⍴⊂↑A
              Value_P proto = val->prototype(LOC);
-             ret = PrintBuffer(*proto, pctx);
-             ret.add_frame(PrintStyle(style), proto->get_rank(),
-                           proto->compute_depth());
-             proto->erase(LOC);
+
+             // compute a shape like ⍴ val but 0 replaced with 1.
+             //
+             Shape sh(val->get_shape());
+             loop(r, sh.get_rank())
+                 {
+                   if (sh.get_shape_item(r) == 0)   sh.set_shape_item(r, 1);
+                 }
+
+             if (sh.get_volume() == 1)   // one prototype
+                {
+                  ret = PrintBuffer(*proto, pctx);
+                  ret.add_frame(PrintStyle(style), proto->get_rank(),
+                                proto->compute_depth());
+                }
+             else                           // several prototypes
+                {
+                  Value_P proto_reshaped(new Value(sh, LOC));
+                  Cell * c = &proto_reshaped->get_ravel(0);
+                  const ShapeItem len = proto_reshaped->element_count();
+
+                  // store proto in the first ravel item, and copies of proto in
+                  // the subsequent ravel items
+                  //
+                  new (c++) PointerCell(proto);
+                  loop(rv, len - 1)   new (c++) PointerCell(proto->clone(LOC));
+
+                  ret = PrintBuffer(*proto_reshaped, pctx);
+                  ret.add_frame(PrintStyle(style), proto_reshaped->get_rank(),
+                                proto_reshaped->compute_depth());
+                }
            }
        else
            {
