@@ -29,6 +29,7 @@
 #include "StateIndicator.hh"
 #include "Symbol.hh"
 #include "UserFunction.hh"
+#include "ValueHistory.hh"
 #include "Workspace.hh"
 
 //-----------------------------------------------------------------------------
@@ -38,11 +39,71 @@ Prefix::Prefix(StateIndicator & _si, const Token_string & _body)
      saved_lookahead(Token(TOK_VOID), Function_PC_invalid),
      body(_body),
      PC(Function_PC_0),
-     assign_pending(false),
+     assign_state(ASS_none),
      lookahead_high(Function_PC_invalid)
 {
 }
 //-----------------------------------------------------------------------------
+void
+Prefix::cleanup()
+{
+
+   //
+   // there is still an unresolved issue with double deletion of values
+   // by this function and Prefix.cc:948 below (line->erase(LOC); in
+   // function reduce_END_GOTO_B_(). Until resolved we do not erase here!
+   //
+   return;
+
+   if (size() == 0)   return;
+
+CERR << "Warning: discarding non-empty stack["
+     << si.get_level() << "]" << " with " << size() << " items " << endl;
+
+   while (size())
+      {
+        Token_loc tl = pop();
+        if (tl.tok.get_Class() == TC_VALUE)
+           {
+             Value_P val = tl.tok.get_apl_val();
+             if (!val)   continue;
+
+CERR << "    Value is " << (const void *)val.get() << " " << *val << " at " LOC;
+// val->print_properties(CERR,  0);
+           }
+      }
+}
+//-----------------------------------------------------------------------------
+void
+Prefix::syntax_error(const char * loc)
+{
+   // move the PC back to the beginning of the failed statement
+   //
+   while (PC > 0)
+      {
+        --PC;
+        if (body[PC].get_Class() == TC_END)
+           {
+             ++PC;
+             break;
+           }
+      }
+
+   // clear values in FIFO
+   //
+   loop (s, size())
+      {
+        Token & tok = at(s).tok;
+        if (tok.get_Class() == TC_VALUE)
+           {
+             Value_P val = tok.get_apl_val();
+          }
+      }
+
+   throw_apl_error(get_assign_state() == ASS_none ? E_SYNTAX_ERROR
+                                                  : E_LEFT_SYNTAX_ERROR, loc);
+}
+//-----------------------------------------------------------------------------
 bool
 Prefix::is_value_bracket() const
 {
@@ -67,7 +128,8 @@ int br_count = 1;   // the bracket at PC
                    if (tok1.get_Class() != TC_SYMBOL)   return false;
 
                    Symbol * sym = tok1.get_sym_ptr();
-                   return sym->resolve_class(assign_pending) == TC_VALUE;
+                   const bool left_sym = get_assign_state() == ASS_arrow_seen;
+                   return sym->resolve_class(left_sym) == TC_VALUE;
                  }
             }
        }
@@ -99,13 +161,18 @@ const int si_depth = si.get_level();
    out << "fifo[si=" << si_depth << " len=" << size()
        << " PC=" << PC << "] is now :";
 
-   loop(s, size())   out << " " << Token::class_name(at(s).tok.get_Class());
+   loop(s, size())
+      {
+        const TokenClass tc = at(s).tok.get_Class();
+        out << " " << Token::class_name(tc);
+      }
+
    out << "  at " << loc << endl;
 }
 //-----------------------------------------------------------------------------
 int
 Prefix::show_owners(const char * prefix, ostream & out,
-                          Value_P value) const
+                          const Value & value) const
 {
 int count = 0;
 
@@ -114,7 +181,7 @@ int count = 0;
         const Token & tok = at(s).tok;
         if (tok.get_ValueType() != TV_VAL)      continue;
 
-        if (tok.get_apl_val() == value)
+        if (tok.get_apl_val().get() == &value)
            {
              out << prefix << " Fifo [" << s << "]" << endl;
              ++count;
@@ -125,22 +192,6 @@ int count = 0;
    return count;
 }
 //-----------------------------------------------------------------------------
-void
-Prefix::cleanup(const char * loc)
-{
-// Backtrace::show(__FILE__, __LINE__);
-   while (size())
-      {
-        Token_loc tl = pop();
-        if (tl.tok.get_Class() == TC_VALUE)
-           {
-// Q(tl.tok.get_apl_val())
-// Q(*tl.tok.get_apl_val())
-//             tl.tok.get_apl_val()->erase(LOC);
-           }
-      }
-}
-//-----------------------------------------------------------------------------
 Function_PC
 Prefix::get_range_high() const
 {
@@ -188,7 +239,7 @@ Prefix::unmark_all_values() const
         if (tok.get_ValueType() != TV_VAL)      continue;
 
         Value_P value = tok.get_apl_val();
-        if (value)   value->clear_marked();
+        if (!!value)   value->clear_marked();
       }
 }
 //-----------------------------------------------------------------------------
@@ -216,7 +267,7 @@ grow:
           // there is a MISC token from a MISC phrase. Use it.
           //
           push(saved_lookahead);
-          saved_lookahead.tok = Token(TOK_VOID);
+          saved_lookahead.tok.clear(LOC);
           goto again;   // success
         }
 
@@ -232,14 +283,17 @@ grow:
      Log(LOG_prefix_parser)
         {
           CERR << "    [si=" << si.get_level() << " PC=" << (PC - 1)
-               << "] Read token[" << size() << "] " << tl.tok << " "
+               << "] Read token[" << size();
+          if (get_assign_state() == ASS_arrow_seen)      CERR << " ← ";
+          else if (get_assign_state() == ASS_var_seen)   CERR << " V← ";
+          CERR << "] " << tl.tok << " "
                << Token::class_name(tl.tok.get_Class()) << endl;
         }
 
      lookahead_high = tl.pc;
      TokenClass tcl = tl.tok.get_Class();
 
-     if (tcl == TC_SYMBOL)   // resolve symbol if neccessary
+     if (tcl == TC_SYMBOL)   // resolve symbol if necessary
         {
           Symbol * sym = tl.tok.get_sym_ptr();
           if (tl.tok.get_tag() == TOK_LSYMB2)
@@ -255,7 +309,9 @@ grow:
              }
           else
              {
-               sym->resolve(tl.tok, assign_pending);
+               const bool left_sym = get_assign_state() == ASS_arrow_seen;
+               sym->resolve(tl.tok, left_sym);
+               if (left_sym)   set_assign_state(ASS_var_seen);
              }
 
           if (tl.tok.get_tag() == TOK_SI_PUSHED)
@@ -266,10 +322,10 @@ grow:
               return Token(TOK_SI_PUSHED);
             }
         }
-     else if (tcl == TC_ASSIGN)   // resolve symbol if neccessary
+     else if (tcl == TC_ASSIGN)   // resolve symbol if necessary
         {
-          if (assign_pending)   throw_apl_error(E_LEFT_SYNTAX_ERROR, LOC);
-          assign_pending = true;
+          if (get_assign_state() != ASS_none)   syntax_error(LOC);
+          set_assign_state(ASS_arrow_seen);
         }
 
      push(tl);
@@ -332,7 +388,8 @@ found_prefix:
           if (tok.get_Class() == TC_SYMBOL)
              {
                Symbol * sym = tok.get_sym_ptr();
-               next = sym->resolve_class(assign_pending);
+               const bool left_sym = get_assign_state() == ASS_arrow_seen;
+               next = sym->resolve_class(left_sym);
             }
         }
 
@@ -407,7 +464,7 @@ found_prefix:
    if (best->misc)   // MISC phrase: save X and remove it
       {
         Assert(saved_lookahead.tok.get_tag() == TOK_VOID);
-        saved_lookahead = pop();
+        saved_lookahead.copy(pop(), LOC);
         --prefix_len;
       }
 
@@ -471,20 +528,6 @@ Prefix::replace_AB(Value_P old_value, Value_P new_value)
 }
 //-----------------------------------------------------------------------------
 void
-Prefix::lock_values(bool lock)
-{
-   loop(s, size())
-      {
-        Token & tok = at(s).tok;
-        if (tok.get_ValueType() == TV_VAL)
-           {
-             if (lock)   tok.get_apl_val()->set_arg();
-             else        tok.get_apl_val()->clear_arg();
-           }
-      }
-}
-//-----------------------------------------------------------------------------
-void
 Prefix::print(ostream & out, int indent) const
 {
    loop(i, indent)   out << "    ";
@@ -512,7 +555,7 @@ Prefix::reduce_LPAR_B_RPAR_()
    Assert1(prefix_len == 3);
 
 Token result = at1();   // B
-   if (result.get_tag() == TOK_APL_VALUE)   result.ChangeTag(TOK_APL_VALUE1);
+   if (result.get_tag() == TOK_APL_VALUE3)   result.ChangeTag(TOK_APL_VALUE1);
    pop_args_push_result(result);
    set_action(result);
 }
@@ -528,9 +571,9 @@ Prefix::reduce_N___()
 {
    Assert1(prefix_len == 1);
 
-const Token result = si.eval_(at0());
-   at0() = result;
-   set_action(result);
+Token result = si.eval_(at0());
+   move_1(at0(), result, LOC);
+   set_action(at0());
 }
 //-----------------------------------------------------------------------------
 void
@@ -538,9 +581,8 @@ Prefix::reduce_MISC_F_B_()
 {
    Assert1(prefix_len == 2);
 
-const Token result = si.eval_B(at0(), at1());
-   pop_args_push_result(result);
-   set_action(result);
+   pop_args_push_result(si.eval_B(at0(), at1()));
+   set_action(at0());
 }
 //-----------------------------------------------------------------------------
 void
@@ -548,9 +590,8 @@ Prefix::reduce_MISC_F_C_B()
 {
    Assert1(prefix_len == 3);
 
-const Token result = si.eval_XB(at0(), at1(), at2());
-   pop_args_push_result(result);
-   set_action(result);
+   pop_args_push_result(si.eval_XB(at0(), at1(), at2()));
+   set_action(at0());
 }
 //-----------------------------------------------------------------------------
 void
@@ -558,9 +599,8 @@ Prefix::reduce_A_F_B_()
 {
    Assert1(prefix_len == 3);
 
-const Token result = si.eval_AB(at0(), at1(), at2());
-   pop_args_push_result(result);
-   set_action(result);
+   pop_args_push_result(si.eval_AB(at0(), at1(), at2()));
+   set_action(at0());
 }
 //-----------------------------------------------------------------------------
 void
@@ -576,9 +616,8 @@ Prefix::reduce_A_F_C_B()
 {
    Assert1(prefix_len == 4);
 
-const Token result = si.eval_AXB(at0(), at1(), at2(), at3());
-   pop_args_push_result(result);
-   set_action(result);
+   pop_args_push_result(si.eval_AXB(at0(), at1(), at2(), at3()));
+   set_action(at0());
 }
 //-----------------------------------------------------------------------------
 void
@@ -591,11 +630,10 @@ void
 Prefix::reduce_F_M__()
 {
 DerivedFunction * derived =
-   Workspace::the_workspace->SI_top()->fun_oper_cache.get(LOC);
+   Workspace::SI_top()->fun_oper_cache.get(LOC);
    new (derived) DerivedFunction(at0(), at1().get_function(), LOC);
 
-Token result = Token(TOK_FUN2, derived);
-   pop_args_push_result(result);
+   pop_args_push_result(Token(TOK_FUN2, derived));
    action = RA_CONTINUE;
 }
 //-----------------------------------------------------------------------------
@@ -621,19 +659,20 @@ void
 Prefix::reduce_F_D_G_()
 {
 DerivedFunction * derived =
-   Workspace::the_workspace->SI_top()->fun_oper_cache.get(LOC);
+   Workspace::SI_top()->fun_oper_cache.get(LOC);
    new (derived) DerivedFunction(at0(), at1().get_function(), at2(), LOC);
 
-Token result = Token(TOK_FUN2, derived);
-   pop_args_push_result(result);
+   pop_args_push_result(Token(TOK_FUN2, derived));
    action = RA_CONTINUE;
 }
 //-----------------------------------------------------------------------------
 void
 Prefix::reduce_A_C__()
 {
-Value_P value = at0().get_apl_val()->index(at1());
-Token result = Token(TOK_APL_VALUE1, value);
+Value_P A = at0().get_apl_val();
+Value_P Z = A->index(at1());
+
+Token result = Token(TOK_APL_VALUE1, Z);
    pop_args_push_result(result);
 
    set_action(result);
@@ -642,23 +681,51 @@ Token result = Token(TOK_APL_VALUE1, value);
 void
 Prefix::reduce_V_C_ASS_B()
 {
-Symbol * var = at0().get_sym_ptr();
-Value_P value = at3().get_apl_val();
+Symbol * V = at0().get_sym_ptr();
+Value_P B = at3().get_apl_val();
 
-   if (at1().get_tag() == TOK_AXES)
+   if (at1().get_tag() == TOK_AXES)   // [] or [x]
       {
-        Value_P idx = at1().get_axes();
-        var->assign_indexed(idx, value);
-        idx->erase(LOC);
+        Value_P v_idx = at1().get_axes();
+
+        try
+           {
+             V->assign_indexed(v_idx, B);
+           }
+        catch (Error err)
+           {
+             Token result = Token(TOK_ERROR, err.error_code);
+             at1().clear(LOC);
+             at3().clear(LOC);
+             pop_args_push_result(result);
+             set_assign_state(ASS_none);
+             set_action(result);
+             return;
+           }
+
       }
-   else
+   else                               // [a;...]
       {
-        var->assign_indexed(at1().get_index_val(), value);
+        try
+           {
+             V->assign_indexed(at1().get_index_val(), B);
+           }
+        catch (Error err)
+           {
+             Token result = Token(TOK_ERROR, err.error_code);
+             delete &at1().get_index_val();
+             at1().clear(LOC);
+             at3().clear(LOC);
+             pop_args_push_result(result);
+             set_assign_state(ASS_none);
+             set_action(result);
+             return;
+           }
       }
 
-Token result = Token(TOK_APL_VALUE2, value);
+Token result = Token(TOK_APL_VALUE2, B);
    pop_args_push_result(result);
-   set_assign_pending(false);
+   set_assign_state(ASS_none);
    set_action(result);
 }
 //-----------------------------------------------------------------------------
@@ -668,7 +735,8 @@ Prefix::reduce_F_V__()
    // turn V into a (left-) value
    //
 Symbol * V = at1().get_sym_ptr();
-   at1() = V->resolve_lv();
+   copy_1(at1(), V->resolve_lv(LOC), LOC);
+   set_assign_state(ASS_var_seen);
    action = RA_CONTINUE;
 }
 //-----------------------------------------------------------------------------
@@ -689,21 +757,21 @@ Value_P B = at2().get_apl_val();
 Token result = Token(TOK_APL_VALUE2, B);
    pop_args_push_result(result);
 
-   set_assign_pending(false);
+   set_assign_state(ASS_none);
    action = RA_CONTINUE;
 }
 //-----------------------------------------------------------------------------
 void
 Prefix::reduce_V_ASS_B_()
 {
-Value_P value = at2().get_apl_val();
+Value_P B = at2().get_apl_val();
 Symbol * V = at0().get_sym_ptr();
-   V->assign(value);
+   V->assign(B, LOC);
 
-Token result = Token(TOK_APL_VALUE2, value);
+Token result = Token(TOK_APL_VALUE2, B);
    pop_args_push_result(result);
 
-   set_assign_pending(false);
+   set_assign_state(ASS_none);
    action = RA_CONTINUE;
 }
 //-----------------------------------------------------------------------------
@@ -712,17 +780,21 @@ Prefix::reduce_RBRA___()
 {
    Assert1(prefix_len == 1);
 
-   // start partial index list
+   // start partial index list. Parse the index as right so that, for example,
+   // A[IDX}←B resolves IDX properly. assign_state is restored when the
+   // index is complete.
    //
-   at0() = Token(TOK_PINDEX, new IndexExpr(LOC));
+const int left = get_assign_state() == ASS_arrow_seen;
+   new (&at0()) Token(TOK_PINDEX, *new IndexExpr(left, LOC));
+   set_assign_state(ASS_none);
    action = RA_CONTINUE;
 }
 //-----------------------------------------------------------------------------
 void
 Prefix::reduce_LBRA_I__()
 {
-   // [ I or ; I
-
+   // [ I or ; I   (elided index)
+   //
    Assert1(prefix_len == 2);
 
 IndexExpr & idx = at1().get_index_val();
@@ -730,8 +802,8 @@ const bool last_index = (at0().get_tag() == TOK_L_BRACK);
 
    if (idx.value_count() == 0 && last_index)   // special case: [ ]
       {
-        idx.add(0);
-        Token result = Token(TOK_INDEX, &idx);
+        assign_state = idx.get_left() ? ASS_arrow_seen : ASS_none;
+        Token result = Token(TOK_INDEX, idx);
         pop_args_push_result(result);
         action = RA_CONTINUE;
         return;
@@ -740,12 +812,18 @@ const bool last_index = (at0().get_tag() == TOK_L_BRACK);
    // add elided index to partial index list
    //
 Token result = at1();
-   result.get_index_val().add(0);
+   result.get_index_val().add(Value_P());
 
-   if (last_index)
+   if (last_index)   // [ seen
       {
-        if (idx.is_axis())   result = Token(TOK_AXES, idx.values[0]);
-        else                 result = Token(TOK_INDEX, &idx);
+        assign_state = idx.get_left() ? ASS_arrow_seen : ASS_none;
+
+        if (idx.is_axis()) move_2(result, Token(TOK_AXES, idx.values[0]), LOC);
+        else               move_2(result, Token(TOK_INDEX, idx), LOC);
+      }
+   else
+      {
+        set_assign_state(ASS_none);
       }
 
    pop_args_push_result(result);
@@ -757,30 +835,36 @@ Prefix::reduce_LBRA_B_I_()
 {
    Assert1(prefix_len == 3);
 
-   // add value to partial index list
+   // [ B I or ; B I   (normal index)
    //
-Token result = at2();
-   result.get_index_val().add(at1().get_apl_val());
+Token I = at2();
+   I.get_index_val().add(at1().get_apl_val());
 
-const bool last_index = (at0().get_tag() == TOK_L_BRACK);
-   if (last_index)
+const bool last_index = (at0().get_tag() == TOK_L_BRACK);   // ; vs. [
+
+   if (last_index)   // [ seen
       {
-        IndexExpr & idx = result.get_index_val();
+        IndexExpr & idx = I.get_index_val();
+        assign_state = idx.get_left() ? ASS_arrow_seen : ASS_none;
+
         if (idx.is_axis())
            {
-             Value * iv = idx.values[0];
-             Assert1(iv);
-             idx.values[0] = 0;
-             iv->clear_index();
-             result = Token(TOK_AXES, iv);
+             Value_P X = idx.extract_value(0);
+             Assert1(!!X);
+             move_2(I, Token(TOK_AXES, X), LOC);
+             delete &idx;
            }
         else
            {
-             result = Token(TOK_INDEX, &idx);
+             move_2(I, Token(TOK_INDEX, idx), LOC);
            }
       }
+   else
+      {
+         set_assign_state(ASS_none);
+      }
 
-   pop_args_push_result(result);
+   pop_args_push_result(I);
    action = RA_CONTINUE;
 }
 //-----------------------------------------------------------------------------
@@ -815,7 +899,9 @@ const int count = vector_ass_count();
         // selective specification. Convert variable V into a (left-) value
         //
         Symbol * V = at0().get_sym_ptr();
-        at0() = V->resolve_lv();
+        Token result = V->resolve_lv(LOC);
+        set_assign_state(ASS_var_seen);
+        move_1(at0(), result, LOC);
         action = RA_CONTINUE;
         return;
       }
@@ -829,13 +915,13 @@ Value_P B = at3().get_apl_val();
            B = B->get_ravel(0).get_pointer_value();
 
         Symbol * V = at0().get_sym_ptr();
-        V->assign(B);
+        V->assign(B, LOC);
         loop(c, count)
            {
              Token_loc tl = lookahead();
              Assert1(tl.tok.get_tag() == TOK_LSYMB2);   // by vector_ass_count()
              V = tl.tok.get_sym_ptr();
-             V->assign(B);
+             V->assign(B, LOC);
            }
       }
    else
@@ -848,8 +934,8 @@ Value_P B = at3().get_apl_val();
         {
           Symbol * V = at0().get_sym_ptr();
           Cell & cell = B->get_ravel(count);
-          Value_P B_last = new Value(cell, LOC);
-          V->assign(B_last);
+          Value_P B_last(new Value(cell, LOC));
+          V->assign(B_last, LOC);
         }
 
         // assign other elements of B to other variables...
@@ -860,12 +946,12 @@ Value_P B = at3().get_apl_val();
              Assert1(tl.tok.get_tag() == TOK_LSYMB2);   // by vector_ass_count()
              Symbol * V = tl.tok.get_sym_ptr();
              Cell & cell = B->get_ravel(count - c - 1);
-             Value_P B_cell = new Value(cell, LOC);
-             V->assign(B_cell);
+             Value_P B_cell(new Value(cell, LOC));
+             V->assign(B_cell, LOC);
            }
       }
 
-   set_assign_pending(false);
+    set_assign_state(ASS_none);
 
    // clean up stack
    //
@@ -886,8 +972,8 @@ Prefix::reduce_END_VOID__()
 
 const bool end_of_line = at0().get_tag() == TOK_ENDL;
 
-   pop();   // pop END
-   pop();   // pop VOID
+   pop_and_discard();   // pop END
+   pop_and_discard();   // pop VOID
 
 Token Void(TOK_VOID);
    si.statement_result(Void);
@@ -909,12 +995,11 @@ Prefix::reduce_END_B__()
 
 const bool end_of_line = at0().get_tag() == TOK_ENDL;
 
-Token B = at1();
-   pop();   // pop END
-   pop();   // pop B
+   pop_and_discard();   // pop END
+Token B = pop().tok;    // pop B
 
-   B.clone_if_owned(LOC);
    si.statement_result(B);
+
    action = RA_NEXT_STAT;
    if (attention_raised && end_of_line)
       {
@@ -931,11 +1016,12 @@ Prefix::reduce_END_GOTO_B_()
 
    if (size() != 3)   syntax_error(LOC);
 
+   // at0() is either TOK_END or TOK_ENDL.
+   //
 const bool end_of_line = at0().get_tag() == TOK_ENDL;
 Value_P line = at2().get_apl_val();
 
 const Token result = si.jump(line);
-   line->erase(LOC);
 
    if (result.get_tag() == TOK_BRANCH)   // branch back into a function
       {
@@ -1020,14 +1106,14 @@ Prefix::reduce_RETC___()
         case TOK_RETURN_EXEC:   // immediate execution context
              Log(LOG_prefix_parser)
                 CERR << "- end of ⍎ context (no result)" << endl;
-             at0() = Token(TOK_VOID);
+             at0().clear(LOC);
              action = RA_RETURN;
              return;
 
         case TOK_RETURN_STATS:   // immediate execution context
              Log(LOG_prefix_parser)
                 CERR << "- end of ◊ context" << endl;
-             at0() = Token(TOK_VOID);
+             at0().clear(LOC);
              action = RA_RETURN;
              return;
 
@@ -1039,7 +1125,7 @@ Prefix::reduce_RETC___()
                const UserFunction * ufun = si.get_executable()->get_ufun();
                Assert1(ufun);
                ufun->pop_local_vars();
-               at0() = Token(TOK_VOID);
+               at0().clear(LOC);
              }
              action = RA_RETURN;
              return;
@@ -1049,19 +1135,19 @@ Prefix::reduce_RETC___()
                const UserFunction * ufun = si.get_executable()->get_ufun();
                Assert1(ufun);
                Value_P Z = ufun->pop_local_vars();
-               if (Z)
+               if (!!Z)
                   {
                     Log(LOG_prefix_parser)
                        CERR << "- end of ∇ context (function result is: "
                             << *Z << ")" << endl;
-                    at0() = Token(TOK_APL_VALUE1, Z);
+                    new (&at0()) Token(TOK_APL_VALUE1, Z);
                   }
                else
                   {
                     Log(LOG_prefix_parser)
                        CERR << "- end of ∇ context (MISSING function result)."
                             << endl;
-                    at0() = Token(TOK_VOID);
+                    at0().clear(LOC);
                   }
              }
              action = RA_RETURN;
@@ -1089,10 +1175,9 @@ Prefix::reduce_RETC_B__()
    Log(LOG_prefix_parser)
       CERR << "- end of ⍎ context.";
 
-Token result = at1();
-   result.clone_if_owned(LOC);
+Token B = at1();
+   pop_args_push_result(B);
 
-   pop_args_push_result(result);
    action = RA_RETURN;
 }
 //-----------------------------------------------------------------------------
