@@ -42,6 +42,7 @@ PrintBuffer::PrintBuffer(const UCS_string & ucs, const ColInfo & ci)
 PrintBuffer::PrintBuffer(const Value & value, const PrintContext & _pctx)
 {
 const PrintStyle outer_style = _pctx.get_style();
+const bool framed = outer_style & (PST_CS_MASK | PST_CS_OUTER);
 PrintContext pctx(_pctx);
    pctx.set_style(PrintStyle(outer_style &~ PST_CS_OUTER));
 
@@ -51,10 +52,12 @@ PrintContext pctx(_pctx);
         if (value.get_ravel(0).need_scaling(pctx))   pctx1.set_scaled();
 
         *this = value.get_ravel(0).character_representation(pctx1);
-        if (value.compute_depth() > 1)
+
+        // pad the value unless it is framed
+        if (value.compute_depth() > 1 && !framed)
            {
-                pad_l(UNI_PAD_U4);
-                pad_r(UNI_PAD_U5);
+                pad_l(UNI_PAD_U4, 1);
+                pad_r(UNI_PAD_U5, 1);
            }
 
         add_outer_frame(outer_style);
@@ -64,17 +67,16 @@ PrintContext pctx(_pctx);
 const ShapeItem ec = value.element_count();
    if (ec == 0)   // empty value of any dimension
       {
-         Shape sh(value.get_shape());
-         sh.remove_last_shape_item();
-         if (sh.element_count() <= 1)   // empty vector
+         const Shape sh = value.get_shape().without_axis(sh.get_rank() - 1);
+         if (sh.get_volume() <= 1)   // empty vector
             {
               if (pctx.get_style() == PR_APL_FUN)
                  {
                    if (value.get_ravel(0).is_character_cell())   // ''
                       {
                         UCS_string ucs;
-                        ucs += UNI_SINGLE_QUOTE;
-                        ucs += UNI_SINGLE_QUOTE;
+                        ucs.append(UNI_SINGLE_QUOTE);
+                        ucs.append(UNI_SINGLE_QUOTE);
 
                         ColInfo ci;
                         *this = PrintBuffer(ucs, ci);
@@ -88,12 +90,12 @@ const ShapeItem ec = value.element_count();
 
          // value has > 0 rows. Compute how many lines we need.
          //
-         ShapeItem lines = sh.element_count();
+         ShapeItem lines = sh.get_volume();
          loop(s, sh.get_rank())
              lines += s * (sh.get_shape_item(sh.get_rank() - s - 1));
 
          buffer.resize(lines);
-        add_outer_frame(outer_style);
+         add_outer_frame(outer_style);
          return;
       }
 
@@ -106,14 +108,14 @@ const ShapeItem rows = ec/cols;
 
         if (value.is_char_vector())
            {
-             ucs += UNI_SINGLE_QUOTE;
+             ucs.append(UNI_SINGLE_QUOTE);
              loop(e, ec)
                 {
                   const Unicode uni = value.get_ravel(e).get_char_value();
-                  ucs += uni;
-                  if (uni == UNI_SINGLE_QUOTE)   ucs += uni;   // ' -> ''
+                  ucs.append(uni);
+                  if (uni == UNI_SINGLE_QUOTE)   ucs.append(uni);   // ' -> ''
                 }
-             ucs += UNI_SINGLE_QUOTE;
+             ucs.append(UNI_SINGLE_QUOTE);
            }
         else
            {
@@ -121,8 +123,8 @@ const ShapeItem rows = ec/cols;
                 {
                   PrintBuffer pb = value.get_ravel(e)
                                         .character_representation(pctx);
-                  if (e)   ucs += UNI_ASCII_SPACE;
-                  ucs += UCS_string(pb, 0);
+                  if (e)   ucs.append(UNI_ASCII_SPACE);
+                  ucs.append(UCS_string(pb, 0));
                 }
            }
 
@@ -175,7 +177,7 @@ const ShapeItem rows = ec/cols;
 
               loop(c, sizeof(cc))
                  {
-                   if (cc[c])   ucs += Unicode(cc[c]);
+                   if (cc[c])   ucs.append(Unicode(cc[c]));
                    else         break;
                  }
            }
@@ -187,7 +189,7 @@ const ShapeItem rows = ec/cols;
       }
 
    // 1. create a vector with a bool per column that tells if the
-   // column needs scaling or not
+   // column needs scaling (i.e. exponential format) or not
    //
 vector<bool> scaling;
    {
@@ -207,7 +209,9 @@ vector<bool> scaling;
         }
    }
 
-   // 2. create a matrix of all items, padding all items to the same height.
+   // 2. create a matrix of items.
+   //    An item of the matrix is a top-level value (possibly nested),
+   //    therefore we have (⍴,value) items. Items are rectangular.
    //
 typedef vector<PrintBuffer> PB_row;
 vector<PB_row> item_matrix;
@@ -229,12 +233,16 @@ vector<PB_row> item_matrix;
               if (row_height < item.get_height())
                  row_height = item.get_height();
 
+              Assert(item.is_rectangular());
               item_row.push_back(item);
             }
 
         // pad all items to the same height
-        loop(x, cols)   item_row[x].pad_height(UNI_PAD_U6, row_height);
-
+        loop(x, cols)
+           {
+             item_row[x].pad_height(UNI_PAD_U6, row_height);
+             Assert(item_row[x].is_rectangular());
+           }
         item_matrix.push_back(item_row);
       }
 
@@ -244,19 +252,10 @@ vector<PB_row> item_matrix;
    loop(x, cols)
       {
         ColInfo col_info_x;
-        loop(y, rows)
-           {
-             PB_row & row = item_matrix[y];
-             PrintBuffer & item = row[x];
-             col_info_x.consider(item.get_info());
-           }
+        loop(y, rows)   col_info_x.consider(item_matrix[y][x].get_info());
+
+        loop(y, rows)   item_matrix[y][x].align(col_info_x);
 
-        loop(y, rows)
-           {
-             PB_row & row = item_matrix[y];
-             PrintBuffer & item = row[x];
-             item.align(col_info_x);
-           }
       }
 
    // 4. combine colums, then rows.
@@ -278,7 +277,7 @@ int break_width = 0;
                    {
                      prod *= value.get_shape_item(value.get_rank() - r - 2);
                      if (y % prod == 0)
-                        pcol.append_string(UCS_string(pcol.get_width(0),
+                        pcol.append_ucs(UCS_string(pcol.get_width(0),
                                                       UNI_PAD_L0));
                      else
                         break;
@@ -291,8 +290,8 @@ int break_width = 0;
              else     pcol = item;
          }
 
-        bool not_char = false;
-	const int32_t col_spacing = value.get_col_spacing(x, not_char);
+        bool not_char = false;   // determined by get_col_spacing()
+	const int32_t col_spacing = value.get_col_spacing(not_char, x, framed);
 
         const int32_t max_spacing = (col_spacing > last_spacing) 
                                   ?  col_spacing : last_spacing;
@@ -307,12 +306,12 @@ int break_width = 0;
            {
               if (last_notchar)
                  {
-                   pad_r(UNI_PAD_U2);
+                   pad_r(UNI_PAD_U2, 1);
                    ++no_char_spacing;
                  }
               else if (not_char)
                  {
-                   pcol.pad_l(UNI_PAD_U3);
+                   pcol.pad_l(UNI_PAD_U3, 1);
                    ++no_char_spacing;
                  }
 
@@ -330,8 +329,9 @@ int break_width = 0;
                  }
               }
 
-              // append the column. we want a spacing of 'max_spacing'
-              // but we deducat the chars appended already('no_char_spacing').
+              // append the column. we want a total spacing of 'max_spacing'
+              // but we deduct the 'no_char_spacing' chars ² and ³
+              // that were appended already.
               //
               add_column(UNI_PAD_U7, max_spacing - no_char_spacing, pcol);
            }
@@ -340,12 +340,20 @@ int break_width = 0;
         last_notchar = not_char;
       }
 
-   if (value.compute_depth() > 1)
+   if (value.compute_depth() > 1 && !framed)
       {
-           pad_l(UNI_PAD_U8);
-           pad_r(UNI_PAD_U9);
+        pad_l(UNI_PAD_U8, 1);
+        pad_r(UNI_PAD_U9, 1);
       }
 
+   if (!is_rectangular())
+      {
+        Q(get_height())
+        loop(h, get_height())   CERR << "w=" << get_width(h) << "*" << endl;
+        loop(h, get_height())   CERR << "*" << get_line(h) << "*" << endl;
+      }
+
+   Assert(is_rectangular());
    add_outer_frame(outer_style);
 
 // debug("PrintBuffer::PrintBuffer(Value_P, ...)");
@@ -380,32 +388,16 @@ PrintBuffer::set_char(uint32_t x, uint32_t y, Unicode uc)
 }
 //-----------------------------------------------------------------------------
 void
-PrintBuffer::pad_l(Unicode pad)
+PrintBuffer::pad_l(Unicode pad, ShapeItem count)
 {
-   loop(y, get_height())   buffer[y].insert(0, 1, pad);
+   loop(y, get_height())   buffer[y].insert(0, count, pad);
 }
 //-----------------------------------------------------------------------------
 void
-PrintBuffer::pad_l(const UCS_string & pad)
+PrintBuffer::pad_r(Unicode pad, ShapeItem count)
 {
-   loop(y, get_height())
-      {
-        UCS_string ucs = pad;
-        ucs += buffer[y];
-        buffer[y] = ucs;
-      }
-}
-//-----------------------------------------------------------------------------
-void
-PrintBuffer::pad_r(Unicode pad)
-{
-   loop(y, get_height())   buffer[y] += pad;
-}
-//-----------------------------------------------------------------------------
-void
-PrintBuffer::pad_r(const UCS_string & pad)
-{
-   loop(y, get_height())   buffer[y] += pad;
+UCS_string ucs(count, pad);
+   loop(y, get_height())   buffer[y].append(ucs);
 }
 //-----------------------------------------------------------------------------
 void
@@ -484,11 +476,13 @@ Unicode HORI, VERT, NW, NE, SE, SW;
    if (get_height() == 0)   // empty
       {
         UCS_string upper;
-        upper += NE;   upper += NW;
+        upper.append(NE);
+        upper.append(NW);
         buffer.push_back(upper);
 
         UCS_string lower;
-        lower += SE;   lower += SW;
+        lower.append(SE);
+        lower.append(SW);
         buffer.push_back(lower);
 
         Assert(is_rectangular());
@@ -500,7 +494,7 @@ Unicode HORI, VERT, NW, NE, SE, SW;
    loop(y, get_height())
       {
         buffer[y].insert(0, 1, VERT);
-        buffer[y] += VERT;
+        buffer[y].append(VERT);
       }
 
    // draw a bar on top and bottom.
@@ -525,15 +519,15 @@ UCS_string hori(get_width(0), HORI);
       {
         loop(d, depth - 1)
             if (d + 1 < get_width(0) - 1)
-               set_char(1 + d, get_height() - 1, UNI_ELEMENT);
+               set_char(1 + d, get_height() - 1, UNI_ELEMENT1);
       }
 
-   if (style & PST_EMPTY_LAST)
+   if (style & PST_EMPTY_LAST)   // last (X-) dimension is empty
       {
         set_char(1, 0, UNI_CIRCLE_BAR);   // ⊖
       }
 
-   if (style & PST_EMPTY_NLAST)
+   if (style & PST_EMPTY_NLAST)   // a non-last (Y-) dimension is empty
       {
         set_char(0, 1, UNI_CIRCLE_STILE);   // ⌽
       }
@@ -553,11 +547,13 @@ Unicode HORI, VERT, NW, NE, SE, SW;
    if (get_height() == 0)   // empty
       {
         UCS_string upper;
-        upper += NE;   upper += NW;
+        upper.append(NE);
+        upper.append(NW);
         buffer.push_back(upper);
 
         UCS_string lower;
-        lower += SE;   lower += SW;
+        lower.append(SE);
+        lower.append(SW);
         buffer.push_back(lower);
 
         Assert(is_rectangular());
@@ -569,7 +565,7 @@ Unicode HORI, VERT, NW, NE, SE, SW;
    loop(y, get_height())
       {
         buffer[y].insert(0, 1, VERT);
-        buffer[y] += VERT;
+        buffer[y].append(VERT);
       }
 
    // draw a bar on top and bottom.
@@ -629,11 +625,11 @@ PrintBuffer::append_col(const PrintBuffer & pb1)
 {
    Assert(get_height() == pb1.get_height());
 
-   loop(h, get_height())   buffer[h] += pb1.buffer[h];
+   loop(h, get_height())   buffer[h].append(pb1.buffer[h]);
 }
 //-----------------------------------------------------------------------------
 void
-PrintBuffer::append_string(const UCS_string & ucs)
+PrintBuffer::append_ucs(const UCS_string & ucs)
 {
    if (buffer.size() == 0)   // empty
       {
@@ -643,13 +639,13 @@ PrintBuffer::append_string(const UCS_string & ucs)
       {
         UCS_string ucs1(ucs);
         UCS_string pad(get_width(0) - ucs.size(), UNI_PAD_L1);
-        ucs1 += pad;
+        ucs1.append(pad);
         buffer.push_back(ucs1);
       }
    else if (ucs.size() > get_width(0))
       {
         UCS_string pad(ucs.size() - get_width(0), UNI_PAD_L2);
-        loop(h, get_height())   buffer[h] += pad;
+        loop(h, get_height())   buffer[h].append(pad);
         buffer.push_back(ucs);
       }
    else
@@ -749,12 +745,12 @@ int this_r = 0;   // padding right of this
 
 UCS_string ucs1;
 
-   if (ucs_l > 0)   ucs1 += UCS_string(ucs_l, UNI_ASCII_SPACE);
-   ucs1 += ucs;
-   if (ucs_r > 0)   ucs1 += UCS_string(ucs_r, UNI_ASCII_SPACE);
+   if (ucs_l > 0)   ucs1.append(UCS_string(ucs_l, UNI_ASCII_SPACE));
+   ucs1.append(ucs);
+   if (ucs_r > 0)   ucs1.append(UCS_string(ucs_r, UNI_ASCII_SPACE));
 
-   if (this_l > 0)   pad_l(UCS_string(this_l, UNI_ASCII_SPACE));
-   if (this_r > 0)   pad_r(UCS_string(this_r, UNI_ASCII_SPACE));
+   if (this_l > 0)   pad_l(UNI_ASCII_SPACE, this_l);
+   if (this_r > 0)   pad_r(UNI_ASCII_SPACE, this_r);
 
    buffer.push_back(ucs1);
 
@@ -775,10 +771,10 @@ PrintBuffer::add_column(Unicode pad, int32_t pad_count, const PrintBuffer & pb)
    if (pad_count)
       {
         UCS_string ucs(pad_count, pad);
-        loop(y, get_height())   buffer[y] += ucs;
+        loop(y, get_height())   buffer[y].append(ucs);
       }
 
-   loop(y, get_height())   buffer[y] += pb.buffer[y];
+   loop(y, get_height())   buffer[y].append(pb.buffer[y]);
 }
 //-----------------------------------------------------------------------------
 void PrintBuffer::add_row(const PrintBuffer & pb)
@@ -790,9 +786,22 @@ void PrintBuffer::add_row(const PrintBuffer & pb)
 void
 PrintBuffer::align(ColInfo & cols)
 {
+   // this PrintBuffer is a (possibly nested) APL value.
+   // Align the buffer:
+   //
+   // to the J (in a column containing complex numbers), or
+   // to the decimal point (in a column containing non-complex numbers), or
+   // to the left (in a column containing text or nested values).
+   //
+   // make sure that the item is (and remains) rectangular).
+   //
+   Assert(is_rectangular());
+
    if      (cols.flags & has_j)          align_j(cols);
    else if ((cols.flags & CT_NUMERIC))   align_dot(cols);
    else                                  align_left(cols);
+
+   Assert(is_rectangular());
 }
 //-----------------------------------------------------------------------------
 void
@@ -821,9 +830,7 @@ PrintBuffer::align_dot(ColInfo & COL_INFO)
         if (COL_INFO.int_len > col_info.int_len)
            {
              const size_t diff = COL_INFO.int_len - col_info.int_len;
-             UCS_string pad(diff, UNI_PAD_L3);
-             pad += buffer[0];
-             buffer[0] = pad;
+             pad_l(UNI_PAD_L3, diff);
              col_info.real_len += diff;
              col_info.int_len = COL_INFO.int_len;
            }
@@ -838,16 +845,14 @@ PrintBuffer::align_dot(ColInfo & COL_INFO)
              const size_t diff = COL_INFO.real_len - col_info.real_len;
              if (col_info.have_expo())   // alrady have an expo field
                 {
-                  UCS_string pad(diff, UNI_PAD_L4);
-                  buffer[0] += pad;
+                  pad_r(UNI_PAD_L4, diff);
                 }
              else                        // no expo yet: create one
                 {
                   Assert1(diff >= 2);
-                  buffer[0] += UNI_ASCII_E;
-                  buffer[0] += UNI_ASCII_0;
-                  UCS_string pad(diff - 2, UNI_PAD_L4);
-                  buffer[0] += pad;
+                  pad_r(UNI_ASCII_E, 1);
+                  pad_r(UNI_ASCII_0, 1);
+                  pad_r(UNI_PAD_L4, diff - 2);
                 }
              col_info.real_len = COL_INFO.real_len;
            }
@@ -860,9 +865,7 @@ PrintBuffer::align_dot(ColInfo & COL_INFO)
         if (LEN > len)
            {
              const size_t diff = LEN - len;
-             UCS_string pad(diff, UNI_PAD_L5);
-             pad += buffer[0];
-             buffer[0] = pad;
+             pad_l(UNI_PAD_L5, diff);
              col_info.int_len   = COL_INFO.int_len;
              col_info.fract_len = COL_INFO.fract_len;
            }
@@ -896,17 +899,14 @@ PrintBuffer::align_j(ColInfo & COL_INFO)
         if (COL_INFO.real_len > col_info.real_len)
            {
              const size_t diff = COL_INFO.real_len - col_info.real_len;
-             UCS_string pad(diff, UNI_PAD_L3);
-             pad += buffer[0];
-             buffer[0] = pad;
+             pad_l(UNI_PAD_L3, diff);
              col_info.real_len = COL_INFO.real_len;
            }
 
         if (COL_INFO.imag_len > col_info.imag_len)
            {
              const size_t diff = COL_INFO.imag_len - col_info.imag_len;
-             UCS_string pad(diff, UNI_PAD_L4);
-             buffer[0] += pad;
+             pad_r(UNI_PAD_L4, diff);
              col_info.imag_len = COL_INFO.imag_len;
            }
       }
@@ -918,9 +918,7 @@ PrintBuffer::align_j(ColInfo & COL_INFO)
         if (LEN > len)
            {
              const size_t diff = LEN - len;
-             UCS_string pad(diff, UNI_PAD_L5);
-             pad += buffer[0];
-             buffer[0] = pad;
+             pad_l(UNI_PAD_L5, diff);
              col_info.real_len = COL_INFO.real_len;
              col_info.imag_len = COL_INFO.imag_len;
            }
@@ -935,39 +933,51 @@ PrintBuffer::pad_fraction(int wanted_fract_len, bool want_expo)
 const int diff = wanted_fract_len - col_info.fract_len;
    Assert1(diff > 0);
 
-UCS_string new_buf;
-
-      // copy integer part to new_buf
+      // copy integer part of this PrintBuffer to to new_buf
       //
-      loop(i, col_info.int_len)   new_buf +=  buffer[0][i];
+UCS_string new_buf(buffer[0], 0, col_info.int_len);
 
       // copy fractional part to new_buf. If the number has no exponent part,
       // then we fill with spaces. Otherwise fill with '0', possibly inserting
       // a decimal point.
       //
-      loop(f, col_info.fract_len)   new_buf += buffer[0][col_info.int_len + f];
+      loop(f, col_info.fract_len)   new_buf.append(buffer[0][col_info.int_len + f]);
       if (!want_expo)
          {
-           loop(d, diff)   new_buf += UNI_PAD_L4;
+           loop(d, diff)   new_buf.append(UNI_PAD_L4);
          }
       else if (col_info.fract_len == 0)   // no fractional part yet
          {
-           new_buf += UNI_ASCII_FULLSTOP;
-           loop(d, diff - 1)   new_buf += UNI_ASCII_0;
+           new_buf.append(UNI_ASCII_FULLSTOP);
+           loop(d, diff - 1)   new_buf.append(UNI_ASCII_0);
          }
       else
          {
-           loop(d, diff)   new_buf += UNI_ASCII_0;
+           loop(d, diff)   new_buf.append(UNI_ASCII_0);
          }
 
    // copy exponent part
    //
-   for (int e = col_info.int_len + col_info.fract_len; e < buffer[0].size(); ++e)
-       new_buf += buffer[0][e];
+   for (int ex = col_info.int_len + col_info.fract_len;
+        ex < buffer[0].size(); ++ex)
+       new_buf.append(buffer[0][ex]);
 
    col_info.fract_len = wanted_fract_len;
    col_info.real_len += diff;
+
    buffer[0] = new_buf;
+
+   // if buffer is multi-len then pad remaining line to new length
+   //
+   for (ShapeItem h = 1; h < get_height(); ++h)
+      {
+        const int diff = new_buf.size() - get_width(h);
+        if (diff > 0)
+           {
+             const UCS_string ucs(diff, UNI_PAD_L4);
+             buffer[h].append(ucs);
+           }
+      }
 }
 //-----------------------------------------------------------------------------
 void
@@ -989,16 +999,10 @@ PrintBuffer::align_left(ColInfo & COL_INFO)
    Assert(col_info.int_len < COL_INFO.int_len);
 
 const size_t diff = COL_INFO.int_len - col_info.int_len;
-UCS_string pad(diff, UNI_PAD_L3);
 
-   if (buffer.size())
-      {
-        loop(s, buffer.size())   buffer[s] += pad;
-      }
-   else
-      {
-        buffer.push_back(pad);
-      }
+   if (buffer.size())   pad_r(UNI_PAD_L3, diff);
+   else                 buffer.push_back(UCS_string(diff, UNI_PAD_L3));
+
    col_info.int_len = COL_INFO.int_len;
 
    Log(LOG_printbuf_align)   debug(CERR, "after align_left()");
@@ -1007,13 +1011,13 @@ UCS_string pad(diff, UNI_PAD_L3);
 bool
 PrintBuffer::is_rectangular() const
 {
-   if (get_height() == 0)   return true;
-
-const int64_t w = get_width(0);
-
-   loop(h, get_height())
+   if (get_height())
       {
-        if (get_width(h) != w)    return false;
+        const ShapeItem w = get_width(0);
+        loop(h, get_height())
+           {
+             if (get_width(h) != w)    return false;
+           }
       }
 
    return true;
