@@ -135,8 +135,7 @@ const APL_Float qct = Workspace::get_CT();
 const Cell * cA = &A->get_ravel(0);
 
 const Shape shZ(var_count, 4);
-Value_P Z = var_count > 1 ? new Value(shZ, LOC) : new Value(4, LOC);
-Cell * cZ = &Z->get_ravel(0);
+Value_P Z(var_count > 1 ? new Value(shZ, LOC) : new Value(4, LOC));
 
    loop(z, var_count)
       {
@@ -165,19 +164,20 @@ Cell * cZ = &Z->get_ravel(0);
 
         // set control.
         //
-        Symbol * sym = Workspace::the_workspace->lookup_existing_symbol(vars[z]);
+        Symbol * sym = Workspace::lookup_existing_symbol(vars[z]);
         if (sym == 0)   throw_symbol_error(vars[z], LOC);
 
         const SV_key key = sym->get_SV_key();
         ctl = Svar_DB::set_control(key, Svar_Control(ctl));
 
-        new (cZ++)   IntCell(ctl & SET_BY_1 ? 1 : 0);
-        new (cZ++)   IntCell(ctl & SET_BY_2 ? 1 : 0);
-        new (cZ++)   IntCell(ctl & USE_BY_1 ? 1 : 0);
-        new (cZ++)   IntCell(ctl & USE_BY_2 ? 1 : 0);
+        new (Z->next_ravel())   IntCell(ctl & SET_BY_1 ? 1 : 0);
+        new (Z->next_ravel())   IntCell(ctl & SET_BY_2 ? 1 : 0);
+        new (Z->next_ravel())   IntCell(ctl & USE_BY_1 ? 1 : 0);
+        new (Z->next_ravel())   IntCell(ctl & USE_BY_2 ? 1 : 0);
       }
 
-   return CHECK(Z, LOC);
+   Z->check_value(LOC);
+   return Token(TOK_APL_VALUE1, Z);
 }
 //-----------------------------------------------------------------------------
 Token
@@ -190,34 +190,34 @@ vector<UCS_string> vars(var_count);
    B->to_varnames(vars, false);
 
 const Shape shZ(var_count, 4);
-Value_P Z = var_count > 1 ? new Value(shZ, LOC) : new Value(4, LOC);
-Cell * cZ = &Z->get_ravel(0);
+Value_P Z(var_count > 1 ? new Value(shZ, LOC) : new Value(4, LOC));
 
    loop(z, var_count)
       {
-        Symbol * sym = Workspace::the_workspace->lookup_existing_symbol(vars[z]);
+        Symbol * sym = Workspace::lookup_existing_symbol(vars[z]);
         if (sym == 0)   throw_symbol_error(vars[z], LOC);
 
         const SV_key key = sym->get_SV_key();
         const Svar_Control control = Svar_DB::get_control(key);
 
-        new (cZ++)   IntCell(control & SET_BY_1 ? 1 : 0);
-        new (cZ++)   IntCell(control & SET_BY_2 ? 1 : 0);
-        new (cZ++)   IntCell(control & USE_BY_1 ? 1 : 0);
-        new (cZ++)   IntCell(control & USE_BY_2 ? 1 : 0);
+        new (Z->next_ravel())   IntCell(control & SET_BY_1 ? 1 : 0);
+        new (Z->next_ravel())   IntCell(control & SET_BY_2 ? 1 : 0);
+        new (Z->next_ravel())   IntCell(control & USE_BY_1 ? 1 : 0);
+        new (Z->next_ravel())   IntCell(control & USE_BY_2 ? 1 : 0);
       }
 
-   return CHECK(Z, LOC);
+   Z->check_value(LOC);
+   return Token(TOK_APL_VALUE1, Z);
 }
 //=============================================================================
 Quad_SVE::Quad_SVE()
    : NL_SystemVariable(ID_QUAD_SVE)
 {
-   Symbol::assign(&Value::Zero);
+   Symbol::assign(Value::Zero_P, LOC);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_SVE::assign(Value_P value)
+Quad_SVE::assign(Value_P value, const char * loc)
 {
    // ⎕SVE←X merely remembers the timer expiration time. The timer is only
    // started if ⎕SVE is referenced (in Quad_SVE::get_apl_value())
@@ -252,7 +252,7 @@ const APL_time wait = timer_end - current_time;
    if (wait <= 0)
       {
         Svar_DB::clear_all_events();
-        return &Value::Zero;
+        return Value::Zero_P;
       }
 
    // At this point the timer is still running.
@@ -296,15 +296,15 @@ bool got_event = false;
 
    // if no event has occurred (timeout) then return 0
    //
-  if (!got_event)   return &Value::Zero;
+  if (!got_event)   return Value::Zero_P;
 
    // we have got an event; return remaining time.
    //
 const APL_Float remaining = timer_end - now();
 
-   if (remaining < 0)   return &Value::Zero;
+   if (remaining < 0)   return Value::Zero_P;
 
-Value_P Z = new Value(LOC);
+Value_P Z(new Value(LOC));
    new (&Z->get_ravel(0))   FloatCell(0.000001 * remaining);
    return Z;
 }
@@ -327,8 +327,7 @@ vector<UCS_string> surrogates(var_count);
 
    if (A->get_rank() == 1 && A->element_count() != var_count)   LENGTH_ERROR;
 
-Value_P Z = var_count > 1 ? new Value(var_count, LOC) : new Value(LOC);
-Cell * cZ = &Z->get_ravel(0);
+Value_P Z(var_count > 1 ? new Value(var_count, LOC) : new Value(LOC));
 
    loop(z, var_count)
       {
@@ -348,13 +347,13 @@ Cell * cZ = &Z->get_ravel(0);
 
         if (bad_name || vlen == 0)
            {
-             new (cZ++) IntCell(NO_COUPLING);
+             new (Z->next_ravel()) IntCell(NO_COUPLING);
              continue;
            }
 
         // make sure current name class allows ⎕SVO
         //
-        Symbol * sym = Workspace::the_workspace->lookup_symbol(apl_vars[z]);
+        Symbol * sym = Workspace::lookup_symbol(apl_vars[z]);
         assert(sym);
 
         const AP_num proc = AP_num(A->get_ravel(a).get_int_value());
@@ -374,7 +373,7 @@ Cell * cZ = &Z->get_ravel(0);
                     //      << " is already shared" << endl;
 
                     const SV_key key = sym->get_SV_key();
-                    new (cZ++) IntCell(Svar_DB::get_coupling(key));
+                    new (Z->next_ravel()) IntCell(Svar_DB::get_coupling(key));
                   }
                   continue;   // next z
 
@@ -389,17 +388,18 @@ Cell * cZ = &Z->get_ravel(0);
              sym->share_var(key);
            }
 
-        new (cZ++) IntCell(coupling);
+        new (Z->next_ravel()) IntCell(coupling);
       }
 
-   return CHECK(Z, LOC);
+   Z->check_value(LOC);
+   return Token(TOK_APL_VALUE1, Z);
 }
 //-----------------------------------------------------------------------------
 SV_key
 Quad_SVO::share_one_variable(AP_num to_ap, const uint32_t * vname,
                              SV_Coupling & coupling)
 {
-   // if to_ap is a (dependent) AP then start it if neccessary...
+   // if to_ap is a (dependent) AP then start it if necessary...
    //
 AP_num3 to_proc(to_ap, ProcessorID::get_own_ID(), ProcessorID::get_parent_ID());
 bool auto_started = false;
@@ -473,15 +473,14 @@ const ShapeItem var_count = B->get_rows();
 vector<UCS_string> vars(var_count);
    B->to_varnames(vars, false);
 
-Value_P Z = var_count > 1 ? new Value(var_count, LOC) : new Value(LOC);
-Cell * cZ = &Z->get_ravel(0);
+Value_P Z(var_count > 1 ? new Value(var_count, LOC) : new Value(LOC));
 
    loop(z, var_count)
       {
-        Symbol * sym = Workspace::the_workspace->lookup_existing_symbol(vars[z]);
+        Symbol * sym = Workspace::lookup_existing_symbol(vars[z]);
         if (sym == 0)   // variable does not exist
            {
-             new (cZ++) IntCell(0);
+             new (Z->next_ravel()) IntCell(0);
              continue;
            }
 
@@ -489,16 +488,17 @@ Cell * cZ = &Z->get_ravel(0);
         //
         if (sym->get_nc() != NC_SHARED_VAR)
            {
-             new (cZ++) IntCell(0);
+             new (Z->next_ravel()) IntCell(0);
              continue;
            }
 
         const SV_key key = sym->get_SV_key();
         const SV_Coupling coupling = Svar_DB::get_coupling(key);
-         new (cZ++) IntCell(coupling);
+         new (Z->next_ravel()) IntCell(coupling);
       }
 
-   return CHECK(Z, LOC);
+   Z->check_value(LOC);
+   return Token(TOK_APL_VALUE1, Z);
 }
 //=============================================================================
 Token
@@ -521,7 +521,8 @@ Value_P Z;
         Z = get_variables(proc);
       }
 
-   return CHECK(Z, LOC);
+   Z->check_value(LOC);
+   return Token(TOK_APL_VALUE1, Z);
 }
 //-----------------------------------------------------------------------------
 Value_P
@@ -617,7 +618,7 @@ vector<int32_t> sorted;
             }
       }
 
-Value_P Z = new Value(sorted.size(), LOC);
+Value_P Z(new Value(sorted.size(), LOC));
    loop(z, sorted.size())   new (&Z->get_ravel(z)) IntCell(sorted[z]);
    return Z;
 }
@@ -641,17 +642,16 @@ ShapeItem max_len = 0;
       }
 
 const Shape shZ(count, max_len);
-Value_P Z = new Value(shZ, LOC);
-Cell * cZ = &Z->get_ravel(0);
+Value_P Z(new Value(shZ, LOC));
 
    loop(z, count)
    loop(c, max_len)
       {
-        if (c < varlen[z])   new (cZ++) CharCell(Unicode(vars[z][c]));
-        else                 new (cZ++) CharCell(UNI_ASCII_SPACE);
+        if (c < varlen[z])   new (Z->next_ravel()) CharCell(Unicode(vars[z][c]));
+        else                 new (Z->next_ravel()) CharCell(UNI_ASCII_SPACE);
       }
 
-   Z->set_default(&Value::Spc);   // prototype: character
+   Z->set_default(*Value::Spc_P);   // prototype: character
    return Z;
 }
 //=============================================================================
@@ -664,19 +664,19 @@ const ShapeItem var_count = B->get_rows();
 vector<UCS_string> vars(var_count);
    B->to_varnames(vars, false);
 
-Value_P Z = var_count > 1 ? new Value(var_count, LOC) : new Value(LOC);
-Cell * cZ = &Z->get_ravel(0);
+Value_P Z(var_count > 1 ? new Value(var_count, LOC) : new Value(LOC));
 
    loop(z, var_count)
       {
-        Symbol * sym = Workspace::the_workspace->lookup_existing_symbol(vars[z]);
+        Symbol * sym = Workspace::lookup_existing_symbol(vars[z]);
         if (sym == 0)   DOMAIN_ERROR;
 
         const SV_Coupling coupling = sym->unshare_var();
-        new (cZ++) IntCell(coupling);
+        new (Z->next_ravel()) IntCell(coupling);
       }
 
-   return CHECK(Z, LOC);
+   Z->check_value(LOC);
+   return Token(TOK_APL_VALUE1, Z);
 }
 //=============================================================================
 Token
@@ -689,23 +689,23 @@ vector<UCS_string> vars(var_count);
    B->to_varnames(vars, false);
 
 const Shape shZ(var_count, 4);
-Value_P Z = var_count > 1 ? new Value(shZ, LOC) : new Value(4, LOC);
-Cell * cZ = &Z->get_ravel(0);
+Value_P Z(var_count > 1 ? new Value(shZ, LOC) : new Value(4, LOC));
 
    loop(z, var_count)
       {
-        Symbol * sym = Workspace::the_workspace->lookup_existing_symbol(vars[z]);
+        Symbol * sym = Workspace::lookup_existing_symbol(vars[z]);
         if (sym == 0)   throw_symbol_error(vars[z], LOC);
 
         const SV_key key = sym->get_SV_key();
         const Svar_state state = Svar_DB::get_state(key);
 
-        new (cZ++)   IntCell(state & SET_BY_1 ? 1 : 0);
-        new (cZ++)   IntCell(state & SET_BY_2 ? 1 : 0);
-        new (cZ++)   IntCell(state & USE_BY_1 ? 1 : 0);
-        new (cZ++)   IntCell(state & USE_BY_2 ? 1 : 0);
+        new (Z->next_ravel())   IntCell(state & SET_BY_1 ? 1 : 0);
+        new (Z->next_ravel())   IntCell(state & SET_BY_2 ? 1 : 0);
+        new (Z->next_ravel())   IntCell(state & USE_BY_1 ? 1 : 0);
+        new (Z->next_ravel())   IntCell(state & USE_BY_2 ? 1 : 0);
       }
 
-   return CHECK(Z, LOC);
+   Z->check_value(LOC);
+   return Token(TOK_APL_VALUE1, Z);
 }
 //=============================================================================
