@@ -24,6 +24,7 @@
 #include "FloatCell.hh"
 #include "IntCell.hh"
 #include "PointerCell.hh"
+#include "Quad_FX.hh"
 #include "Quad_TF.hh"
 #include "Symbol.hh"
 #include "Tokenizer.hh"
@@ -41,23 +42,24 @@ Quad_TF::eval_AB(Value_P A, Value_P B)
    if (A->element_count() != 1)   LENGTH_ERROR;
 
 const APL_Integer mode = A->get_ravel(0).get_int_value();
-const UCS_string symbol_name(B);
+const UCS_string symbol_name(*B.get());
 
-Value_P Z = 0;
+Value_P Z;
 
    if (mode == 1)        Z = tf1(symbol_name);
    else if (mode == 2)   return tf2(symbol_name);
    else if (mode == 3)   Z = tf3(symbol_name);
    else                  DOMAIN_ERROR;
 
-   return CHECK(Z, LOC);
+   Z->check_value(LOC);
+   return Token(TOK_APL_VALUE1, Z);
 }
 //-----------------------------------------------------------------------------
 Value_P
 Quad_TF::tf1(const UCS_string & name)
 {
-NamedObject * obj = Workspace::the_workspace->lookup_existing_name(name);
-   if (obj == 0)   return &Value::Str0;
+NamedObject * obj = Workspace::lookup_existing_name(name);
+   if (obj == 0)   return Value::Str0_P;
 
 const Function * function = obj->get_function();
 
@@ -66,19 +68,19 @@ const Function * function = obj->get_function();
         if (obj->is_user_defined())   return tf1(name, *function);
 
         // quad function or primitive: return ''
-        return &Value::Str0;
+        return Value::Str0_P;
       }
 
 Symbol * symbol = obj->get_symbol();
    if (symbol)
       {
         Value_P value = symbol->get_apl_value();
-        if (value)   return tf1(name, value);
+        if (!!value)   return tf1(name, value);
 
         /* not a variable: fall through */
       }
 
-   return &Value::Str0;
+   return Value::Str0_P;
 }
 //-----------------------------------------------------------------------------
 Value_P
@@ -88,19 +90,19 @@ Quad_TF::tf1(const UCS_string & var_name, Value_P val)
    //
    {
      Value_P Z = tf1_inv(val);
-     if (Z)   return Z;
+     if (!!Z)   return Z;
    }
 
 const bool is_char_array = val->get_ravel(0).is_character_cell();
 UCS_string ucs(is_char_array ? UNI_ASCII_C : UNI_ASCII_N);
 
-   ucs += var_name;
-   ucs += UNI_ASCII_SPACE;
+   ucs.append(var_name);
+   ucs.append(UNI_ASCII_SPACE);
    ucs.append_number(val->get_rank());   // rank
 
    loop(r, val->get_rank())
       {
-        ucs += UNI_ASCII_SPACE;
+        ucs.append(UNI_ASCII_SPACE);
         ucs.append_number(val->get_shape_item(r));   // shape
       }
 
@@ -108,18 +110,17 @@ const ShapeItem ec = val->element_count();
 
    if (is_char_array)
       {
-        ucs += UNI_ASCII_SPACE;
+        ucs.append(UNI_ASCII_SPACE);
 
         loop(e, ec)
            {
              const Cell & cell = val->get_ravel(e);
              if (!cell.is_character_cell())
                 {
-                  val->erase(LOC);
-                  return  &Value::Str0;
+                  return  Value::Str0_P;
                 }
 
-             ucs += cell.get_char_value();
+             ucs.append(cell.get_char_value());
            }
       }
    else   // number
@@ -127,7 +128,7 @@ const ShapeItem ec = val->element_count();
         const APL_Float qct = Workspace::get_CT();
         loop(e, ec)
            {
-             ucs += UNI_ASCII_SPACE;
+             ucs.append(UNI_ASCII_SPACE);
 
              const Cell & cell = val->get_ravel(e);
              if (cell.is_integer_cell())
@@ -138,13 +139,14 @@ const ShapeItem ec = val->element_count();
                 {
                   PrintContext pctx(PR_APL_MIN, MAX_QUAD_PP, 0.0, MAX_QUAD_PW);
                   const APL_Float value = cell.get_real_value();
-                  ucs +=  FloatCell::format_float_scaled(value, pctx);
+                  bool scaled = true;
+                  UCS_string ucs1(value, scaled, pctx);
+                  ucs.append(ucs1);
                 }
            }
       }
 
-   val->erase(LOC);
-   return new Value(ucs, LOC);
+   return Value_P(new Value(ucs, LOC));
 }
 //-----------------------------------------------------------------------------
 Value_P
@@ -155,57 +157,57 @@ vector<UCS_string> lines;
 const size_t max_len = text.to_vector(lines);
 
 UCS_string ucs;
-   ucs += UNI_ASCII_F;
-   ucs += fun_name;
-   ucs += UNI_ASCII_SPACE;
+   ucs.append(UNI_ASCII_F);
+   ucs.append(fun_name);
+   ucs.append(UNI_ASCII_SPACE);
    ucs.append_number(2);   // rank
-   ucs += UNI_ASCII_SPACE;
+   ucs.append(UNI_ASCII_SPACE);
    ucs.append_number(lines.size());   // rows
-   ucs += UNI_ASCII_SPACE;
+   ucs.append(UNI_ASCII_SPACE);
    ucs.append_number(max_len);        // cols
-   ucs += UNI_ASCII_SPACE;
+   ucs.append(UNI_ASCII_SPACE);
 
    loop(l, lines.size())
       {
-       ucs += lines[l];
-       loop(c, max_len - lines[l].size())   ucs += UNI_ASCII_SPACE;
+       ucs.append(lines[l]);
+       loop(c, max_len - lines[l].size())   ucs.append(UNI_ASCII_SPACE);
       }
 
-   return new Value(ucs, LOC);
+   return Value_P(new Value(ucs, LOC));
 }
 //-----------------------------------------------------------------------------
 Value_P
 Quad_TF::tf1_inv(Value_P val)
 {
-   if (!val->is_char_vector())   return 0;
+   if (!val->is_char_vector())   return Value_P();
 
-UCS_string ravel(val);
+UCS_string ravel(*val.get());
 
 const size_t len = ravel.size();
-   if (len < 2)   return 0;
+   if (len < 2)   return Value_P();
 
    // mode should be 'F', 'N', or 'C'.
 const Unicode mode = ravel[0];
-   if (mode != 'C' && mode != 'F' && mode != 'N')   return 0;
+   if (mode != 'C' && mode != 'F' && mode != 'N')   return Value_P();
 
 UCS_string name(ravel[1]);
-   if (name[0] != UNI_QUAD_QUAD && !Avec::is_first_symbol_char(name[0]))
-      return 0;
+   if (!Avec::is_quad(name[0]) && !Avec::is_first_symbol_char(name[0]))
+      return Value_P();
 
 ShapeItem idx = 2;
-   while (idx < len && Avec::is_symbol_char(ravel[idx]))   name += ravel[idx++];
+   while (idx < len && Avec::is_symbol_char(ravel[idx]))   name.append(ravel[idx++]);
 
-   if (name[0] == UNI_QUAD_QUAD && name.size() == 1)   return 0;
-   if (ravel[idx++] != UNI_ASCII_SPACE)   return 0;
+   if (Avec::is_quad(name[0]) && name.size() == 1)   return Value_P();
+   if (ravel[idx++] != UNI_ASCII_SPACE)   return Value_P();
 
 ShapeItem idx0 = idx;
 Rank rank = 0;
    while (idx < len && Avec::is_digit(ravel[idx]))
       rank = 10 * rank + ravel[idx++] - UNI_ASCII_0;
 
-   if (rank == 0 && idx0 == idx)   return 0;   // no rank
-   if (ravel[idx++] != UNI_ASCII_SPACE)   return 0;
-   if (rank > MAX_RANK)    return 0;
+   if (rank == 0 && idx0 == idx)   return Value_P();   // no rank
+   if (ravel[idx++] != UNI_ASCII_SPACE)   return Value_P();
+   if (rank > MAX_RANK)    return Value_P();
 
 Shape shape;
    loop(r, rank)
@@ -214,62 +216,64 @@ Shape shape;
         ShapeItem sh = 0;
         while (idx < len && Avec::is_digit(ravel[idx]))
            sh = 10 * sh + ravel[idx++] - UNI_ASCII_0;
-        if (sh == 0 && idx0 == idx)   return 0;   // shape
-        if (ravel[idx++]!= UNI_ASCII_SPACE)  return 0;
+        if (sh == 0 && idx0 == idx)   return Value_P();   // shape
+        if (ravel[idx++]!= UNI_ASCII_SPACE)  return Value_P();
         shape.add_shape_item(sh);
       }
 
 Symbol * symbol = 0;
 NameClass nc = NC_UNUSED_USER_NAME;
-NamedObject * sym_or_fun = Workspace::the_workspace->lookup_existing_name(name); 
+NamedObject * sym_or_fun = Workspace::lookup_existing_name(name); 
    if (sym_or_fun)   // existing name
       {
         symbol = sym_or_fun->get_symbol();
         nc = sym_or_fun->get_nc();
-        if (symbol && symbol->is_readonly())   return 0;
+        if (symbol && symbol->is_readonly())   return Value_P();
      }
 
 const size_t data_chars = len - idx;
 
    if (mode == UNI_ASCII_F)   // function
       {
-        if (rank != 2)   return 0;
+        if (rank != 2)   return Value_P();
 
         if (nc != NC_UNUSED_USER_NAME && nc != NC_FUNCTION && nc != NC_OPERATOR)
-           return 0;
+           return Value_P();
 
-        if (data_chars != shape.element_count())   return 0;
+        if (data_chars != shape.get_volume())   return Value_P();
 
-        Value_P new_val = new Value(shape, LOC);
+        Value_P new_val(new Value(shape, LOC));
         loop(d, data_chars)
            new (&new_val->get_ravel(d)) CharCell(ravel[idx + d]);
+        new_val->check_value(LOC);
 
          Token t = Quad_FX::fun.eval_B(new_val);
-         t.get_apl_val()->erase(LOC);
-         new_val->erase(LOC);
       }
    else if (mode == UNI_ASCII_C)   // char array
       {
-        if (data_chars != shape.element_count())   return 0;
-        if (nc != NC_UNUSED_USER_NAME && nc != NC_VARIABLE)   return 0;
+        if (data_chars != shape.get_volume())   return Value_P();
+        if (nc != NC_UNUSED_USER_NAME && nc != NC_VARIABLE)   return Value_P();
 
-        Value_P new_val = new Value(shape, LOC);
+        Value_P new_val(new Value(shape, LOC));
         loop(d, data_chars)
            new (&new_val->get_ravel(d)) CharCell(ravel[idx + d]);
 
-        if (symbol == 0)   symbol = Workspace::the_workspace->lookup_symbol(name);
-        symbol->assign(new_val);
-        new_val->erase(LOC);   // if assign fails
+        new_val->check_value(LOC);
+
+        if (symbol == 0)
+           symbol = Workspace::lookup_symbol(name);
+
+        symbol->assign(new_val, LOC);
       }
    else if (mode == UNI_ASCII_N)   // numeric array
       {
-        if (nc != NC_UNUSED_USER_NAME && nc != NC_VARIABLE)   return 0;
+        if (nc != NC_UNUSED_USER_NAME && nc != NC_VARIABLE)   return Value_P();
 
         UCS_string data(ravel, idx, len - idx);
         Tokenizer tokenizer(PM_EXECUTE, LOC);
         Token_string tos;
-        if (tokenizer.tokenize(data, tos) != E_NO_ERROR)   return 0;
-        if (tos.size() != shape.element_count())           return 0;
+        if (tokenizer.tokenize(data, tos) != E_NO_ERROR)   return Value_P();
+        if (tos.size() != shape.get_volume())           return Value_P();
 
         // check that all token are numeric...
         //
@@ -279,12 +283,12 @@ const size_t data_chars = len - idx;
              if (tag == TOK_INTEGER)   continue;
              if (tag == TOK_REAL)      continue;
              if (tag == TOK_COMPLEX)   continue;
-             return 0;
+             return Value_P();
            }
 
         // at this point, we have a valid inverse 1 ⎕TF.
         //
-        Value_P new_val = new Value(shape, LOC);
+        Value_P new_val(new Value(shape, LOC));
 
         loop(t, tos.size())
            {
@@ -295,47 +299,53 @@ const size_t data_chars = len - idx;
                 new (&new_val->get_ravel(t)) FloatCell(tos[t].get_flt_val());
              else if (tag == TOK_COMPLEX)
                 new (&new_val->get_ravel(t)) ComplexCell(tos[t].get_cpx_real(),
-                                                         tos[t].get_cpx_imag());
+                                                        tos[t].get_cpx_imag());
              else Assert(0);   // since checked above
            }
 
-        if (!symbol)   symbol = Workspace::the_workspace->lookup_symbol(name);
+        new_val->check_value(LOC);
 
-        symbol->assign(new_val);
-        new_val->erase(LOC);   // if assign(0 failed
+        if (!symbol)   symbol = Workspace::lookup_symbol(name);
+
+        symbol->assign(new_val, LOC);
       }
    else Assert(0);   // since checked above
    
-   return new Value(name, LOC);
+   return Value_P(new Value(name, LOC));
 }
 //-----------------------------------------------------------------------------
 Token
 Quad_TF::tf2(const UCS_string & name)
 {
-NamedObject * obj = Workspace::the_workspace->lookup_existing_name(name);
-   if (obj == 0)   return CHECK(&Value::Str0, LOC);
+NamedObject * obj = Workspace::lookup_existing_name(name);
+   if (obj == 0)   return Token(TOK_APL_VALUE1, Value::Str0_P);
 
 const Function * function = obj->get_function();
 
    if (function)
       {
         if (obj->is_user_defined())
-           return CHECK(tf2_fun(name, *function), LOC);
+           {
+             UCS_string ucs = tf2_fun(name, *function);
+             Value_P Z(new Value(ucs, LOC));
+             Z->check_value(LOC);
+             return Token(TOK_APL_VALUE1, Z);
+           }
 
         // quad function or primitive: return ''
-        return CHECK(&Value::Str0, LOC);
+        return Token(TOK_APL_VALUE1, Value::Str0_P);
       }
 
 Symbol * symbol = obj->get_symbol();
    if (symbol)
       {
         Value_P value = symbol->get_apl_value();
-        if (value)   return tf2_var(name, value);
+        if (!!value)   return tf2_var(name, value);
 
         /* not a variable: fall through */
       }
 
-   return CHECK(&Value::Str0, LOC);
+   return Token(TOK_APL_VALUE1, Value::Str0_P);
 }
 //-----------------------------------------------------------------------------
 Token
@@ -345,28 +355,30 @@ Quad_TF::tf2_var(const UCS_string & var_name, Value_P value)
    //
    if (value->is_apl_char_vector())
       {
-        UCS_string data(value);
+        UCS_string data(*value.get());
         UCS_string new_var_or_fun;
 
         tf2_parse(data, new_var_or_fun);
         if (new_var_or_fun.size())   // if data is an inverse TF2
            {
-             value->erase(LOC);
-             Value_P vname = new Value(new_var_or_fun, LOC);
-             return CHECK(vname, LOC);
+             Value_P Z(new Value(new_var_or_fun, LOC));
+             Z->check_value(LOC);
+             return Token(TOK_APL_VALUE1, Z);
           }
       }
 
 UCS_string ucs(var_name);
-   ucs += UNI_LEFT_ARROW;
+   ucs.append(UNI_LEFT_ARROW);
 
 const bool error = tf2_ravel(0, ucs, value);
-   value->erase(LOC);
 
    // return '' for invalid values.
-   if (error)   return CHECK(&Value::Str0, LOC);
+   //
+   if (error)   return Token(TOK_APL_VALUE1, Value::Str0_P);
 
-   return CHECK(new Value(ucs, LOC), LOC);
+Value_P Z(new Value(ucs, LOC));
+   Z->check_value(LOC);
+   return Token(TOK_APL_VALUE1, Z);
 }
 //-----------------------------------------------------------------------------
 void
@@ -402,35 +414,42 @@ Parser parser(PM_EXECUTE, LOC);
         if (tos.size() != 3)                  return;
         if (tos[2].get_Class() != TC_VALUE)   return;
 
-         tos[0].get_sym_ptr()->assign(tos[2].get_apl_val());
+         tos[0].get_sym_ptr()->assign(tos[2].get_apl_val(), LOC);
          new_var_or_fun = tos[0].get_sym_ptr()->get_name();   // valid 2⎕TF
          return;
       }
 
    // Try ⎕FX fun-text
-   if (tos.size() != 2)   return;
+   //
+   if (tos.size() != 2)                   return;
    if (tos[0].get_tag() != TOK_QUAD_FX)   return;
    if (tos[1].get_Class() != TC_VALUE)    return;
 
-const Token tok = Quad_FX::fun.eval_B(tos[1].get_apl_val());
+   {
+     Value_P function_text =  tos[1].get_apl_val();
+     const Token tok = Quad_FX::fun.eval_B(function_text);
 
    if (tok.get_Class() == TC_VALUE)   // ⎕FX successful
-      new_var_or_fun = UCS_string(tok.get_apl_val());
+      {
+        Value_P val = tok.get_apl_val();
+        new_var_or_fun = UCS_string(*val.get());
+      }
+   }
 }
 //-----------------------------------------------------------------------------
 void
 Quad_TF::tf2_shape(UCS_string & ucs, const Shape & shape)
 {
    // dont print anything if shape is a skalar or non-empty vector
-   if (shape.element_count() != 0 && shape.get_rank() <=1)   return;
+   if (shape.get_volume() != 0 && shape.get_rank() <=1)   return;
 
    loop(r, shape.get_rank())
       {
-        if (r)   ucs += UNI_ASCII_SPACE;
+        if (r)   ucs.append(UNI_ASCII_SPACE);
         ucs.append_number(shape.get_shape_item(r));
       }
 
-   ucs += UNI_RHO;
+   ucs.append(UNI_RHO);
 }
 //-----------------------------------------------------------------------------
 bool
@@ -444,30 +463,29 @@ const ShapeItem ec = value->element_count();
       {
         if (value->get_ravel(0).is_character_cell())   // ''
            {
-              ucs += UNI_SINGLE_QUOTE;
-              ucs += UNI_SINGLE_QUOTE;
+              ucs.append(UNI_SINGLE_QUOTE);
+              ucs.append(UNI_SINGLE_QUOTE);
               return  false;
            }
 
         if (value->get_ravel(0).is_numeric())          // ⍳0
            {
-              ucs += UNI_ASCII_L_PARENT;
-              ucs += UNI_ASCII_0;
-              ucs += UNI_RHO;
-              ucs += UNI_ASCII_0;
-              ucs += UNI_ASCII_R_PARENT;
+              ucs.append(UNI_ASCII_L_PARENT);
+              ucs.append(UNI_ASCII_0);
+              ucs.append(UNI_RHO);
+              ucs.append(UNI_ASCII_0);
+              ucs.append(UNI_ASCII_R_PARENT);
               return  false;
            }
 
         // other empty values: print ( shape ⍴ prototype )
-        ucs += UNI_ASCII_L_PARENT;
+        ucs.append(UNI_ASCII_L_PARENT);
 
         tf2_shape(ucs, value->get_shape());
 
         Value_P proto = value->prototype(LOC);
         tf2_ravel(level + 1, ucs, proto);
-        proto->erase(LOC);
-        ucs += UNI_ASCII_R_PARENT;
+        ucs.append(UNI_ASCII_R_PARENT);
         return  false;
       }
 
@@ -491,35 +509,35 @@ const ShapeItem ec = value->element_count();
 
         if (use_UCS)
            {
-             if (level)   ucs += UNI_ASCII_L_PARENT;
-             ucs += UNI_QUAD_QUAD;
-             ucs += UNI_ASCII_U;
-             ucs += UNI_ASCII_C;
-             ucs += UNI_ASCII_S;
+             if (level)   ucs.append(UNI_ASCII_L_PARENT);
+             ucs.append(UNI_QUAD_QUAD);
+             ucs.append(UNI_ASCII_U);
+             ucs.append(UNI_ASCII_C);
+             ucs.append(UNI_ASCII_S);
              loop(e, ec)
                 {
-                  ucs += UNI_ASCII_SPACE;
+                  ucs.append(UNI_ASCII_SPACE);
                   const Unicode uni = value->get_ravel(e).get_char_value();
                   ucs.append_number(uni);
                 }
-             if (level)   ucs += UNI_ASCII_R_PARENT;
+             if (level)   ucs.append(UNI_ASCII_R_PARENT);
            }
         else
            {
-             ucs += UNI_SINGLE_QUOTE;
+             ucs.append(UNI_SINGLE_QUOTE);
              loop(e, ec)
                 {
                   const Unicode uni = value->get_ravel(e).get_char_value();
-                  ucs += uni;
-                  if (uni == UNI_SINGLE_QUOTE)   ucs += UNI_SINGLE_QUOTE;
+                  ucs.append(uni);
+                  if (uni == UNI_SINGLE_QUOTE)   ucs.append(UNI_SINGLE_QUOTE);
                 }
-             ucs += UNI_SINGLE_QUOTE;
+             ucs.append(UNI_SINGLE_QUOTE);
            }
         return  false;
       }
 
 const Depth depth = value->compute_depth();
-   if (depth && level)   ucs += UNI_ASCII_L_PARENT;
+   if (depth && level)   ucs.append(UNI_ASCII_L_PARENT);
 
    // maybe print shape followed by ⍴
    //
@@ -527,7 +545,7 @@ const Depth depth = value->compute_depth();
 
    loop(e, ec)
       {
-        if (e)   ucs += UNI_ASCII_SPACE;
+        if (e)   ucs.append(UNI_ASCII_SPACE);
         const Cell & cell = value->get_ravel(e);
 
         if (cell.is_pointer_cell())
@@ -542,14 +560,19 @@ const Depth depth = value->compute_depth();
         else if (cell.is_complex_cell())
            {
              PrintContext pctx(PR_APL_MIN, MAX_QUAD_PP, 0.0, MAX_QUAD_PW);
-             ucs += FloatCell::format_float_scaled(cell.get_real_value(), pctx);
-             ucs += UNI_ASCII_J;
-             ucs += FloatCell::format_float_scaled(cell.get_imag_value(), pctx);
+             bool scaled = true;
+             UCS_string ucs1(cell.get_real_value(), scaled, pctx);
+             UCS_string ucs2(cell.get_imag_value(), scaled, pctx);
+             ucs.append(ucs1);
+             ucs.append(UNI_ASCII_J);
+             ucs.append(ucs1);
            }
         else if (cell.is_float_cell())
            {
              PrintContext pctx(PR_APL_MIN, MAX_QUAD_PP, 0.0, MAX_QUAD_PW);
-             ucs += FloatCell::format_float_scaled(cell.get_real_value(), pctx);
+             bool scaled = true;
+             UCS_string ucs1(cell.get_real_value(), scaled, pctx);
+             ucs.append(ucs1);
            }
         else
            {
@@ -560,7 +583,7 @@ const Depth depth = value->compute_depth();
            }
       }
 
-   if (depth && level)   ucs += UNI_ASCII_R_PARENT;
+   if (depth && level)   ucs.append(UNI_ASCII_R_PARENT);
 
    return false;
 }
@@ -579,7 +602,7 @@ bool ucs_active = false;
               continue;
            }
 
-        if (d != s)   tos[d] = tos[s];   // dont copy to itself
+        if (d != s)   move_1(tos[d], tos[s], LOC);   // dont copy to itself
 
         if (ucs_active)   // translate
            {
@@ -613,11 +636,11 @@ int d = 0;
              Value_P aval = tos[s].get_apl_val();
              Value_P bval = tos[s + 2].get_apl_val();
              Shape sh(aval, 0, 0);
-             if (sh.element_count() == bval->element_count())   // same shape
+             if (sh.get_volume() == bval->element_count())   // same shape
                 {
-                  tos[d] = Token(TOK_APL_VALUE1, bval);   // grouped value
+                  Token t(TOK_APL_VALUE1, bval);   // grouped value
+                  move_1(tos[d], t, LOC);
                   bval->set_shape(sh);
-                  aval->erase(LOC);
                   progress = true;
                   ++d;
                   s += 2;
@@ -625,8 +648,7 @@ int d = 0;
                 }
              else
                 {
-                  tos[d] = Bif_F12_RHO::fun.do_reshape(sh, bval);
-                  aval->erase(LOC);
+                  move_2(tos[d], Bif_F12_RHO::fun.do_reshape(sh, bval), LOC);
                   progress = true;
                   ++d;
                   s += 2;
@@ -634,7 +656,7 @@ int d = 0;
                 }
            }
 
-        if (d != s)   tos[d] = tos[s];   // dont copy to itself
+        if (d != s)   move_1(tos[d], tos[s], LOC);   // dont copy to itself
         ++d;
       }
 
@@ -657,14 +679,14 @@ int d = 0;
             tos[s + 1].get_Class() == TC_VALUE     &&
             tos[s + 2].get_tag()   == TOK_R_PARENT)   // ( B )
            {
-             tos[d] = tos[s + 1];   // override A of A⍴B
+             move_1(tos[d], tos[s + 1], LOC);   // override A of A⍴B
              progress = true;
              ++d;
              s += 2;
              continue;
            }
 
-        if (d != s)   tos[d] = tos[s];   // dont copy to itself
+        if (d != s)   move_1(tos[d], tos[s], LOC);   // dont copy to itself
         ++d;
       }
 
@@ -679,7 +701,7 @@ int d = 0;
 
    loop(s, tos.size())   // ⍴ must not be first or last
       {
-        if (d != s)   tos[d] = tos[s];   // dont copy to itself
+        if (d != s)   move_1(tos[d], tos[s], LOC);   // dont copy to itself
 
         if (tos[d].get_Class() == TC_VALUE)
            {
@@ -698,13 +720,13 @@ int d = 0;
    return 0;   // no error(s)
 }
 //-----------------------------------------------------------------------------
-Value_P
+UCS_string
 Quad_TF::tf2_fun(const UCS_string & fun_name, const Function & fun)
 {
 UCS_string ucs;
    tf2_fun_ucs(ucs, fun_name, fun);
 
-   return new Value(ucs, LOC);
+   return ucs;
 }
 //-----------------------------------------------------------------------------
 void
@@ -715,22 +737,22 @@ const UCS_string text = fun.canonical(false);
 vector<UCS_string> lines;
    text.to_vector(lines);
 
-   ucs += UNI_QUAD_QUAD;
-   ucs += UNI_ASCII_F;
-   ucs += UNI_ASCII_X;
+   ucs.append(UNI_QUAD_QUAD);
+   ucs.append(UNI_ASCII_F);
+   ucs.append(UNI_ASCII_X);
 
    loop(l, lines.size())
       {
-        ucs += UNI_ASCII_SPACE;
-        ucs += UNI_SINGLE_QUOTE;
+        ucs.append(UNI_ASCII_SPACE);
+        ucs.append(UNI_SINGLE_QUOTE);
 
         loop(c, lines[l].size())
             {
               const Unicode uni = lines[l][c];
-              ucs += uni;
-              if (uni == UNI_SINGLE_QUOTE)   ucs += UNI_SINGLE_QUOTE;   // ''
+              ucs.append(uni);
+              if (uni == UNI_SINGLE_QUOTE)   ucs.append(UNI_SINGLE_QUOTE);
             }
-        ucs += UNI_SINGLE_QUOTE;
+        ucs.append(UNI_SINGLE_QUOTE);
       }
 
 }
@@ -738,31 +760,31 @@ vector<UCS_string> lines;
 Value_P
 Quad_TF::tf3(const UCS_string & name)
 {
-NamedObject * obj = Workspace::the_workspace->lookup_existing_name(name);
-   if (obj == 0)   return &Value::Str0;
+NamedObject * obj = Workspace::lookup_existing_name(name);
+   if (obj == 0)   return Value::Str0_P;
 
    if (obj->get_function())
       {
         // 3⎕TF is not defined for functions.
         //
-        return &Value::Str0;
+        return Value::Str0_P;
       }
 
 Symbol * symbol = obj->get_symbol();
    if (symbol)
       {
         Value_P value = symbol->get_apl_value();
-        if (value)
+        if (!!value)
            {
              CDR_string cdr;
              CDR::to_CDR(cdr, value);
 
-             return new Value(cdr, LOC);
+             return Value_P(new Value(cdr, LOC));
            }
 
         /* not a variable: fall through */
       }
 
-   return &Value::Str0;
+   return Value::Str0_P;
 }
 //-----------------------------------------------------------------------------
