@@ -34,20 +34,20 @@ StateIndicator::StateIndicator(const Executable * exec, StateIndicator * _par)
    : executable(exec),
      safe_execution(false),
      eoc_handler(0),
+     eoc_arg(Value_P()),
      level(_par ? 1 + _par->get_level() : 0),
      error(E_NO_ERROR, LOC),
      current_stack(*this, exec->get_body()),
-     eval_arg_A(0),
-     eval_arg_F(0),
-     eval_arg_X(0),
-     eval_arg_B(0),
      parent(_par)
 {
 }
 //-----------------------------------------------------------------------------
 StateIndicator::~StateIndicator()
 {
-   current_stack.cleanup(LOC);
+   // flush the FIFO. Do that before delete executable so that values
+   // copied directly from the body of the executable are not killed.
+   //
+   current_stack.cleanup();
 
    // some args could come from the body, so we erase them BEFORE the body.
    //
@@ -66,31 +66,25 @@ StateIndicator::~StateIndicator()
 void
 StateIndicator::clear_args(const char * loc)
 {
-   // dont erase twice
+   // discard duplicate args
    //
-   if (eval_arg_B == eval_arg_A)   eval_arg_A = 0;
-   if (eval_arg_B == eval_arg_X)   eval_arg_X = 0;
-   if (eval_arg_A == eval_arg_X)   eval_arg_X = 0;
+   if (eval_arg_B == eval_arg_A)   ptr_clear(eval_arg_A, loc);
+   if (eval_arg_B == eval_arg_X)   ptr_clear(eval_arg_X, loc);
+   if (eval_arg_A == eval_arg_X)   ptr_clear(eval_arg_X, loc);
 
-   if (eval_arg_A)
+   if (!!eval_arg_A)
       {
-        eval_arg_A->clear_arg();
-        eval_arg_A->erase(loc);
-        eval_arg_A = 0;
+        ptr_clear(eval_arg_A, loc);
       }
 
-   if (eval_arg_B)
+   if (!!eval_arg_B)
       {
-        eval_arg_B->clear_arg();
-        eval_arg_B->erase(loc);
-        eval_arg_B = 0;
+        ptr_clear(eval_arg_B, loc);
       }
 
-   if (eval_arg_X)
+   if (!!eval_arg_X)
       {
-        eval_arg_X->clear_arg();
-        eval_arg_X->erase(loc);
-        eval_arg_X = 0;
+        ptr_clear(eval_arg_X, loc);
       }
 
    eval_arg_F = 0;
@@ -102,31 +96,16 @@ StateIndicator::set_args(Token * A, Token & F, Token * X, Token * B)
    if (A)
       {
         eval_arg_A = A->get_apl_val();
-        if (!eval_arg_A->is_arg())   eval_arg_A->set_arg();
-      }
-   else
-      {
-        eval_arg_A = 0;
       }
 
    if (X)
       {
         eval_arg_X = X->get_axes();
-        if (!eval_arg_X->is_arg())   eval_arg_X->set_arg();
-      }
-   else
-      {
-        eval_arg_X = 0;
       }
 
    if (B)
       {
         eval_arg_B = B->get_apl_val();
-        if (!eval_arg_B->is_arg())   eval_arg_B->set_arg();
-      }
-   else
-      {
-        eval_arg_B = 0;
       }
 
    eval_arg_F = F.get_function();
@@ -158,7 +137,7 @@ StateIndicator::retry(const char * loc)
         return;
       }
 
-   if (eval_arg_B == 0)
+   if (!eval_arg_B)
       {
         CERR << endl << "Can't RETRY: no right argument" << endl << endl;
         return;
@@ -168,20 +147,20 @@ Token Z;
 Token B(TOK_APL_VALUE1, eval_arg_B);
 int phrase_len = 1;
 
-   if (eval_arg_A)   // dyadic
+   if (!!eval_arg_A)   // dyadic
       {
         Token A(TOK_APL_VALUE1, eval_arg_A);
         Token F2(TOK_FUN2, eval_arg_F);
 
-        if (eval_arg_X)   // dyadic with axis
+        if (!!eval_arg_X)   // dyadic with axis
            {
              Token X(TOK_APL_VALUE1, eval_arg_X);
-             Z = eval_AXB(A, F2, X, B);
+             move_2(Z, eval_AXB(A, F2, X, B), LOC);
              phrase_len = 4;
            }
         else              // dyadic without axis
            {
-             Z = eval_AB(A, F2, B);
+             move_2(Z, eval_AB(A, F2, B), LOC);
              phrase_len = 3;
            }
       }
@@ -189,15 +168,15 @@ int phrase_len = 1;
       {
         Token F1(TOK_FUN1, eval_arg_F);
 
-        if (eval_arg_X)   // monadic with axis
+        if (!!eval_arg_X)   // monadic with axis
            {
              Token X(TOK_APL_VALUE1, eval_arg_X);
-             Z = eval_XB(F1, X, B);
+             move_2(Z, eval_XB(F1, X, B), LOC);
              phrase_len = 3;
            }
         else              // monadic without axis
            {
-             Z = eval_B(F1, B);
+             move_2(Z, eval_B(F1, B), LOC);
              phrase_len = 2;
            }
       }
@@ -223,14 +202,14 @@ StateIndicator::function_name() const
         case PM_STATEMENT_LIST:
              {
                UCS_string ret;
-               ret += UNI_DIAMOND;
+               ret.append(UNI_DIAMOND);
                return ret;
              }
 
         case PM_EXECUTE:
              {
                UCS_string ret;
-               ret += UNI_EXECUTE;
+               ret.append(UNI_EXECUTE);
                return ret;
              }
       }
@@ -284,7 +263,10 @@ void
 StateIndicator::clear(ostream & out)
 {
 const UserFunction * ufun = get_executable()->get_ufun();
-   if (ufun)   ufun->pop_local_vars();
+   if (ufun)
+      {
+         Value_P Z = ufun->pop_local_vars();
+      }
 }
 //-----------------------------------------------------------------------------
 void
@@ -391,8 +373,8 @@ const UserFunction * ufun = executable->get_ufun();
         else
            {
              err.error_message_2 = ufun->get_name_and_line(get_PC());
-             err.error_message_2 += UNI_ASCII_SPACE;
-             err.error_message_2 += UNI_ASCII_SPACE;
+             err.error_message_2.append(UNI_ASCII_SPACE);
+             err.error_message_2.append(UNI_ASCII_SPACE);
            }
       }
    else
@@ -410,9 +392,10 @@ const UserFunction * ufun = executable->get_ufun();
         executable->get_ufun()->set_locked_error_info(err);
       }
    else
-      { 
-        const Function_PC2 error_range(current_stack.get_range_low(),
-                                       current_stack.get_range_high());
+      {
+        const Function_PC from = current_stack.get_range_low();
+        Function_PC to   = current_stack.get_range_high();
+        const Function_PC2 error_range(from, to);
         // current_stack.print_range(CERR);
         // CERR  << endl;
         executable->set_error_info(err, error_range);
@@ -420,9 +403,9 @@ const UserFunction * ufun = executable->get_ufun();
 
    // print error, unless we are in safe execution mode.
    //
-   if (!safe_execution)   err.print_em(COUT, LOC);
+   if (!safe_execution)   err.print_em(UERR, LOC);
 
-   Workspace::the_workspace->update_EM_ET(err);   // update ⎕EM and ⎕ET
+   Workspace::update_EM_ET(err);   // update ⎕EM and ⎕ET
 
    error = err;
 }
@@ -487,7 +470,6 @@ const UserFunction * ufun = get_executable()->get_ufun();
    if (ufun)
       {
         Value_P Z = ufun->pop_local_vars();
-        if (Z)   Z->erase(LOC);
       }
 }
 //-----------------------------------------------------------------------------
@@ -505,9 +487,9 @@ Token result = current_stack.reduce_statements();
 void
 StateIndicator::unmark_all_values() const
 {
-   if (eval_arg_A)   eval_arg_A->clear_marked();
-   if (eval_arg_X)   eval_arg_X->clear_marked();
-   if (eval_arg_B)   eval_arg_B->clear_marked();
+   if (!!eval_arg_A)   eval_arg_A->clear_marked();
+   if (!!eval_arg_X)   eval_arg_X->clear_marked();
+   if (!!eval_arg_B)   eval_arg_B->clear_marked();
 
    Assert(executable);
    executable->unmark_all_values();
@@ -516,17 +498,17 @@ StateIndicator::unmark_all_values() const
 }
 //-----------------------------------------------------------------------------
 int
-StateIndicator::show_owners(ostream & out, Value_P value) const
+StateIndicator::show_owners(ostream & out, const Value & value) const
 {
 int count = 0;
 
-   if (value == eval_arg_A)
+   if (&value == eval_arg_A.get())
       { out << "    SI[" << level << "] eval_arg_A" << endl;   ++count; }
 
-   if (value == eval_arg_X)
+   if (&value == eval_arg_X.get())
       { out << "    SI[" << level << "] eval_arg_X" << endl;   ++count; }
 
-   if (value == eval_arg_B)
+   if (&value == eval_arg_B.get())
       { out << "    SI[" << level << "] eval_arg_B" << endl;   ++count; }
 
    Assert(executable);
@@ -552,8 +534,8 @@ StateIndicator::info(ostream & out, const char * loc) const
 Value_P
 StateIndicator::get_L() const
 {
-   if (eval_arg_F == 0)                 return 0;   // no function
-   if (eval_arg_F->is_user_defined())   return 0;   // user defined function
+   if (eval_arg_F == 0)                 return Value_P();   // no function
+   if (eval_arg_F->is_user_defined())   return Value_P();   // user defined function
    return eval_arg_A;
 }
 //-----------------------------------------------------------------------------
@@ -561,18 +543,15 @@ void
 StateIndicator::set_L(Value_P new_value)
 {
 Value_P old_value = eval_arg_A;
-   old_value->clear_arg();
-   old_value->erase(LOC);
 
    eval_arg_A = new_value;
-   new_value->set_arg();
 }
 //-----------------------------------------------------------------------------
 Value_P
 StateIndicator::get_X() const
 {
-   if (eval_arg_F == 0)                 return 0;   // no function
-   if (eval_arg_F->is_user_defined())   return 0;   // user defined function
+   if (eval_arg_F == 0)                 return Value_P();   // no function
+   if (eval_arg_F->is_user_defined())   return Value_P();   // user defined function
    return eval_arg_X;
 }
 //-----------------------------------------------------------------------------
@@ -580,18 +559,15 @@ void
 StateIndicator::set_X(Value_P new_value)
 {
 Value_P old_value = eval_arg_X;
-   old_value->clear_arg();
-   old_value->erase(LOC);
 
    eval_arg_X = new_value;
-   new_value->set_arg();
 }
 //-----------------------------------------------------------------------------
 Value_P
 StateIndicator::get_R() const
 {
-   if (eval_arg_F == 0)                 return 0;   // no function
-   if (eval_arg_F->is_user_defined())   return 0;   // user defined function
+   if (eval_arg_F == 0)                 return Value_P();   // no function
+   if (eval_arg_F->is_user_defined())   return Value_P();   // user defined function
    return eval_arg_B;
 }
 //-----------------------------------------------------------------------------
@@ -599,11 +575,8 @@ void
 StateIndicator::set_R(Value_P new_value)
 {
 Value_P old_value = eval_arg_B;
-   old_value->clear_arg();
-   old_value->erase(LOC);
 
    eval_arg_B = new_value;
-   new_value->set_arg();
 }
 //-----------------------------------------------------------------------------
 bool
@@ -630,14 +603,30 @@ bool success = false;
         success = true;
       }
 
-   if (success)
-      {
-        old_value->clear_arg();
-        new_value->set_arg();
-      }
-
    return success;
 }
+//-----------------------------------------------------------------------------
+void
+StateIndicator::recover_from_error(const char * loc)
+{
+   Value::finish_incomplete(loc);
+}
+//-----------------------------------------------------------------------------
+/// a macro for the common part of all StateIndicator::eval_XXX() calls
+/// catch exceptions and check for errors,
+#define FINISH_EVAL \
+   catch (Error e)                              \
+      {                                         \
+       recover_from_error(LOC);                 \
+       return Token(TOK_ERROR, e.error_code);   \
+      }                                         \
+   catch (...)                                  \
+      {                                         \
+        FIXME;                                  \
+      }                                         \
+                                                \
+   return Token(TOK_ERROR, error.error_code);
+
 //-----------------------------------------------------------------------------
 Token
 StateIndicator::eval_(Token & fun)
@@ -654,9 +643,8 @@ StateIndicator::eval_(Token & fun)
              return result;
            }
       }
-   catch (...)   { if (safe_execution)   Value::erase_stale(LOC); }
 
-   return Token(TOK_ERROR, error.error_code);
+   FINISH_EVAL
 }
 //-----------------------------------------------------------------------------
 Token
@@ -674,9 +662,8 @@ StateIndicator::eval_B(Token & fun, Token & B)
              return result;
            }
       }
-   catch (...)   { if (safe_execution)   Value::erase_stale(LOC); }
 
-   return Token(TOK_ERROR, error.error_code);
+   FINISH_EVAL
 }
 //-----------------------------------------------------------------------------
 Token
@@ -694,9 +681,8 @@ StateIndicator::eval_AB(Token & A, Token & fun, Token & B)
              return result;
            }
       }
-   catch (...)   { if (safe_execution)   Value::erase_stale(LOC); }
 
-   return Token(TOK_ERROR, error.error_code);
+   FINISH_EVAL
 }
 //-----------------------------------------------------------------------------
 Token
@@ -714,9 +700,8 @@ StateIndicator::eval_XB(Token & fun, Token & X, Token & B)
              return result;
            }
       }
-   catch (...)   { if (safe_execution)   Value::erase_stale(LOC); }
 
-   return Token(TOK_ERROR, error.error_code);
+   FINISH_EVAL
 }
 //-----------------------------------------------------------------------------
 Token
@@ -735,9 +720,8 @@ StateIndicator::eval_AXB(Token & A, Token & fun, Token & X, Token & B)
              return result;
            }
       }
-   catch (...)   { if (safe_execution)   Value::erase_stale(LOC); }
 
-   return Token(TOK_ERROR, error.error_code);
+   FINISH_EVAL
 }
 //-----------------------------------------------------------------------------
 Token
@@ -754,9 +738,8 @@ StateIndicator::eval_LB(Token & LO, Token & oper, Token & B)
              return result;
            }
       }
-   catch (...)   { if (safe_execution)   Value::erase_stale(LOC); }
 
-   return Token(TOK_ERROR, error.error_code);
+   FINISH_EVAL
 }
 //-----------------------------------------------------------------------------
 Token
@@ -773,9 +756,8 @@ StateIndicator::eval_ALB(Token & A, Token & LO, Token & oper, Token & B)
              return result;
            }
       }
-   catch (...)   { if (safe_execution)   Value::erase_stale(LOC); }
 
-   return Token(TOK_ERROR, error.error_code);
+   FINISH_EVAL
 }
 //-----------------------------------------------------------------------------
 Token
@@ -792,9 +774,8 @@ StateIndicator::eval_LXB(Token & LO, Token & oper, Token & X, Token & B)
              return result;
            }
       }
-   catch (...)   { if (safe_execution)   Value::erase_stale(LOC); }
 
-   return Token(TOK_ERROR, error.error_code);
+   FINISH_EVAL
 }
 //-----------------------------------------------------------------------------
 Token
@@ -811,9 +792,8 @@ StateIndicator::eval_LRB(Token & LO, Token & oper, Token & RO, Token & B)
              return result;
            }
       }
-   catch (...)   { if (safe_execution)   Value::erase_stale(LOC); }
 
-   return Token(TOK_ERROR, error.error_code);
+   FINISH_EVAL
 }
 //-----------------------------------------------------------------------------
 Token
@@ -832,9 +812,8 @@ StateIndicator::eval_ALXB(Token & A, Token & LO, Token & oper,
              return result;
            }
       }
-   catch (...)   { if (safe_execution)   Value::erase_stale(LOC); }
 
-   return Token(TOK_ERROR, error.error_code);
+   FINISH_EVAL
 }
 //-----------------------------------------------------------------------------
 Token
@@ -853,9 +832,8 @@ StateIndicator::eval_ALRB(Token & A, Token & LO, Token & oper,
              return result;
            }
       }
-   catch (...)   { if (safe_execution)   Value::erase_stale(LOC); }
 
-   return Token(TOK_ERROR, error.error_code);
+   FINISH_EVAL
 }
 //-----------------------------------------------------------------------------
 Token
@@ -874,9 +852,8 @@ StateIndicator::eval_LRXB(Token & LO, Token & oper,
              return result;
            }
       }
-   catch (...)   { if (safe_execution)   Value::erase_stale(LOC); }
 
-   return Token(TOK_ERROR, error.error_code);
+   FINISH_EVAL
 }
 //-----------------------------------------------------------------------------
 Token
@@ -896,9 +873,8 @@ StateIndicator::eval_ALRXB(Token & A, Token & LO, Token & oper,
              return result;
            }
       }
-   catch (...)   { if (safe_execution)   Value::erase_stale(LOC); }
 
-   return Token(TOK_ERROR, error.error_code);
+   FINISH_EVAL
 }
 //-----------------------------------------------------------------------------
 Function_Line
@@ -924,18 +900,16 @@ StateIndicator::statement_result(Token & result)
    if (result.get_ValueType() == TV_VAL)
       {
         const TokenTag tag = result.get_tag();
-        Value_P B = result.get_apl_val();
-        Assert(B);
+        Value_P B(result.get_apl_val());
+        Assert(!!B);
 
         // print TOK_APL_VALUE and TOK_APL_VALUE1, but not TOK_APL_VALUE2
         //
-        if (tag == TOK_APL_VALUE || tag == TOK_APL_VALUE1)
+        if (tag == TOK_APL_VALUE1 || tag == TOK_APL_VALUE3)
            {
              Quad_QUOTE::done(false, LOC);
              B->print(COUT);
            }
-
-        B->erase(LOC);
       }
 }
 //-----------------------------------------------------------------------------
