@@ -20,6 +20,7 @@
 
 #include <errno.h>
 #include <fcntl.h>           /* For O_* constants */
+#include <signal.h>
 #include <string.h>
 #include <stdio.h>
 #include <sys/mman.h>
@@ -69,11 +70,7 @@ Svar_partner::pid_alive(pid_t p)
 {
    if (p == 0)   return false;   // no pid
 
-char proc_file[40];
-   snprintf(proc_file, sizeof(proc_file), "/proc/%u", p);
-
-struct stat st;
-   return (stat(proc_file, &st) == 0);
+   return kill(p, 0) == 0;
 }
 //-----------------------------------------------------------------------------
 ostream &
@@ -307,7 +304,6 @@ usleep(50000);
 
    // the control vector as seen by the offering side
    //
-const Svar_state old_state = state;
 const int control = offering.get_control() | accepting.get_control();
 Svar_event event = SVE_NO_EVENTS;
 uint16_t peer_port = 0;
@@ -457,7 +453,7 @@ Svar_DB_memory::register_processor(const Svar_partner & p)
 {
    // if processor is already (i.e. still) registered, update it.
    //
-   for (int a = 0; a < MAX_ACTIVE_PROCESSORS; ++a)
+   for (int a = 0; a < MAX_ACTIVE_PROCS; ++a)
        {
          Svar_partner_events & slot = active_processors[a];
 
@@ -474,7 +470,7 @@ Svar_DB_memory::register_processor(const Svar_partner & p)
 
    // new processor: find empty slot and insert
    //
-   for (int a = 0; a < MAX_ACTIVE_PROCESSORS; ++a)
+   for (int a = 0; a < MAX_ACTIVE_PROCS; ++a)
        {
          Svar_partner_events & slot = active_processors[a];
          if (slot.partner.id.proc == AP_NULL)   // free slot
@@ -495,7 +491,7 @@ Svar_DB_memory::register_processor(const Svar_partner & p)
 Svar_partner_events
 Svar_DB_memory::get_registered(const AP_num3 & id) const
 {
-   for (int a = 0; a < MAX_ACTIVE_PROCESSORS; ++a)
+   for (int a = 0; a < MAX_ACTIVE_PROCS; ++a)
        {
          const Svar_partner_events & slot = active_processors[a];
          if (slot.partner.id == id)   return slot;
@@ -509,7 +505,7 @@ Svar_partner_events svp;
 bool
 Svar_DB_memory::is_unused_id(AP_num id) const
 {
-   for (int a = 0; a < MAX_ACTIVE_PROCESSORS; ++a)
+   for (int a = 0; a < MAX_ACTIVE_PROCS; ++a)
        {
          const Svar_partner_events & slot = active_processors[a];
          if (slot.partner.id.proc   == id)   return false;
@@ -539,7 +535,7 @@ Svar_DB_memory::get_unused_id() const
 uint16_t
 Svar_DB_memory::get_udp_port(AP_num proc, AP_num parent) const
 {
-   for (int a = 0; a < MAX_ACTIVE_PROCESSORS; ++a)
+   for (int a = 0; a < MAX_ACTIVE_PROCS; ++a)
        {
          const Svar_partner_events & slot = active_processors[a];
          if (slot.partner.id.proc == proc &&
@@ -554,7 +550,7 @@ Svar_DB_memory::unregister_processor(const Svar_partner & p)
 {
    retract_all(p.id);
 
-   for (int a = 0; a < MAX_ACTIVE_PROCESSORS; ++a)
+   for (int a = 0; a < MAX_ACTIVE_PROCS; ++a)
        {
          Svar_partner_events & slot = active_processors[a];
          if (slot.partner.id == p.id)   // found
@@ -568,7 +564,7 @@ Svar_DB_memory::unregister_processor(const Svar_partner & p)
 
    // send DISCONNECT to dependent processors
    //
-   for (int a = 0; a < MAX_ACTIVE_PROCESSORS; ++a)
+   for (int a = 0; a < MAX_ACTIVE_PROCS; ++a)
        {
          const Svar_partner_events & slot = active_processors[a];
          if (slot.partner.id.parent == p.id.proc &&
@@ -590,7 +586,7 @@ Svar_DB_memory::remove_stale()
    // don't use CERR here since CERR may not yet have been initialized yet.
    //
 int stale_procs = 0;
-   for (int a = 0; a < MAX_ACTIVE_PROCESSORS; ++a)
+   for (int a = 0; a < MAX_ACTIVE_PROCS; ++a)
        {
          Svar_partner_events & sp = active_processors[a];
          if (sp.partner.pid_alive())   continue;
@@ -798,7 +794,7 @@ Svar_DB::print(ostream & out)
    out << "┌───────────┬─────┬─────┬──┐" << endl
        << "│ Proc, par │ PID │ Port│Fl│" << endl
        << "╞═══════════╪═════╪═════╪══╡" << endl;
-   for (int p = 0; p < MAX_ACTIVE_PROCESSORS; ++p)
+   for (int p = 0; p < MAX_ACTIVE_PROCS; ++p)
        {
          const Svar_partner_events & sp =
                                     the_Svar_DB.DB_memory->active_processors[p];
@@ -961,7 +957,7 @@ const AP_num3 & caller_id = ProcessorID::get_id();
 
    // clear event bits of calling partner
    //
-   for (int a = 0; a < MAX_ACTIVE_PROCESSORS; ++a)
+   for (int a = 0; a < MAX_ACTIVE_PROCS; ++a)
        {
          Svar_partner_events & slot = active_processors[a];
 
@@ -1029,7 +1025,7 @@ const AP_num3 & caller_id = ProcessorID::get_id();
 
    // no events remaining: clear global event bits.
    //
-   for (int a = 0; a < MAX_ACTIVE_PROCESSORS; ++a)
+   for (int a = 0; a < MAX_ACTIVE_PROCS; ++a)
        {
          Svar_partner_events & slot = active_processors[a];
 
@@ -1040,7 +1036,7 @@ const AP_num3 & caller_id = ProcessorID::get_id();
 SV_key
 Svar_DB_memory::get_events(Svar_event & events, AP_num3 proc) const
 {
-   for (int a = 0; a < MAX_ACTIVE_PROCESSORS; ++a)
+   for (int a = 0; a < MAX_ACTIVE_PROCS; ++a)
        {
          const Svar_partner_events & slot = active_processors[a];
 
@@ -1070,7 +1066,7 @@ Svar_DB_memory::add_event(Svar_event event, AP_num3 proc, SV_key key)
 {
    // set event bit for proc
    //
-   for (int a = 0; a < MAX_ACTIVE_PROCESSORS; ++a)
+   for (int a = 0; a < MAX_ACTIVE_PROCS; ++a)
        {
          Svar_partner_events & slot = active_processors[a];
 
@@ -1250,7 +1246,6 @@ Svar_DB_memory::create_offer(const uint32_t * UCS_varname,
    // is non-general (i.e. 'to' is a specific processor) then to should
    // get an offer mismatch event.
    //
-const bool mismatch = to.proc > AP_NULL;
 
    // find free entry and insert the offer
    //
