@@ -83,8 +83,8 @@ Symbol::print_verbose(ostream & out) const
 
               case NC_VARIABLE:
                    {
-                      Value_P val = item.sym_val.value;
-                      out << "Variable at " << val << endl;
+                      Value_P val = item.apl_val;
+                      out << "Variable at " << (const void *)val.get() << endl;
                       val->print_properties(out, 8);
                       out << endl;
                    }
@@ -107,34 +107,40 @@ Symbol::print_verbose(ostream & out) const
 }
 //-----------------------------------------------------------------------------
 void
-Symbol::assign(Value_P new_value)
+Symbol::assign(Value_P new_value, const char * loc)
 {
    Assert(value_stack.size());
+
+#if 0 // TODO: make this work
+   if (!new_value->is_complete())
+      {
+        CERR << "Incomplete value at " LOC << endl;
+        new_value->print_properties(CERR, 0);
+        Assert(0);
+      }
+#endif
+
 ValueStackItem & vs = value_stack.back();
 
    switch(vs.name_class)
       {
         case NC_UNUSED_USER_NAME:
-             new_value = new_value->clone_if_owned(LOC);
+             new_value = new_value->clone(loc);
+
              vs.name_class = NC_VARIABLE;
-             vs.sym_val.value = new_value;
-             new_value->set_assigned();
+             vs.apl_val = new_value;
              return;
 
         case NC_VARIABLE:
-             if (vs.sym_val.value == new_value)   return;   // X←X
+             if (vs.apl_val == new_value)   return;   // X←X
 
-             new_value = new_value->clone_if_owned(LOC);
+             new_value = new_value->clone(loc);
 
              // un-assign and erase old value
              //
-             vs.sym_val.value->clear_assigned();
-             if (vs.sym_val.value->is_arg())   Workspace::the_workspace
-                                    ->replace_arg(vs.sym_val.value, new_value);
-             vs.sym_val.value->erase(LOC);
+             Workspace::replace_arg(vs.apl_val, new_value);
 
-             vs.sym_val.value = new_value;
-             new_value->set_assigned();
+             vs.apl_val = new_value;
              return;
 
         case NC_SHARED_VAR:
@@ -179,9 +185,9 @@ ValueStackItem & vs = value_stack.back();
                    }
 
                // update shared var state (BEFORE sending request to peer)
-               svar->set_state(false, LOC);
+               svar->set_state(false, loc);
                {
-                 UdpClientSocket sock(LOC, svar->data_owner_port());
+                 UdpClientSocket sock(loc, svar->data_owner_port());
                  ASSIGN_VALUE_c request(sock, get_SV_key(), data);
 
                  uint8_t buffer[Signal_base::get_class_size()];
@@ -208,7 +214,7 @@ ValueStackItem & vs = value_stack.back();
                                 << endl;
                          }
 
-                      throw_apl_error(ec, LOC);
+                      throw_apl_error(ec, loc);
                     }
                }
              }
@@ -221,12 +227,28 @@ ValueStackItem & vs = value_stack.back();
 void
 Symbol::assign_indexed(Value_P X, Value_P B)   // A[X] ← B
 {
+   // this function is called for A[X}←B when X is one-dimensional, i.e.
+   // an index with no semicolons. If X contains semicolons, then
+   // assign_indexed(IndexExpr IX, ...) is called instead.
+   // 
 Value_P A = get_apl_value();
-   if (A->get_rank() != 1)   INDEX_ERROR;
+   if (A->get_rank() != 1)   RANK_ERROR;
+
+const ShapeItem max_idx = A->element_count();
+   if (!X)   // X[] ← B
+      {
+        const Cell & src = B->get_ravel(0);
+        loop(a, max_idx)
+            {
+              Cell & dest = A->get_ravel(a);
+              dest.release(LOC);   // free sub-values etc (if any)
+              dest.init(src);
+            }
+        return;
+      }
 
 const ShapeItem ec_B = B->element_count();
 const ShapeItem ec_X = X->element_count();
-const ShapeItem max_idx = A->element_count();
 const APL_Integer qio = Workspace::get_IO();
 const APL_Float qct = Workspace::get_CT();
 const int incr_B = (ec_B == 1) ? 0 : 1;   // maybe skalar extend B
@@ -241,12 +263,7 @@ const Cell * cB = &B->get_ravel(0);
         if (idx < 0)          INDEX_ERROR;
         if (idx >= max_idx)   INDEX_ERROR;
         Cell & dest = A->get_ravel(idx);
-        if (dest.is_pointer_cell())   // overriding a sub-value
-           {
-             Value_P old_val = dest.get_pointer_value();
-             old_val->clear_nested();
-             old_val->erase(LOC);
-           }
+        dest.release(LOC);   // free sub-values etc (if any)
         dest.init(*cB);
 
          cB += incr_B;
@@ -256,7 +273,7 @@ const Cell * cB = &B->get_ravel(0);
 void
 Symbol::assign_indexed(const IndexExpr & IX, Value_P B)   // A[IX;...] ← B
 {
-   if (IX.value_count() == 1 && IX.values[0])   // one-dimensional index
+   if (IX.value_count() == 1 && !!IX.values[0])   // one-dimensional index
       {
          assign_indexed(IX.values[0], B);
         return;
@@ -267,6 +284,55 @@ Symbol::assign_indexed(const IndexExpr & IX, Value_P B)   // A[IX;...] ← B
 Value_P A = get_apl_value();
    if (A->get_rank() != IX.value_count())   INDEX_ERROR;
 
+   // B must either B a skalar (and is then skalar extended to the size
+   // of the updated area, or else have the shape of the concatenated index
+   // items for example:
+   //
+   //  X:   X1    ; X2    ; X3
+   //  ⍴B:  b1 b2   b3 b4   b5 b6
+   //
+   if (1 && !B->is_skalar())
+      {
+        // remove dimensions with len 1 from X's and B's shapes...
+        // if we see an empty Xn then we return.
+        //
+        Shape B1;
+        loop(b, B->get_rank())
+           {
+             const ShapeItem sb = B->get_shape_item(b);
+             if (sb != 1)   B1.add_shape_item(sb);
+           }
+
+        Shape IX1;
+        loop(ix, IX.value_count())
+            {
+              const Value * ix_val = IX.values[ix].get();
+              if (ix_val)   // normal index
+                 {
+                   if (ix_val->element_count() == 0)   return;   // empty index
+                   loop(xx, ix_val->get_rank())
+                      {
+                        const ShapeItem sxx = ix_val->get_shape_item(xx);
+                        if (sxx != 1)   IX1.add_shape_item(sxx);
+                      }
+                 }
+               else     // elided index: add corresponding B dimenssion
+                 {
+                   if (ix >= A->get_rank())   RANK_ERROR;
+                   const ShapeItem sbx =
+                         A->get_shape_item(A->get_rank() - ix - 1);
+                   if (sbx == 0)   return;   // empty index
+                   if (sbx != 1)   IX1.add_shape_item(sbx);
+                 }
+            }
+
+        if (B1 != IX1)
+           {
+             if (B1.get_rank() != IX1.get_rank())   RANK_ERROR;
+             LENGTH_ERROR;
+           }
+      }
+
 MultiIndexIterator mult(A->get_shape(), IX);
 
 const ShapeItem ec_B = B->element_count();
@@ -276,12 +342,7 @@ const int incr_B = (ec_B == 1) ? 0 : 1;
    while (!mult.done())
       {
         Cell & dest = A->get_ravel(mult.next());
-        if (dest.is_pointer_cell())   // overriding a sub-value
-           {
-             Value_P old_val = dest.get_pointer_value();
-             old_val->clear_nested();
-             old_val->erase(LOC);
-           }
+        dest.release(LOC);   // free sub-values etc (if any)
         dest.init(*cB);
         cB += incr_B;
      }
@@ -290,32 +351,37 @@ const int incr_B = (ec_B == 1) ? 0 : 1;
 Value_P
 Symbol::pop(bool erase)
 {
-   Log(LOG_SYMBOL_push_pop)
-      {
-        CERR << "-pop " << symbol;
-        if (value_stack.size() == 1)   CERR << " (last)";
-        CERR << endl;
-      }
-
    Assert(value_stack.size());
 const ValueStackItem & vs = value_stack.back();
 
-Value_P ret = 0;
-
    if (vs.name_class == NC_VARIABLE)
       {
-        ret = vs.sym_val.value;
-        ret->clear_assigned();
-        if (erase)
+        Value_P ret = vs.apl_val;
+        Log(LOG_SYMBOL_push_pop)
            {
-             ret->erase(LOC);
-             ret = 0;
+             CERR << "-pop-value " << symbol;
+             if (erase)   CERR  << " (and erase) ";
+             CERR << " flags " << ret->get_flags() << " ";
+             if (value_stack.size() == 0)   CERR << " (last)";
+             CERR << " addr " << (const void *)ret.get() << endl;
            }
+        if (erase)   ptr_clear(ret, LOC);
+
+        value_stack.pop_back();
+        return ret;
+      }
+   else
+      {
+        Log(LOG_SYMBOL_push_pop)
+           {
+             CERR << "-pop " << symbol
+                  << " name_class " << vs.name_class << " ";
+             if (value_stack.size() == 0)   CERR << " (last)";
+             CERR << endl;
+           }
+        value_stack.pop_back();
+        return Value_P();
       }
-
-   value_stack.pop_back();
-
-   return ret;
 }
 //-----------------------------------------------------------------------------
 void
@@ -364,16 +430,17 @@ ValueStackItem vs;
 void
 Symbol::push_value(Value_P value)
 {
-   Log(LOG_SYMBOL_push_pop)
-      {
-        CERR << "+push_value " << symbol;
-        if (value_stack.size() == 0)   CERR << " (initial)";
-        CERR << endl;
-      }
-
 ValueStackItem vs;
    value_stack.push_back(vs);
-   assign(value);
+   assign(value, LOC);
+
+   Log(LOG_SYMBOL_push_pop)
+      {
+        CERR << "+push-value " << symbol << " flags ";
+        print_flags(CERR, get_value()->get_flags()) << " ";
+        if (value_stack.size() == 0)   CERR << " (initial)";
+        CERR << " addr " << (const void *)get_value().get() << endl;
+      }
 }
 //-----------------------------------------------------------------------------
 int
@@ -402,15 +469,16 @@ Symbol::get_value()
 {
 const ValueStackItem & vs = value_stack.back();
 
-   if (vs.name_class == NC_VARIABLE)   return vs.sym_val.value;
+   if (vs.name_class == NC_VARIABLE)   return vs.apl_val;
 
-   return 0;
+   return Value_P();
 }
 //-----------------------------------------------------------------------------
 bool
 Symbol::can_be_defined() const
 {
-   if (value_stack.size() > 1)   return false;   // symbol was localized
+   if (value_stack.size() > 1)         return false;   // localized
+   if (Workspace::is_called(symbol))   return false;  // on stack
 
    if (value_stack.back().name_class == NC_UNUSED_USER_NAME)   return true;
    if (value_stack.back().name_class == NC_FUNCTION)           return true;
@@ -425,7 +493,7 @@ Symbol::get_apl_value() const
    if (value_stack.back().name_class != NC_VARIABLE)
       throw_symbol_error(get_name(), LOC);
 
-   return value_stack.back().sym_val.value;
+   return value_stack.back().apl_val;
 }
 //-----------------------------------------------------------------------------
 bool
@@ -594,8 +662,9 @@ Symbol::resolve(Token & tok, bool left_sym)
 
              {
                IntCell lab(value_stack.back().sym_val.label);
-               Value_P value = new Value(lab, LOC);
-               tok = Token(TOK_APL_VALUE1, value);
+               Value_P value(new Value(lab, LOC));
+               Token t(TOK_APL_VALUE1, value);
+               move_1(tok, t, LOC);
              }
              return;
 
@@ -603,12 +672,15 @@ Symbol::resolve(Token & tok, bool left_sym)
              if (left_sym)   return;   // leave symbol as is
 
              // if we resolve a variable. the value is considered grouped.
-             tok = Token(TOK_APL_VALUE1, get_apl_value());
+             {
+               Token t(TOK_APL_VALUE1, get_apl_value()->clone(LOC));
+               move_1(tok, t, LOC);
+             }
              return;
 
         case NC_FUNCTION:
         case NC_OPERATOR:
-             tok = value_stack.back().sym_val.function->get_token();
+             move_2(tok, value_stack.back().sym_val.function->get_token(), LOC);
              return;
 
         case NC_SHARED_VAR:
@@ -681,7 +753,7 @@ Symbol::resolve(Token & tok, bool left_sym)
 
                CDR_string cdr;
                const string & data = response->get__VALUE_IS__value();
-               loop(d, data.size())   cdr += data[d];
+               loop(d, data.size())   cdr.append(data[d]);
                Value_P value = CDR::from_CDR(cdr, LOC);
 
                if (!value)     VALUE_ERROR;
@@ -692,7 +764,8 @@ Symbol::resolve(Token & tok, bool left_sym)
                  Assert(svar);
                  svar->set_state(true, LOC);
                }
-               tok = CHECK(value, LOC);
+               value->check_value(LOC);
+               new (&tok) Token(TOK_APL_VALUE1, value);
              }
              return;
 
@@ -703,7 +776,7 @@ Symbol::resolve(Token & tok, bool left_sym)
 }
 //-----------------------------------------------------------------------------
 Token
-Symbol::resolve_lv()
+Symbol::resolve_lv(const char * loc)
 {
    Log(LOG_SYMBOL_resolve)
       CERR << "resolve_lv() symbol " << get_name() << endl; 
@@ -717,11 +790,11 @@ Symbol::resolve_lv()
              << "' has changed type from variable to name class "
              << value_stack.back().name_class << endl
              << " while executing an assignment" << endl;
-        throw_apl_error(E_LEFT_SYNTAX_ERROR, LOC);
+        throw_apl_error(E_LEFT_SYNTAX_ERROR, loc);
       }
 
-Value_P val = value_stack.back().sym_val.value;
-   return Token(TOK_APL_VALUE1, val->get_cellrefs(LOC));
+Value_P val = value_stack.back().apl_val;
+   return Token(TOK_APL_VALUE1, val->get_cellrefs(loc));
 }
 //-----------------------------------------------------------------------------
 TokenClass
@@ -766,15 +839,14 @@ ValueStackItem & vs = value_stack.back();
 
    if (vs.name_class == NC_VARIABLE)
       {
-        vs.sym_val.value->clear_assigned();
-        vs.sym_val.value->erase(LOC);
+        ptr_clear(vs.apl_val, LOC);
       }
    else if (vs.name_class == NC_FUNCTION || vs.name_class == NC_OPERATOR)
       {
         const UserFunction * ufun = vs.sym_val.function->get_ufun1();
         Assert(ufun);
         const Executable * exec = ufun;
-        StateIndicator * oexec = Workspace::the_workspace->oldest_exec(exec);
+        StateIndicator * oexec = Workspace::oldest_exec(exec);
         if (oexec)
            {
              // ufun is still used on the SI stack. We do not delete ufun,
@@ -782,7 +854,7 @@ ValueStackItem & vs = value_stack.back();
              //
              // CERR << "⎕EX function " << ufun->get_name()
              //      << " is on SI !" << endl;
-             Workspace::the_workspace->expunged_functions.push_back(ufun);
+             Workspace::add_expunged_function(ufun);
            }
         else
            {
@@ -825,7 +897,6 @@ ValueStackItem & vs = value_stack.back();
         // remember old value
         //
         Value_P old_value = get_apl_value();
-        Assert(old_value->is_assigned());
 
         // change name class and store AP number
         //
@@ -833,9 +904,7 @@ ValueStackItem & vs = value_stack.back();
         set_SV_key(key);
 
         // assign old value to shared variable
-        assign(old_value);
-        old_value->clear_assigned();
-        old_value->erase(LOC);
+        assign(old_value, LOC);
 
         return;
       }
@@ -915,10 +984,10 @@ UCS_string data;
       {
         case NC_VARIABLE:
              {
-               data += UNI_ASCII_A;
-               data += get_name();
-               data += UNI_LEFT_ARROW;
-               Quad_TF::tf2_ravel(0, data, value_stack[0].sym_val.value);
+               data.append(UNI_ASCII_A);
+               data.append(get_name());
+               data.append(UNI_LEFT_ARROW);
+               Quad_TF::tf2_ravel(0, data, value_stack[0].apl_val);
              }
              break;
 
@@ -941,9 +1010,9 @@ UCS_string data;
 
                // write function record(s)
                //
-               data += UNI_ASCII_F;
-               data += get_name();
-               data += UNI_ASCII_SPACE;
+               data.append(UNI_ASCII_F);
+               data.append(get_name());
+               data.append(UNI_ASCII_SPACE);
                Quad_TF::tf2_fun_ucs(data, get_name(), fun);
              }
              break;
@@ -977,11 +1046,12 @@ Symbol::unmark_all_values() const
          switch(item.name_class)
             {
               case NC_VARIABLE:
-                   item.sym_val.value->unmark();
+                   item.apl_val->unmark();
                    break;
 
               case NC_FUNCTION:
               case NC_OPERATOR:
+                   if (item.sym_val.function->is_native())   break;
                    {
                      const UserFunction * ufun =
                                           item.sym_val.function->get_ufun1();
@@ -1005,7 +1075,7 @@ Symbol::unmark_all_values() const
 }
 //-----------------------------------------------------------------------------
 int
-Symbol::show_owners(ostream & out, Value_P value) const
+Symbol::show_owners(ostream & out, const Value & value) const
 {
 int count = 0;
 
@@ -1015,7 +1085,7 @@ int count = 0;
          switch(item.name_class)
             {
               case NC_VARIABLE:
-                   if (item.sym_val.value == value)
+                   if (item.apl_val.get() == &value)
                       {
                          out << "    Variable[vs=" << v << "] "
                             << get_name() << endl;
@@ -1056,19 +1126,59 @@ const Cell * cV = &values->get_ravel(0);
         Symbol * sym = symbols[sym_count - s - 1];
         if (cV->is_pointer_cell())
            {
-             sym->assign(cV->get_pointer_value());
+             sym->assign(cV->get_pointer_value(), LOC);
            }
         else
            {
-             Value_P val = new Value(LOC);
+             Value_P val(new Value(LOC));
              val->get_ravel(0).init(*cV);
-             sym->assign(val);
+             sym->assign(val, LOC);
            }
 
         cV += incr;   // skalar extend values
       }
 }
 //-----------------------------------------------------------------------------
+void
+Symbol::clear_vs()
+{
+   while (value_stack.size() > 1)
+      {
+        Value_P deleted = pop(true);
+      }
+
+ValueStackItem & tos = value_stack[0];
+
+   switch(tos.name_class)
+      {
+        case NC_LABEL:
+             Assert(0 && "should not happen since stack height == 1");
+             break;
+
+        case NC_VARIABLE:
+             tos.name_class = NC_UNUSED_USER_NAME;
+             ptr_clear(tos.apl_val, LOC);
+             break;
+
+        case NC_UNUSED_USER_NAME:
+             break;
+
+        case NC_FUNCTION:
+        case NC_OPERATOR:
+             if (tos.sym_val.function->is_native())   break;
+
+             {
+               const UserFunction * ufun = tos.sym_val.function->get_ufun1();
+               Assert(ufun);
+               const Executable * exec = ufun;
+               Assert(!Workspace::oldest_exec(exec));
+             }
+
+             delete tos.sym_val.function;
+             break;
+      }
+}
+//-----------------------------------------------------------------------------
 ostream &
 operator <<(ostream & out, const Symbol & sym)
 {
