@@ -45,7 +45,7 @@ SymbolTable::SymbolTable()
 NamedObject *
 SymbolTable::lookup_existing_name(const UCS_string & name)
 {
-   Assert(name[0] != UNI_QUAD_QUAD);   // should not be called for ⎕xx
+   Assert(!Avec::is_quad(name[0]));   // should not be called for ⎕xx
 
 Symbol * sym = lookup_existing_symbol(name);
    if (sym == 0)   return 0;
@@ -64,7 +64,7 @@ Symbol * sym = lookup_existing_symbol(name);
 Symbol *
 SymbolTable::lookup_existing_symbol(const UCS_string & sym_name)
 {
-   Assert(sym_name[0] != UNI_QUAD_QUAD);   // should not be called for ⎕xx
+   Assert(!Avec::is_quad(sym_name[0]));   // should not be called for ⎕xx
 
 uint32_t hash = FNV_Offset_32;
    loop(s, sym_name.size())   hash = hash*FNV_Prime_32 ^ sym_name[s];
@@ -81,7 +81,7 @@ uint32_t hash = FNV_Offset_32;
 Symbol *
 SymbolTable::lookup_symbol(const UCS_string & sym_name)
 {
-   Assert(sym_name[0] != UNI_QUAD_QUAD);   // should not be called for ⎕xx
+   Assert(!Avec::is_quad(sym_name[0]));   // should not be called for ⎕xx
 
    // compute hash for sym_name
    //
@@ -298,7 +298,7 @@ SymbolTable::unmark_all_values() const
 }
 //-----------------------------------------------------------------------------
 int
-SymbolTable::show_owners(ostream & out, Value_P value) const
+SymbolTable::show_owners(ostream & out, const Value & value) const
 {
 int count = 0;
    loop(s, MAX_SYMBOL_COUNT)
@@ -347,7 +347,7 @@ SymbolTable::clear(ostream & out)
               if (sym->is_erased())               continue;
               if (sym->value_stack.size() == 0)   continue;
 
-              Assert(!Workspace::the_workspace->is_called(sym->get_name()));
+              Assert(!Workspace::is_called(sym->get_name()));
               Assert(sym->value_stack.size() == 1);
 
               sym->clear_vs();
@@ -358,81 +358,47 @@ SymbolTable::clear(ostream & out)
 }
 //-----------------------------------------------------------------------------
 void
-Symbol::clear_vs()
-{
-ValueStackItem & tos = value_stack[0];
-
-   switch(tos.name_class)
-      {
-        case NC_LABEL:
-             Assert(0 && "should not happen since stack height == 1");
-             break;
-
-        case NC_VARIABLE:
-             tos.name_class = NC_UNUSED_USER_NAME;
-             tos.sym_val.value->clear_assigned();
-             tos.sym_val.value->erase(LOC);
-             break;
-
-        case NC_UNUSED_USER_NAME:
-             break;
-
-        case NC_FUNCTION:
-        case NC_OPERATOR:
-             {
-               const UserFunction * ufun = tos.sym_val.function->get_ufun1();
-               Assert(ufun);
-               const Executable * exec = ufun;
-               Assert(!Workspace::the_workspace->oldest_exec(exec));
-             }
-
-             delete tos.sym_val.function;
-             break;
-      }
-}
-//-----------------------------------------------------------------------------
-void
 SymbolTable::erase_one_symbol(ostream & out, const UCS_string & sym)
 {
 Symbol * symbol = lookup_existing_symbol(sym);
 
    if (symbol == 0)
       {
-        UCS_string & t4 = Workspace::the_workspace->more_error;
+        UCS_string & t4 = Workspace::more_error();
         t4.clear();
-        t4.app("Can't )ERASE symbol '");
+        t4.append_utf8("Can't )ERASE symbol '");
         t4.append(sym);
-        t4.app("': unknown symbol ");
+        t4.append_utf8("': unknown symbol ");
         return;
       }
 
    if (symbol->is_erased())
       {
-        UCS_string & t4 = Workspace::the_workspace->more_error;
+        UCS_string & t4 = Workspace::more_error();
         t4.clear();
-        t4.app("Can't )ERASE symbol '");
+        t4.append_utf8("Can't )ERASE symbol '");
         t4.append(sym);
-        t4.app("': already erased");
+        t4.append_utf8("': already erased");
         return;
       }
 
    if (symbol->value_stack.size() != 1)
       {
-        UCS_string & t4 = Workspace::the_workspace->more_error;
+        UCS_string & t4 = Workspace::more_error();
         t4.clear();
-        t4.app("Can't )ERASE symbol '");
+        t4.append_utf8("Can't )ERASE symbol '");
         t4.append(sym);
-        t4.app("': symbol is pushed (localized)");
+        t4.append_utf8("': symbol is pushed (localized)");
         return;
       }
 
-   if (Workspace::the_workspace->is_called(sym))
+   if (Workspace::is_called(sym))
       {
-        UCS_string & t4 = Workspace::the_workspace->more_error;
+        UCS_string & t4 = Workspace::more_error();
         t4.clear();
-        t4.app("Can't )ERASE symbol '");
+        t4.append_utf8("Can't )ERASE symbol '");
         t4.append(sym);
-        t4.app("': symbol is called (is on SI)");
+        t4.append_utf8("': symbol is called (is on SI)");
         return;
       }
 
@@ -446,31 +412,30 @@ ValueStackItem & tos = symbol->value_stack[0];
 
              case NC_VARIABLE:
                   tos.name_class = NC_UNUSED_USER_NAME;
-                  tos.sym_val.value->clear_assigned();
-                  tos.sym_val.value->erase(LOC);
-                  tos.sym_val.value = 0;
+                  ptr_clear(tos.apl_val, LOC);
                   symbol->set_erased(true);
-                  out << ")ERASEed '" << sym << "' (variable)" << endl;
                   return;
 
              case NC_UNUSED_USER_NAME:
                   symbol->set_erased(true);
-                  out << ")ERASEed '" << sym << "' (unused)" << endl;
                   return;
 
              case NC_FUNCTION:
              case NC_OPERATOR:
+                  Assert(tos.sym_val.function);
+                  if (tos.sym_val.function->is_native())   return;
+
                   {
                     const UserFunction * ufun =
                                          tos.sym_val.function->get_ufun1();
                     Assert(ufun);
-                    if (Workspace::the_workspace->oldest_exec(ufun))
+                    if (Workspace::oldest_exec(ufun))
                        {
-                         UCS_string & t4 = Workspace::the_workspace->more_error;
+                         UCS_string & t4 = Workspace::more_error();
                          t4.clear();
-                         t4.app("Can't )ERASE symbol '");
+                         t4.append_utf8("Can't )ERASE symbol '");
                          t4.append(sym);
-                         t4.app("':  pushed on SI-stack");
+                         t4.append_utf8("':  pushed on SI-stack");
                          return;
                        }
                   }
@@ -479,7 +444,6 @@ ValueStackItem & tos = symbol->value_stack[0];
                   tos.sym_val.function = 0;
                   tos.name_class = NC_UNUSED_USER_NAME;
                   symbol->set_erased(true);
-                  out << ")ERASEed '" << sym << "' (function)" << endl;
                   return;
       }
 
