@@ -21,12 +21,17 @@
 #include <langinfo.h>
 #include <stdint.h>
 #include <stdlib.h>
-#include <utmpx.h>           // for ⎕UL
 #include <sys/resource.h>   // for ⎕WA
 #include <sys/time.h>
 
+// FreeBSD 8.x lacks utmpx.h
+#ifdef HAVE_UTMPX_H
+# include <utmpx.h>           // for ⎕UL
+#endif
+
 #include "CDR.hh"
 #include "CharCell.hh"
+#include "Common.hh"
 #include "FloatCell.hh"
 #include "IndexExpr.hh"
 #include "Input.hh"
@@ -42,12 +47,16 @@
 
 UCS_string Quad_QUOTE::prompt;
 
+ShapeItem Quad_SYL::si_depth_limit = 0;
+ShapeItem Quad_SYL::value_count_limit = 0;
+ShapeItem Quad_SYL::ravel_count_limit = 0;
+
 int Quad_ARG::argc = 0;
 const char ** Quad_ARG::argv = 0;
 
 //=============================================================================
 void
-SystemVariable::assign(Value_P value)
+SystemVariable::assign(Value_P value, const char * loc)
 {
    CERR << "SystemVariable::assign() not (yet) implemented for "
         << get_Id() << endl;
@@ -115,13 +124,13 @@ SystemVariable::get_attributes(int mode, Cell * dest) const
 Quad_AV::Quad_AV()
    : RO_SystemVariable(ID_QUAD_AV)
 {
-   Symbol::assign(&Value::AV);
+   Symbol::assign(Value::AV_P, LOC);
 }
 //-----------------------------------------------------------------------------
 Unicode
 Quad_AV::indexed_at(uint32_t pos)
 {
-   if (pos < MAX_AV)   return Value::AV.get_ravel(pos).get_char_value();
+   if (pos < MAX_AV)   return Value::AV_P->get_ravel(pos).get_char_value();
    return UNI_AV_MAX;
 }
 //=============================================================================
@@ -130,7 +139,7 @@ Quad_AI::Quad_AI()
      session_start(now()),
      user_wait(0)
 {
-   Symbol::assign(get_apl_value());
+   Symbol::assign(get_apl_value(), LOC);
 }
 //-----------------------------------------------------------------------------
 Value_P
@@ -139,12 +148,13 @@ Quad_AI::get_apl_value() const
 const int total_ms = (now() - session_start)/1000;
 const int user_ms  = user_wait/1000;
 
-Value_P ret = new Value(4, LOC);
+Value_P ret(new Value(4, LOC));
    new (&ret->get_ravel(0))   IntCell(ProcessorID::get_own_ID());
    new (&ret->get_ravel(1))   IntCell(total_ms - user_ms);
    new (&ret->get_ravel(2))   IntCell(total_ms);
    new (&ret->get_ravel(3))   IntCell(user_ms);
 
+   ret->check_value(LOC);
    return ret;
 }
 //=============================================================================
@@ -156,19 +166,20 @@ Quad_ARG::Quad_ARG()
 Value_P
 Quad_ARG::get_apl_value() const
 {
-Value_P Z = new Value(argc, LOC);
+Value_P Z(new Value(argc, LOC));
 Cell * C = &Z->get_ravel(0);
 
    loop(a, Quad_ARG::argc)
       {
         const char * arg = Quad_ARG::argv[a];
         const int len = strlen(arg);
-        Value_P val = new Value(len, LOC);
+        Value_P val(new Value(len, LOC));
         loop(l, len)   new (&val->get_ravel(l))   CharCell(Unicode(arg[l]));
 
         new (C++)   PointerCell(val);
       }
 
+   Z->check_value(LOC);
    return Z;
 }
 //=============================================================================
@@ -176,14 +187,16 @@ Quad_CT::Quad_CT()
    : SystemVariable(ID_QUAD_CT),
      current_ct(DEFAULT_QUAD_CT)
 {
-Value_P value = new Value(LOC);
+Value_P value(new Value(LOC));
    new (&value->get_ravel(0)) FloatCell(current_ct);
 
-   Symbol::assign(value);
+   value->check_value(LOC);
+
+   Symbol::assign(value, LOC);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_CT::assign(Value_P value)
+Quad_CT::assign(Value_P value, const char * loc)
 {
    if (!value->is_skalar_or_len1_vector())
       {
@@ -201,13 +214,12 @@ APL_Float val = cell.get_real_value();
    // APL2 "discourages" the use of ⎕CT > 1E¯9.
    // We round down to MAX_QUAD_CT (= 1E¯9) instead.
    //
-   if (val > Value::Max_CT.get_ravel(0).get_real_value())
+   if (val > Value::Max_CT_P->get_ravel(0).get_real_value())
       {
-        value->erase(LOC);
-        value = &Value::Max_CT;
+        value = Value::Max_CT_P;
       }
 
-   Symbol::assign(value);
+   Symbol::assign(value, LOC);
    current_ct = value->get_ravel(0).get_real_value();
 }
 //=============================================================================
@@ -218,7 +230,7 @@ Quad_EM::get_apl_value() const
       {
         // return 3 0⍴' '
         //
-        return &Value::Str3_0;
+        return Value::Str3_0_P;
       }
 
 const UCS_string msg_1 = error.get_error_line_1();
@@ -236,7 +248,7 @@ ShapeItem max_len = len_1;
    if (max_len < len_3)   max_len = len_3;
 
 const Shape sh(3, max_len);
-Value_P Z = new Value(sh, LOC);
+Value_P Z(new Value(sh, LOC));
    loop(l, max_len)
       {
         new (&Z->get_ravel(l))
@@ -253,16 +265,19 @@ Value_P Z = new Value(sh, LOC);
          CharCell((l < len_3) ? msg_3[l] : UNI_ASCII_SPACE);
       }
 
+   Z->check_value(LOC);
    return Z;
 }
 //=============================================================================
 Value_P
 Quad_ET::get_apl_value() const
 {
-Value_P Z = new Value(2, LOC);
+Value_P Z(new Value(2, LOC));
 
    new (&Z->get_ravel(0)) IntCell(Error::error_major(error.error_code));
    new (&Z->get_ravel(1)) IntCell(Error::error_minor(error.error_code));
+
+   Z->check_value(LOC);
    return Z;
 }
 //=============================================================================
@@ -270,11 +285,13 @@ Quad_FC::Quad_FC() : SystemVariable(ID_QUAD_FC)
 {
    set_default();
 
-Value_P value = new Value(6, LOC);
+Value_P value(new Value(6, LOC));
 
    loop(c, 6)   new (&value->get_ravel(c))  CharCell(current_fc[c]);
 
-   Symbol::assign(value);
+   value->check_value(LOC);
+
+   Symbol::assign(value, LOC);
 }
 //-----------------------------------------------------------------------------
 void
@@ -289,11 +306,11 @@ Quad_FC::set_default()
 }
 //-----------------------------------------------------------------------------
 void
-Quad_FC::assign(Value_P value)
+Quad_FC::assign(Value_P value, const char * loc)
 {
    if (!value->is_skalar_or_vector())   RANK_ERROR;
 
-uint32_t value_len = value->element_count();
+ShapeItem value_len = value->element_count();
    if (value_len > 6)   value_len = 6;
 
    loop(c, value_len)
@@ -312,24 +329,20 @@ uint32_t value_len = value->element_count();
    //
    if (Bif_F12_FORMAT::is_control_char(current_fc[4]))
       current_fc[4] = UNI_ASCII_SPACE;
-
-Value_P v1 = new Value(6, LOC);
-
-   loop(c, 6)   new (&v1->get_ravel(c))   CharCell(current_fc[c]);
-   Symbol::assign(v1);
 }
 //-----------------------------------------------------------------------------
 void
 Quad_FC::assign_indexed(const IndexExpr & IX, Value_P value)
 {
+Q(LOC)
    if (IX.value_count() != 1)   INDEX_ERROR;
 
    // at this point we have a one dimensional index. It it were non-empty,
    // then assign_indexed(Value) would have been called instead. Therefore
    // IX must be an elided index (like in ⎕FC[] ← value)
    //
-   Assert1(IX.values[0] == 0);
-   assign(value);   // ⎕FC[]←value
+   Assert1(!IX.values[0]);
+   assign(value, LOC);   // ⎕FC[]←value
 }
 //-----------------------------------------------------------------------------
 void
@@ -358,25 +371,31 @@ const APL_Integer qio = Workspace::get_IO();
    //
    if (Bif_F12_FORMAT::is_control_char(current_fc[4]))
       current_fc[4] = UNI_ASCII_SPACE;
-
-Value_P v1 = new Value(6, LOC);
-
+}
+//-----------------------------------------------------------------------------
+Value_P Quad_FC::get_apl_value() const
+{
+Value_P v1(new Value(6, LOC));
    loop(c, 6)   new (&v1->get_ravel(c))   CharCell(current_fc[c]);
-   Symbol::assign(v1);
+   v1->check_value(LOC);
+   return v1;
 }
 //=============================================================================
 Quad_IO::Quad_IO()
    : SystemVariable(ID_QUAD_IO),
      current_io(1)
 {
-Value_P value = new Value(LOC);
+Value_P value(new Value(LOC));
 
    new (&value->get_ravel(0)) IntCell(current_io);
-   Symbol::assign(value);
+
+   value->check_value(LOC);
+
+   Symbol::assign(value, LOC);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_IO::assign(Value_P value)
+Quad_IO::assign(Value_P value, const char * loc)
 {
    if (!value->is_skalar_or_len1_vector())
       {
@@ -386,7 +405,7 @@ Quad_IO::assign(Value_P value)
 
 const Cell & cell = value->get_ravel(0);
    current_io = value->get_ravel(0).get_near_bool(0.1);
-   Symbol::assign(current_io ? &Value::One : &Value::Zero);
+   Symbol::assign(current_io ? Value::One_P : Value::Zero_P, LOC);
 }
 //-----------------------------------------------------------------------------
 int
@@ -400,13 +419,13 @@ Quad_IO::expunge()
 Quad_L::Quad_L()
  : NL_SystemVariable(ID_QUAD_L)
 {
-   Symbol::assign(&Value::Zero);
+   Symbol::assign(Value::Zero_P, LOC);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_L::assign(Value_P value)
+Quad_L::assign(Value_P value, const char * loc)
 {
-StateIndicator * si = Workspace::the_workspace->SI_top_fun();
+StateIndicator * si = Workspace::SI_top_fun();
    if (si == 0)   return;
 
    // ignore assignments if error was SYNTAX ERROR or VALUE ERROR
@@ -419,11 +438,11 @@ StateIndicator * si = Workspace::the_workspace->SI_top_fun();
 Value_P
 Quad_L::get_apl_value() const
 {
-const StateIndicator * si = Workspace::the_workspace->SI_top_error();
+const StateIndicator * si = Workspace::SI_top_error();
    if (si)
       {
         Value_P ret = si->get_L();
-        if (ret)   return  ret;
+        if (!!ret)   return  ret;
       }
 
    VALUE_ERROR;
@@ -432,7 +451,7 @@ const StateIndicator * si = Workspace::the_workspace->SI_top_error();
 Quad_LC::Quad_LC()
    : RO_SystemVariable(ID_QUAD_LC)
 {
-   Symbol::assign(&Value::Zero);
+   Symbol::assign(Value::Zero_P, LOC);
 }
 //-----------------------------------------------------------------------------
 Value_P
@@ -442,40 +461,43 @@ Quad_LC::get_apl_value() const
    // count how many function elements Quad-LC has...
    //
 int len = 0;
-   for (StateIndicator * si = Workspace::the_workspace->SI_top(); si; si = si->get_parent())
+   for (StateIndicator * si = Workspace::SI_top();
+        si; si = si->get_parent())
        {
          if (si->get_executable()->get_parse_mode() == PM_FUNCTION)   ++len;
        }
 
-Value_P Z = new Value(len, LOC);
-Cell * cZ = &Z->get_ravel(0);
+Value_P Z(new Value(len, LOC));
 
-   for (StateIndicator * si = Workspace::the_workspace->SI_top(); si; si = si->get_parent())
+   for (StateIndicator * si = Workspace::SI_top();
+        si; si = si->get_parent())
        {
          if (si->get_executable()->get_parse_mode() == PM_FUNCTION)
-            new (cZ++)   IntCell(si->get_line());
+            new (Z->next_ravel())   IntCell(si->get_line());
        }
 
+   Z->set_default(*Value::Zero_P);
+   Z->check_value(LOC);
    return Z;
 }
 //=============================================================================
 Quad_LX::Quad_LX()
    : NL_SystemVariable(ID_QUAD_LX)
 {
-   Symbol::assign(&Value::Str0);
+   Symbol::assign(Value::Str0_P, LOC);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_LX::assign(Value_P value)
+Quad_LX::assign(Value_P value, const char * loc)
 {
    if (value->get_rank() > 1)      RANK_ERROR;
    if (!value->is_char_string())   DOMAIN_ERROR;
 
-   Symbol::assign(value);
+   Symbol::assign(value, LOC);
 }
 //=============================================================================
 void
-Quad_NLT::assign(Value_P value)
+Quad_NLT::assign(Value_P value, const char * loc)
 {
    if (value->get_rank() > 1)      RANK_ERROR;
    if (!value->is_char_string())   DOMAIN_ERROR;
@@ -503,7 +525,7 @@ const char * locale = setlocale(LC_ALL, new_locale);
    if (locale == 0)   DOMAIN_ERROR;
 
    // setlocale() has not complained, so we can set the ⎕NLT;
-   Symbol::assign(value);
+   Symbol::assign(value, LOC);
 }
 //-----------------------------------------------------------------------------
 Value_P
@@ -512,13 +534,14 @@ Quad_NLT::get_apl_value() const
    // get current locale
    //
 const char * locale = setlocale(LC_CTYPE, 0);   // e,g, "en_US.utf8"
-   if (locale == 0)   return &Value::Str0;
+   if (locale == 0)   return Value::Str0_P;
 
 UCS_string ulocale(locale);
 
-Value * Z = new Value(ulocale, LOC);
-   Z->set_default(&Value::Spc);
+Value_P Z(new Value(ulocale, LOC));
+   Z->set_default(*Value::Spc_P);
 
+   Z->check_value(LOC);
    return Z;
 }
 //=============================================================================
@@ -526,14 +549,16 @@ Quad_PP::Quad_PP()
    : SystemVariable(ID_QUAD_PP),
      current_pp(DEFAULT_QUAD_PP)
 {
-Value_P value = new Value(LOC);
+Value_P value(new Value(LOC));
 
    new (&value->get_ravel(0)) IntCell(current_pp);
-   Symbol::assign(value);
+
+   value->check_value(LOC);
+   Symbol::assign(value, LOC);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_PP::assign(Value_P value)
+Quad_PP::assign(Value_P value, const char * loc)
 {
    if (!value->is_skalar_or_len1_vector())
       {
@@ -548,27 +573,28 @@ const APL_Integer val = cell.get_near_int(0.1);
    if (val > MAX_QUAD_PP)
       {
         current_pp = MAX_QUAD_PP;
-        value = &Value::Max_PP;
+        value = Value::Max_PP_P;
       }
    else
       {
         current_pp = val;
       }
 
-   Symbol::assign(value);
+   Symbol::assign(value, LOC);
 }
 //=============================================================================
 Quad_PR::Quad_PR()
    : SystemVariable(ID_QUAD_PR),
      current_pr(UNI_ASCII_SPACE)
 {
-Value_P value = new Value(current_pr, LOC);
+Value_P value(new Value(current_pr, LOC));
 
-   Symbol::assign(value);
+   value->check_value(LOC);
+   Symbol::assign(value, LOC);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_PR::assign(Value_P value)
+Quad_PR::assign(Value_P value, const char * loc)
 {
 UCS_string ucs = value->get_UCS_ravel();
 
@@ -576,18 +602,18 @@ UCS_string ucs = value->get_UCS_ravel();
 
    current_pr = ucs;
 
-   Symbol::assign(value);
+   Symbol::assign(value, LOC);
 }
 //=============================================================================
 Quad_PS::Quad_PS()
    : SystemVariable(ID_QUAD_PS),
      current_ps(PR_APL)
 {
-   Symbol::assign(&Value::Zero);
+   Symbol::assign(Value::Zero_P, LOC);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_PS::assign(Value_P value)
+Quad_PS::assign(Value_P value, const char * loc)
 {
    if (!value->is_skalar_or_len1_vector())
       {
@@ -604,21 +630,22 @@ const APL_Integer val = cell.get_near_int(0.1);
    else                 DOMAIN_ERROR;
 
    new (&value->get_ravel(0))  IntCell(val);
-   Symbol::assign(value);
+   Symbol::assign(value, LOC);
 }
 //=============================================================================
 Quad_PT::Quad_PT()
    : RO_SystemVariable(ID_QUAD_PT)
 {
-   Symbol::assign(&Value::Zero);
+   Symbol::assign(Value::Zero_P, LOC);
 }
 //-----------------------------------------------------------------------------
 Value_P
 Quad_PT::get_apl_value() const
 {
-Value_P Z = new Value(LOC);
+Value_P Z(new Value(LOC));
    new (&Z->get_ravel(0))   FloatCell(Workspace::get_CT());
 
+   Z->check_value(LOC);
    return Z;
 }
 //=============================================================================
@@ -626,14 +653,16 @@ Quad_PW::Quad_PW()
    : SystemVariable(ID_QUAD_PW),
      current_pw(80)
 {
-Value_P value = new Value(LOC);
+Value_P value(new Value(LOC));
 
    new (&value->get_ravel(0)) IntCell(current_pw);
-   Symbol::assign(value);
+
+   value->check_value(LOC);
+   Symbol::assign(value, LOC);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_PW::assign(Value_P value)
+Quad_PW::assign(Value_P value, const char * loc)
 {
    if (!value->is_skalar_or_len1_vector())
       {
@@ -651,7 +680,7 @@ const APL_Integer val = cell.get_near_int(0.1);
    if (val > MAX_QUAD_PW)   return;
 
    current_pw = val;
-   Symbol::assign(value);
+   Symbol::assign(value, LOC);
 }
 //=============================================================================
 Quad_QUAD::Quad_QUAD()
@@ -660,7 +689,7 @@ Quad_QUAD::Quad_QUAD()
 }
 //-----------------------------------------------------------------------------
 void
-Quad_QUAD::assign(Value_P value)
+Quad_QUAD::assign(Value_P value, const char * loc)
 {
    // write pending LF from  ⍞ (if any)
    Quad_QUOTE::done(true, LOC);
@@ -680,7 +709,7 @@ Quad_QUAD::resolve(Token & token, bool left)
 
 UCS_string in = Input::get_line();
 
-   token = Bif_F1_EXECUTE::execute_statement(in);
+   move_2(token, Bif_F1_EXECUTE::execute_statement(in), LOC);
 }
 //=============================================================================
 Quad_QUOTE::Quad_QUOTE()
@@ -688,9 +717,9 @@ Quad_QUOTE::Quad_QUOTE()
 {
    // we assign a dummy value so that ⍞ is not undefined.
    //
-Value_P dummy = new Value(UCS_string(LOC), LOC);
-   Symbol::assign(dummy);
-   dummy->clear_assigned();
+Value_P dummy(new Value(UCS_string(LOC), LOC));
+   dummy->set_complete();
+   Symbol::assign(dummy, LOC);
    dummy->set_forever();
 }
 //-----------------------------------------------------------------------------
@@ -709,7 +738,7 @@ Quad_QUOTE::done(bool with_LF, const char * loc)
 }
 //-----------------------------------------------------------------------------
 void
-Quad_QUOTE::assign(Value_P value)
+Quad_QUOTE::assign(Value_P value, const char * loc)
 {
    Log(LOG_cork)
       CERR << "Quad_QUOTE::assign() called, buffer = ["
@@ -761,19 +790,21 @@ const UCS_string qpr = Workspace::get_PR();
            }
       }
 
-   return new Value(in, LOC);
+Value_P Z(new Value(in, LOC));
+   Z->check_value(LOC);
+   return Z;
 }
 //=============================================================================
 Quad_R::Quad_R()
  : NL_SystemVariable(ID_QUAD_R)
 {
-   Symbol::assign(&Value::Zero);
+   Symbol::assign(Value::Zero_P, LOC);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_R::assign(Value_P value)
+Quad_R::assign(Value_P value, const char * loc)
 {
-StateIndicator * si = Workspace::the_workspace->SI_top_fun();
+StateIndicator * si = Workspace::SI_top_fun();
    if (si == 0)   return;
 
    // ignore assignments if error was SYNTAX ERROR or VALUE ERROR
@@ -786,41 +817,138 @@ StateIndicator * si = Workspace::the_workspace->SI_top_fun();
 Value_P
 Quad_R::get_apl_value() const
 {
-const StateIndicator * si = Workspace::the_workspace->SI_top_error();
+const StateIndicator * si = Workspace::SI_top_error();
    if (si)
       {
         Value_P ret = si->get_R();
-        if (ret)   return  ret;
+        if (!!ret)   return  ret;
       }
 
    VALUE_ERROR;
 }
 //=============================================================================
+void
+Quad_SYL::assign(Value_P value, const char * loc)
+{
+   // Quad_SYL is mostly read-only, so we only allow assign_indexed() with
+   // certain values.
+   //
+   if (!!value)   SYNTAX_ERROR;
+
+   // this assign is called from the constructor in order to trigger the
+   // creation of a symbol for Quad_SYL.
+   //
+   Symbol::assign(Value::Zero_P, LOC);
+}
+//-----------------------------------------------------------------------------
+void
+Quad_SYL::assign_indexed(const IndexExpr & IDX, Value_P value)
+{
+   //  must be an array index of the form [2; something]
+   //
+   if (IDX.value_count() != 2)   INDEX_ERROR;
+
+   // IDX is in reverse order: ⎕SYL[X1;X2]
+   //
+Value_P X2 = IDX.values[0];
+
+const APL_Float qct = Workspace::get_CT();
+const APL_Integer qio = Workspace::get_IO();
+
+   if (!X2)                                             INDEX_ERROR;
+   if (X2->element_count() != 1)                        INDEX_ERROR;
+   if (!X2->get_ravel(0).is_near_int(qct))              INDEX_ERROR;
+   if (X2->get_ravel(0).get_near_int(qct) != qio + 1)   INDEX_ERROR;
+
+Value_P X1 = IDX.values[1];
+   delete &IDX;
+
+   assign_indexed(X1, value);
+}
+//-----------------------------------------------------------------------------
+void
+Quad_SYL::assign_indexed(Value_P X, Value_P B)
+{
+const APL_Float qct = Workspace::get_CT();
+
+   if (!(X->is_int_skalar(qct) || X->is_int_vector(qct)))   INDEX_ERROR;
+   if (!(B->is_int_skalar(qct) || B->is_int_vector(qct)))   DOMAIN_ERROR;
+   if (X->element_count() != B->element_count())            LENGTH_ERROR;
+
+const ShapeItem ec = X->element_count();
+const APL_Integer qio = Workspace::get_IO();
+
+   loop(e, ec)
+      {
+        const APL_Integer x = X->get_ravel(e).get_near_int(qct) - qio;
+        const APL_Integer b = B->get_ravel(e).get_near_int(qct);
+
+        if (x == SYL_SI_DEPTH_LIMIT)   // SI depth limit
+           {
+             const int depth = Workspace::SI_entry_count();
+             if (b < (depth + 4))   DOMAIN_ERROR;    // limit too low
+             si_depth_limit = b;
+           }
+        else if (x == SYL_VALUE_COUNT_LIMIT)   // value count limit
+           {
+             if (b < (Value::value_count + 10))   DOMAIN_ERROR;    // too low
+             value_count_limit = b;
+           }
+        else if (x == SYL_RAVEL_BYTES_LIMIT)   // ravel bytes depth limit
+           {
+             const int cells = b / sizeof(Cell);
+             if (cells < (Value::total_ravel_count + 1000))   DOMAIN_ERROR;
+
+             ravel_count_limit = cells;
+           }
+        else
+           {
+             INDEX_ERROR;
+           }
+      }
+}
+//-----------------------------------------------------------------------------
+Value_P
+Quad_SYL::get_apl_value() const
+{
+const Shape sh(SYL_MAX, 2);
+Value_P Z( new Value(sh, LOC));
+
+#define syl2(n, e, v) syl1(n, e, v)
+#define syl1(n, _e, v) \
+  new (Z->next_ravel()) PointerCell(Value_P(new Value(UCS_string(UTF8_string(n)), LOC))); \
+  new (Z->next_ravel()) IntCell(v);
+#include "SystemLimits.def"
+
+   Z->check_value(LOC);
+   return Z;
+}
+//=============================================================================
 Quad_TC::Quad_TC()
    : RO_SystemVariable(ID_QUAD_TC)
 {
-   Symbol::assign(&Value::Quad_TC);
+   Symbol::assign(Value::Quad_TC_P, LOC);
 }
 //-----------------------------------------------------------------------------
 Value_P
 Quad_TC::get_apl_value() const
 {
-   return &Value::Quad_TC;
+   return Value::Quad_TC_P;
 }
 //=============================================================================
 Quad_TS::Quad_TS()
    : RO_SystemVariable(ID_QUAD_TS)
 {
-   Symbol::assign(&Value::Zero);
+   Symbol::assign(Value::Zero_P, LOC);
 }
 //-----------------------------------------------------------------------------
 Value_P
 Quad_TS::get_apl_value() const
 {
-const int offset = Workspace::the_workspace->v_quad_TZ.get_offset();
+const int offset = Workspace::get_v_quad_TZ().get_offset();
 const YMDhmsu time(now() + offset);
 
-Value_P Z = new Value(7, LOC);
+Value_P Z(new Value(7, LOC));
    new (&Z->get_ravel(0)) IntCell(time.year);
    new (&Z->get_ravel(1)) IntCell(time.month);
    new (&Z->get_ravel(2)) IntCell(time.day);
@@ -829,6 +957,7 @@ Value_P Z = new Value(7, LOC);
    new (&Z->get_ravel(5)) IntCell(time.second);
    new (&Z->get_ravel(6)) IntCell(time.micro / 1000);
 
+   Z->check_value(LOC);
    return Z;
 }
 //=============================================================================
@@ -854,17 +983,19 @@ tm * gmean = gmtime(&now.tv_sec);
 const int diff_minutes = local_minutes - gm_minutes;
    offset_seconds = 60*diff_minutes;
 
-Value_P tz = new Value(LOC);
+Value_P value(new Value(LOC));
    if (offset_seconds % 3600 == 0)   // full hour
-      new (&tz->get_ravel(0))   IntCell(offset_seconds/3600);
+      new (&value->get_ravel(0))   IntCell(offset_seconds/3600);
    else
-      new (&tz->get_ravel(0))   FloatCell(offset_seconds/3600.0);
+      new (&value->get_ravel(0))   FloatCell(offset_seconds/3600.0);
 
-   Symbol::assign(tz);
+   value->check_value(LOC);
+
+   Symbol::assign(value, LOC);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_TZ::assign(Value_P value)
+Quad_TZ::assign(Value_P value, const char * loc)
 {
    if (!value->is_skalar())   RANK_ERROR;
 
@@ -877,7 +1008,7 @@ Cell & cell = value->get_ravel(0);
         if (ival < -12)   return;
         if (ival > 12)    return;
         offset_seconds = ival*3600;
-        Symbol::assign(value);
+        Symbol::assign(value, LOC);
         return;
       }
 
@@ -887,7 +1018,7 @@ Cell & cell = value->get_ravel(0);
         if (fval < -12)   return;
         if (fval > 12)    return;
         offset_seconds = int(0.5 + fval*3600);
-        Symbol::assign(value);
+        Symbol::assign(value, LOC);
         return;
       }
 
@@ -897,7 +1028,7 @@ Cell & cell = value->get_ravel(0);
 Quad_UL::Quad_UL()
    : RO_SystemVariable(ID_QUAD_UL)
 {
-   Symbol::assign(get_apl_value());
+   Symbol::assign(get_apl_value(), LOC);
 }
 //-----------------------------------------------------------------------------
 Value_P
@@ -921,21 +1052,22 @@ int user_count = 0;
    user_count = 1;
 #endif
 
-Value_P Z = new Value(LOC);
+Value_P Z(new Value(LOC));
    new (&Z->get_ravel(0))   IntCell(user_count);
+   Z->check_value(LOC);
    return Z;
 }
 //=============================================================================
 Quad_WA::Quad_WA()
    : RO_SystemVariable(ID_QUAD_WA)
 {
-   Symbol::assign(&Value::Zero);
+   Symbol::assign(Value::Zero_P, LOC);
 }
 //-----------------------------------------------------------------------------
 Value_P
 Quad_WA::get_apl_value() const
 {
-Value_P Z = new Value(LOC);
+Value_P Z(new Value(LOC));
 
 uint64_t total = total_memory;   // max memory as reported by RLIM_INFINITY
 uint64_t proc_mem = 0;            // memory as reported proc/mem_info
@@ -972,7 +1104,8 @@ uint64_t proc_mem = 0;            // memory as reported proc/mem_info
            }
         else                          // process has rlimit
            {
-             total -= used_memory;
+             total -= Value::total_ravel_count * sizeof(Cell)
+                    + Value::value_count * sizeof(Value);
            }
       }
    else
@@ -983,13 +1116,15 @@ uint64_t proc_mem = 0;            // memory as reported proc/mem_info
            }
         else                          // process has rlimit
            {
-             total -= used_memory;
+             total -= Value::total_ravel_count * sizeof(Cell)
+                    + Value::value_count * sizeof(Value);
              if (total > proc_mem)   total = proc_mem;   // use minimum
            }
       }
 
    new (&Z->get_ravel(0))   IntCell(total);
 
+   Z->check_value(LOC);
    return Z;
 }
 //=============================================================================
