@@ -27,6 +27,49 @@
 #include "Token.hh"
 #include "Workspace.hh"
 
+//-----------------------------------------------------------------------------
+Token::Token(const Token & other)
+   : tag(TOK_VOID)
+{
+   copy_1(*this, other, "Token::Token(other)");
+}
+//-----------------------------------------------------------------------------
+Token::Token(TokenTag tg, IndexExpr & idx)
+{
+   Assert(&idx);
+
+   if (tg == TOK_PINDEX)
+      {
+        // this token is a partial index in the prefix parser.
+        // use it as is.
+        //
+        tag = TOK_PINDEX;
+        value.index_val = &idx;
+      }
+   else
+      {
+        // this token is a complete index
+        //
+        Assert(tg == TOK_INDEX);
+        if (idx.value_count() < 2)   // [idx] or []
+           {
+             tag = TOK_AXES;
+             if (idx.value_count() == 0)   // []
+                {
+                  new (&value._apl_val()) Value_P;
+                }
+             else                          // [x]
+                {
+                  new (&value._apl_val())   Value_P(idx.extract_value(0));
+                }
+           }
+        else                         // [idx; ...]
+           {
+             tag = TOK_INDEX;
+             value.index_val = &idx;
+           }
+      }
+}
 //-----------------------------------------------------------------------------
 ostream &
 operator << (ostream & out, TokenTag tag)
@@ -85,13 +128,13 @@ operator << (ostream & out, const Token & token)
         return out;
       }
 
-   if (token.get_tag() == TOK_APL_VALUE  ||
-       token.get_tag() == TOK_APL_VALUE1 ||
-       token.get_tag() == TOK_APL_VALUE2)
+   if (token.get_tag() == TOK_APL_VALUE1 ||
+       token.get_tag() == TOK_APL_VALUE2 ||
+       token.get_tag() == TOK_APL_VALUE3)
       {
-        if (token.get_tag() == TOK_APL_VALUE)         out << "VALUE";
-        else if (token.get_tag() == TOK_APL_VALUE1)   out << "VALUE1";
+        if      (token.get_tag() == TOK_APL_VALUE1)   out << "VALUE1";
         else if (token.get_tag() == TOK_APL_VALUE2)   out << "VALUE2";
+        else if (token.get_tag() == TOK_APL_VALUE3)   out << "VALUE3";
         else                                          out << "VALUE???";
         Value_P value = token.get_apl_val();
         Assert(value);
@@ -145,12 +188,13 @@ operator << (ostream & out, const Token & token)
              return out << "RETURN ???";
 
         case TC_VALUE:    return token.print_value(out);
+
         case TC_INDEX:
         case TC_PINDEX:
              if (token.get_tag() == TOK_INDEX)
                 return out << token.get_index_val();
              else if (token.get_tag() == TOK_AXES)
-                return out << token.get_axes();
+                return out << *token.get_axes();
              else
                 FIXME;
 
@@ -182,13 +226,6 @@ operator << (ostream & out, const Token & token)
 }
 //-----------------------------------------------------------------------------
 void
-Token::clone_if_owned(const char * loc)
-{
-   if (get_ValueType() == TV_VAL)   
-      value.apl_val = value.apl_val->clone_if_owned(loc);
-}
-//-----------------------------------------------------------------------------
-void
 Token::ChangeTag(TokenTag new_tag)
 {
    Assert((tag & TV_MASK) == (new_tag & TV_MASK));
@@ -314,10 +351,10 @@ Token::print_value(ostream & out) const
         case TOK_COMPLEX:   return out << value.complex_val.real << "J"
                                        << value.complex_val.imag;
 
-        case TOK_APL_VALUE:
         case TOK_APL_VALUE1:
+        case TOK_APL_VALUE3:
              {
-               Value_P v = value.apl_val;
+               Value_P v = value._apl_val();
                if (v->get_rank() == 0)   out << "''";
                loop(r, v->get_rank())
                    {
@@ -388,34 +425,47 @@ UCS_string ucs;
 
    switch(get_Class())
       {
-        case TC_ASSIGN:   ucs += UNI_LEFT_ARROW;
-                          break;
+        case TC_ASSIGN:
+             ucs.append(UNI_LEFT_ARROW);
+             break;
 
-        case TC_R_ARROW:  ucs += UNI_RIGHT_ARROW;
-                          break;
+        case TC_R_ARROW:
+             ucs.append(UNI_RIGHT_ARROW);
+             break;
 
-        case TC_L_BRACK:  if (get_tag() == TOK_L_BRACK)
-                             ucs += UNI_ASCII_L_BRACK;
-                          else if (get_tag() == TOK_SEMICOL)
-                             ucs += UNI_ASCII_SEMICOLON;
-                          else Assert(0);
-                         break;
+        case TC_L_BRACK:
+             if (get_tag() == TOK_L_BRACK)
+                ucs.append(UNI_ASCII_L_BRACK);
+             else if (get_tag() == TOK_SEMICOL)
+                ucs.append(UNI_ASCII_SEMICOLON);
+             else
+                Assert(0);
+                break;
 
-        case TC_R_BRACK:  ucs += UNI_ASCII_R_BRACK;
-                          break;
+        case TC_R_BRACK:
+             ucs.append(UNI_ASCII_R_BRACK);
+             break;
 
-        case TC_END:      ucs += UNI_DIAMOND;
-                          break;
+        case TC_END:
+             ucs.append(UNI_DIAMOND);
+             break;
 
         case TC_RETURN:                                                  break;
-        case TC_LINE:     ucs += UNI_ASCII_LF;
-                          break;
+        case TC_LINE:
+             ucs.append(UNI_ASCII_LF);
+             break;
 
-        case TC_VALUE:    return PrintBuffer(*get_apl_val(),
-                                     PrintContext(style, 16, 1.0E-18, 80)).l1();
+        case TC_VALUE:
+             {
+               PrintContext pctx(style, 16, 1.0E-18, 80);
+               PrintBuffer pbuf(*get_apl_val(), pctx);
+               if (pbuf.get_height() == 0)   return ucs;
+               return pbuf.l1();
+             }
 
-        case TC_SYMBOL:  ucs.append(get_sym_ptr()->get_name());
-                         break;
+        case TC_SYMBOL:
+             ucs.append(get_sym_ptr()->get_name());
+             break;
 
 
         case TC_FUN0:
@@ -423,19 +473,21 @@ UCS_string ucs;
         case TC_R_PARENT:
         case TC_L_PARENT:
         case TC_OPER1:
-        case TC_OPER2: if (get_Id() != ID_No_ID)       return id_name(get_Id());
+        case TC_OPER2:
+             if (get_Id() != ID_No_ID)       return id_name(get_Id());
 
-                        CERR << "Token: " << HEX4(tag) << " " << *this
-                             << " at " << LOC << endl;
-                        FIXME;
-                        break;
+             CERR << "Token: " << HEX4(tag) << " " << *this
+                  << " at " << LOC << endl;
+             FIXME;
+             break;
 
 
-        default: CERR << "Token: " << HEX4(tag) << " " << *this
-                      << " at " << LOC << endl;
-                 Q((void *)get_Class())
-                 Backtrace::show(__FILE__, __LINE__);
-                 FIXME;
+        default:
+             CERR << "Token: " << HEX4(tag) << " " << *this
+                  << " at " << LOC << endl;
+             Q((void *)get_Class())
+             Backtrace::show(__FILE__, __LINE__);
+             FIXME;
       }
 
    return ucs;
@@ -468,9 +520,9 @@ const Unicode c2 = can.size() ? can[0] : Invalid_Unicode;
    //
 bool need_space = ! (Avec::no_space_after(c1) || Avec::no_space_before(c2));
 
-   if (need_space)   ucs += UNI_ASCII_SPACE;
+   if (need_space)   ucs.append(UNI_ASCII_SPACE);
 
-   ucs += can;
+   ucs.append(can);
    return need_space ? -can.size() : can.size();
 }
 //-----------------------------------------------------------------------------
@@ -558,3 +610,73 @@ Token::class_name(TokenClass tc)
    return "*** Obscure token class ***";
 }
 //-----------------------------------------------------------------------------
+inline void 
+Token::copy_N(const Token & src)
+{
+   tag = src.tag;
+   switch(src.get_ValueType())
+      {
+        case TV_NONE:  value.int_val    = 0;                              break;
+        case TV_CHAR:  value.char_val   = src.value.char_val;             break;
+        case TV_INT:   value.int_val    = src.value.int_val;              break;
+        case TV_FLT:   value.flt_val    = src.value.flt_val;              break;
+        case TV_CPX:   value.complex_val.real = src.value.complex_val.real;
+                       value.complex_val.imag = src.value.complex_val.imag;
+                                                                          break;
+        case TV_SYM:   value.sym_ptr    = src.value.sym_ptr;              break;
+        case TV_LIN:   value.fun_line   = src.value.fun_line;             break;
+        case TV_VAL:   value._apl_val() = src.value._apl_val();           break;
+        case TV_INDEX: value.index_val  = src.value.index_val;            break;
+        case TV_FUN:   value.function   = src.value.function;             break;
+        default:       FIXME;
+      }
+}
+//-----------------------------------------------------------------------------
+void
+copy_1(Token & dst, const Token & src, const char * loc)
+{
+   dst.clear(loc);
+   if (src.is_apl_val())
+      {
+        Value * val = src.get_apl_val_pointer();
+        if (val)
+           {
+             ADD_EVENT(val, VHE_TokCopy1, src.value_use_count(), loc);
+           }
+        else
+           {
+             ADD_EVENT(val, VHE_TokCopy1, -1, loc);
+           }
+      }
+
+   dst.copy_N(src);
+}
+//-----------------------------------------------------------------------------
+void
+move_1(Token & dst, Token & src, const char * loc)
+{
+   dst.clear(loc);
+   dst.copy_N(src);
+
+   if (src.is_apl_val())
+      {
+        Value * val = src.get_apl_val_pointer();
+	ADD_EVENT(val, VHE_TokMove1, src.value_use_count() - 1, loc);
+        src.clear(loc);
+      }
+}
+//-----------------------------------------------------------------------------
+void
+move_2(Token & dst, const Token & src, const char * loc)
+{
+   dst.clear(loc);
+   dst.copy_N(src);
+
+   if (src.is_apl_val())
+      {
+        Value * val = src.get_apl_val_pointer();
+        ADD_EVENT(val, VHE_TokMove2, src.value_use_count() - 1, loc);
+        ((Token &)src).clear(loc);
+      }
+}
+//-----------------------------------------------------------------------------
