@@ -22,6 +22,7 @@
 #include <string.h>
 
 #include "Common.hh"
+#include "FloatCell.hh"
 #include "Output.hh"
 #include "PrintBuffer.hh"
 #include "PrintOperator.hh"
@@ -56,9 +57,20 @@ UCS_string::UCS_string(const UTF8_string & utf)
         else if ((b0 & 0xE0) == 0xC0)   { more = 1; bx &= 0x1F; }
         else if ((b0 & 0xF0) == 0xE0)   { more = 2; bx &= 0x0F; }
         else if ((b0 & 0xF8) == 0xF0)   { more = 3; bx &= 0x0E; }
+        else if ((b0 & 0xFC) == 0xF8)   { more = 4; bx &= 0x07; }
+        else if ((b0 & 0xFE) == 0xFC)   { more = 5; bx &= 0x03; }
         else
            {
-             CERR << "Bad UTF8 sequence: " << HEX(b0) << "..." << endl;
+             CERR << "Bad UTF8 sequence: " << HEX(b0);
+             loop(j, 6)
+                {
+                  if ((i + j) >= utf.size())   break;
+                  const uint32_t bx = utf[i + j];
+                  if (bx & 0x80)   CERR << " " << HEX(bx);
+                  else             break;
+
+                }
+             CERR << endl;
              Backtrace::show(__FILE__, __LINE__);
              Assert(0 && "Internal error in UCS_string::UCS_string()");
              break;
@@ -79,64 +91,182 @@ UCS_string::UCS_string(const UTF8_string & utf)
               uni |= subc & 0x3F;
             }
 
-         *this += Unicode(bx | uni);
+         append(Unicode(bx | uni));
       }
 
    Log(LOG_char_conversion)
       CERR << "UCS_string::UCS_string(): ucs = " << *this << endl;
 }
 //-----------------------------------------------------------------------------
-UCS_string::UCS_string(APL_Float value, int mant_digs)
+UCS_string::UCS_string(APL_Float value, bool & scaled,
+                       const PrintContext & pctx)
    : Simple_string<Unicode>(0, 0)
 {
-int digs = mant_digs;
-   if (digs > 20)   digs = 20;
-   if (digs <  1)   digs = 1;
+int quad_pp = pctx.get_PP();
+   if (quad_pp > MAX_QUAD_PP)   quad_pp = MAX_QUAD_PP;
+   if (quad_pp < MIN_QUAD_PP)   quad_pp = MIN_QUAD_PP;
 
-char format[20];
-   snprintf(format, sizeof(format), "%%.%dE", digs - 1);
+const bool negative = (value < 0);
+   if (negative)   value = -value;
 
-char data[40];
-   snprintf(data, sizeof(data), format, value);
+int expo = 0;
 
-const char * d = data;
-
-   // skip leading 0s
-   //
-   while (*d == '0')   ++d;
-
-bool e_seen = false;
-bool skipping = false;   // skip leading 0s in exponent
-
-   while (*d)
+   if (value >= 10.0)   // large number, positive exponent
       {
-        const char cc = *d++;
-        if (cc == 'E')
+       while (value >= 1e16)
+             {
+               if (value > 1e307)
+                  { append(negative ? UNI_ASCII_MINUS : UNI_ASCII_PLUS);
+                    append(UNI_INFINITY);
+                    FloatCell::map_FC(*this);
+                    return;
+                  }
+               value *= 1e-16;   expo += 16;
+             }
+       while (value >= 1e4)    { value *= 1e-4;    expo +=  4; }
+       while (value >= 1e1)    { value *= 1e-1;    ++expo;     }
+      }
+   else if (value < 1.0)   // small number, negative exponent
+      {
+       while (value < 1e-16)
+             {
+               if (value < 1e-305)   // very small number: make it 0
+                  {
+                    append(UNI_ASCII_0);
+                    return;
+                  }
+               value *= 1e16;   expo -= 16;
+             }
+       while (value < 1e-4)    { value *= 1e4;    expo -=  4; }
+       while (value < 1.0)     { value *= 10.0;   --expo;     }
+      }
+
+   // In theory, at this point, 1.0 ≤ value < 10.0. In reality value can
+   // be outside, though, due to rounding errors.
+
+   // create a string with quad_pp + 1 significant digits.
+   // The last digit is used for rounding and then discarded.
+   //
+UCS_string digits;
+   loop(d, (quad_pp + 2))
+      {
+        if (value >= 10.0)
            {
-             e_seen = true;
-             skipping = true;
-             *this += UNI_ASCII_E;
+             digits.append((Unicode)(10 + '0'));
+             value = 0.0;
            }
-        else if (cc == '-')
-           {
-             *this += UNI_OVERBAR;
-           }
-        else if (!e_seen)   // copy mantissa as is
-           {
-             *this += Unicode(cc);
-           }
-        else if (cc == '+')
-           {
-           }
-        else if (cc == '0' && skipping)
+        else if (value < 0.0)
            {
+             digits.append((Unicode)('0'));
+             value = 0.0;
            }
         else
            {
-             skipping = false;
-             *this += Unicode(cc);
+             const int dig = (int)value;
+             value -= dig;
+             value *= 10;
+             digits.append((Unicode)(dig + '0'));
            }
       }
+
+   if (digits[0] != '0')   digits.pop();
+
+   // round last digit
+   //
+const Unicode last = digits.last();
+   digits.pop();
+
+   if (last >= '5')   digits.last() = (Unicode)(digits.last() + 1);
+ 
+   // adjust carries of 2nd to last digit
+   //
+   for (int d = digits.size() - 1; d > 0; --d)   // all but first
+       {
+        if (digits[d] > '9')
+           {
+             digits[d] =     (Unicode)(digits[d]     - 10);
+             digits[d - 1] = (Unicode)(digits[d - 1] +  1);
+           }
+       }
+
+   // adjust carry of 1st digit
+   //
+   if (digits[0] > '9')
+      {
+        digits[0] = (Unicode)(digits[0] - 10);
+        digits.insert_before(0, UNI_ASCII_1);
+        ++expo;
+        digits.pop();
+      }
+
+   // remove trailing zeros
+   //
+   while (digits.size() && digits.last() == UNI_ASCII_0)   digits.pop();
+
+   // force scaled format if:
+   //
+   // value < .00001     ( value has ≥ 5 leading zeroes)
+   // value > 10⋆quad_pp ( integer larger than ⎕PP)
+   //
+   if ((expo < -6) || (expo > quad_pp))   scaled = true;
+
+   if (negative)   append(UNI_OVERBAR);
+
+   if (scaled)
+      {
+        append(digits[0]);       // integer part
+        if (digits.size() > 1)   // fractional part
+           {
+             append(UNI_ASCII_FULLSTOP);
+             loop(d, (digits.size() - 1))   append(digits[d + 1]);
+           }
+        if (expo < 0)
+           {
+             append(UNI_ASCII_E);
+             append(UNI_OVERBAR);
+             append_number(-expo);
+           }
+        else if (expo > 0)
+           {
+             append(UNI_ASCII_E);
+             append_number(expo);
+           }
+        else if (!(pctx.get_style() & PST_NO_EXPO_0)) // expo == 0
+           {
+             append(UNI_ASCII_E);
+             append(UNI_ASCII_0);
+           }
+      }
+   else
+      {
+        if (expo < 0)   // 0.000...
+           {
+             append(UNI_ASCII_0);
+             append(UNI_ASCII_FULLSTOP);
+             loop(e, (-(expo + 1)))   append(UNI_ASCII_0);
+             append(digits);
+           }
+        else   // expo >= 0
+           {
+             loop(e, expo + 1)
+                {
+                  if (e < digits.size())   append(digits[e]);
+                  else                     append(UNI_ASCII_0);
+                }
+
+             if ((expo + 1) < digits.size())   // there are fractional digits
+                {
+                  append(UNI_ASCII_FULLSTOP);
+                  for (int e = expo + 1; e < digits.size(); ++e)
+                     {
+                       if (e < digits.size())   append(digits[e]);
+                       else                     break;
+                     }
+                }
+           }
+      }
+
+   FloatCell::map_FC(*this);
 }
 //-----------------------------------------------------------------------------
 UCS_string::UCS_string(const PrintBuffer & pb, Rank rank)
@@ -156,24 +286,22 @@ int col = 0;
          const int brkp = pb.get_break_points()[b];
          add_chunk(pb, col, brkp - col);
          col = brkp;
-         loop(r, rank)   *this += UNI_ASCII_LF;
+         loop(r, rank)   append(UNI_ASCII_LF);
        }
 
    add_chunk(pb, col, pb.get_width(0) - col);
 }
 //-----------------------------------------------------------------------------
 /// constructor
-UCS_string::UCS_string(Value * value)
+UCS_string::UCS_string(const Value & value)
    : Simple_string<Unicode>(0, 0)
 {
-   Assert(value);
+   if (value.get_rank() > 1) RANK_ERROR;
 
-   if (value->get_rank() > 1) RANK_ERROR;
-
-const ShapeItem ec = value->element_count();
+const ShapeItem ec = value.element_count();
    reserve(ec);
 
-   loop(e, ec)   *this += value->get_ravel(e).get_char_value();
+   loop(e, ec)   append(value.get_ravel(e).get_char_value());
 }
 //-----------------------------------------------------------------------------
 UCS_string::UCS_string(istream & in)
@@ -184,7 +312,7 @@ UCS_string::UCS_string(istream & in)
         const Unicode uni = UTF8_string::getc(in);
         if (uni == Invalid_Unicode)   return;
         if (uni == UNI_ASCII_LF)      return;
-        *this += uni;
+        append(uni);
       }
 }
 //-----------------------------------------------------------------------------
@@ -193,11 +321,11 @@ UCS_string::add_chunk(const PrintBuffer & pb, int from, size_t width)
 {
    loop(y, pb.get_height())
        {
-         if (y)   *this += UNI_ASCII_LF;
+         if (y)   append(UNI_ASCII_LF);
 
          UCS_string ucs;
-         if (from)   ucs += UCS_string(UTF8_string("      "));
-         ucs += UCS_string (pb.get_line(y), from, width);
+         if (from)   ucs.append(UCS_string(UTF8_string("      ")));
+         ucs.append(UCS_string (pb.get_line(y), from, width));
 
          // remove trailing pad chars from align() and append_string(),
          // but leave other pad chars intact.
@@ -239,8 +367,8 @@ UCS_string::add_chunk(const PrintBuffer & pb, int from, size_t width)
          loop(u, ucs.size())
             {
               Unicode uni = ucs[u];
-              if (is_pad(uni))   uni = UNI_ASCII_SPACE;
-              *this += uni;
+              if (is_pad_char(uni))   uni = UNI_ASCII_SPACE;
+              append(uni);
             }
        }
 }
@@ -249,7 +377,7 @@ void
 UCS_string::copy_black(UCS_string & dest, int & idx) const
 {
    while (idx < size() && operator[](idx) <= ' ')   ++idx;
-   while (idx < size() && operator[](idx) >  ' ')   dest += operator[](idx++);
+   while (idx < size() && operator[](idx) >  ' ')   dest.append(operator[](idx++));
    while (idx < size() && operator[](idx) <= ' ')   ++idx;
 }
 //-----------------------------------------------------------------------------
@@ -341,8 +469,8 @@ UCS_string ret;
    loop(s, size())
       {
         Unicode uni = (*this)[s];
-        if (is_pad(uni))   uni = UNI_ASCII_SPACE;
-        ret += uni;
+        if (is_pad_char(uni))   uni = UNI_ASCII_SPACE;
+        ret.append(uni);
       }
 
    return ret;
@@ -355,7 +483,7 @@ UCS_string ret;
    loop(s, size())
       {
         Unicode uni = (*this)[s];
-        if (!is_pad(uni))   ret += uni;
+        if (!is_pad_char(uni))   ret.append(uni);
       }
 
    return ret;
@@ -386,12 +514,12 @@ UCS_string
 UCS_string::reverse() const
 {
 UCS_string ret;
-   for (int s = size(); s > 0;)   ret += (*this)[--s];
+   for (int s = size(); s > 0;)   ret.append((*this)[--s]);
    return ret;
 }
 //-----------------------------------------------------------------------------
 void
-UCS_string::app(const UTF8 * str)
+UCS_string::append_utf8(const UTF8 * str)
 {
 const size_t len = strlen((const char *)str);
 const UTF8_string utf(str, len);
@@ -401,19 +529,19 @@ const UCS_string ucs(utf);
 }
 //-----------------------------------------------------------------------------
 void
-UCS_string::append_string(const char * str)
+UCS_string::append_ascii(const char * str)
 {
-   while (*str)   *this += Unicode(*str++);
+   while (*str)   append(Unicode(*str++));
 }
 //-----------------------------------------------------------------------------
 void
-UCS_string::append_number(long long num)
+UCS_string::append_number(ShapeItem num)
 {
 char cc[40];
    snprintf(cc, sizeof(cc) - 1, "%lld", num);
    loop(c, sizeof(cc))
       {
-        if (cc[c])   *this += Unicode(cc[c]);
+        if (cc[c])   append(Unicode(cc[c]));
         else         break;
       }
 }
@@ -425,7 +553,7 @@ char cc[60];
    snprintf(cc, sizeof(cc) - 1, "%lf", num);
    loop(c, sizeof(cc))
       {
-        if (cc[c])   *this += Unicode(cc[c]);
+        if (cc[c])   append(Unicode(cc[c]));
         else         break;
       }
 }
@@ -453,7 +581,7 @@ size_t max_len = 0;
         else
            {
              if (uni != UNI_ASCII_CR)         // ignore \r.
-                result[result.size() - 1] += uni;
+                result[result.size() - 1].append(uni);
            }
       }
 
@@ -516,7 +644,8 @@ operator << (ostream & os, Unicode uni)
 ostream &
 operator << (ostream & os, const UCS_string & ucs)
 {
-   loop(c, ucs.size())   os << ucs[c];
+UTF8_string utf(ucs);
+   os << utf.c_str();
    return os;
 }
 //-----------------------------------------------------------------------------
@@ -545,7 +674,7 @@ int * d = digits;
       }
 
 UCS_string ret;
-   while (d > digits)   ret += Unicode(UNI_ASCII_0 + *--d);
+   while (d > digits)   ret.append(Unicode(UNI_ASCII_0 + *--d));
    return ret;
 }
 //-----------------------------------------------------------------------------
@@ -570,9 +699,9 @@ long double fract;
    val = initial_fract;
 
 UCS_string ret;
-   if (d == digits)   ret += UNI_ASCII_0;   // 0.xxx
+   if (d == digits)   ret.append(UNI_ASCII_0);   // 0.xxx
 
-   while (d > digits)   ret += Unicode(UNI_ASCII_0 + *--d);
+   while (d > digits)   ret.append(Unicode(UNI_ASCII_0 + *--d));
    return ret;
 }
 //-----------------------------------------------------------------------------
@@ -583,18 +712,18 @@ UCS_string ret;
 
    if (v == 0.0)
       {
-        ret += UNI_ASCII_0;
+        ret.append(UNI_ASCII_0);
         if (fract_digits)   // unless integer only
            {
-             ret += UNI_ASCII_FULLSTOP;
-             loop(f, fract_digits)   ret += UNI_ASCII_0;
+             ret.append(UNI_ASCII_FULLSTOP);
+             loop(f, fract_digits)   ret.append(UNI_ASCII_0);
            }
-        ret += UNI_ASCII_E;
-        ret += UNI_ASCII_0;
+        ret.append(UNI_ASCII_E);
+        ret.append(UNI_ASCII_0);
         return ret;
       }
 
-   if (v < 0)   { ret += UNI_OVERBAR;   v = - v; }
+   if (v < 0)   { ret.append(UNI_OVERBAR);   v = - v; }
 
 int expo = 0;
    while (v >= 1.0E1)
@@ -670,9 +799,9 @@ int expo = 0;
 
    // print mantissa in fixed format
    //
-   ret += from_double_fixed_prec(v, fract_digits);
-   ret += UNI_ASCII_E;
-   ret += from_int(expo);
+   ret.append(from_double_fixed_prec(v, fract_digits));
+   ret.append(UNI_ASCII_E);
+   ret.append(from_int(expo));
 
    return ret;
 }
@@ -682,17 +811,17 @@ UCS_string::from_double_fixed_prec(double v, unsigned int fract_digits)
 {
 UCS_string ret;
 
-   if (v < 0)   { ret += UNI_OVERBAR;   v = - v; }
+   if (v < 0)   { ret.append(UNI_OVERBAR);   v = - v; }
 
-   ret += from_big(v);   // leaves fractional part of v in v
+   ret.append(from_big(v));   // leaves fractional part of v in v
 
-   ret += UNI_ASCII_FULLSTOP;
+   ret.append(UNI_ASCII_FULLSTOP);
 
    loop(f, fract_digits + 1)
       {
         v *= 10;
         const int vv = v;
-        ret += Unicode(UNI_ASCII_0 + vv);
+        ret.append(Unicode(UNI_ASCII_0 + vv));
         v -= vv;
       }
 
