@@ -39,7 +39,7 @@
 
 //-----------------------------------------------------------------------------
 UserFunction::UserFunction(const UCS_string txt, int & error_line,
-                           const char * loc)
+                           bool keep_existing, const char * loc)
   : Executable(txt, loc),
     Function(ID_USER_SYMBOL, TOK_FUN2),
     sym_Z(0),
@@ -67,6 +67,7 @@ UserFunction::UserFunction(const UCS_string txt, int & error_line,
    if (!sym_FUN->can_be_defined())   DEFN_ERROR;
      
 Function * old_function = sym_FUN->get_function();
+   if (old_function && keep_existing)    DEFN_ERROR;
 
    for (int l = 1; l < get_text_size(); ++l)
       {
@@ -74,8 +75,7 @@ Function * old_function = sym_FUN->get_function();
         line_starts.push_back(Function_PC(body.size()));
 
         const UCS_string & text = get_text(l);
-        const Token_string tos = parse_body_line(Function_Line(l), text, loc);
-        loop(t, tos.size())   body += tos[t];
+        parse_body_line(Function_Line(l), text, loc);
       }
 
    Log(LOG_UserFunction__fix)
@@ -87,21 +87,13 @@ Function * old_function = sym_FUN->get_function();
    // let [0] be the end of the function.
    line_starts[0] = Function_PC(body.size());
 
-   if (sym_Z)     body += Token(TOK_RETURN_SYMBOL, sym_Z);
-   else           body += Token(TOK_RETURN_VOID);
+   if (sym_Z)     body.append(Token(TOK_RETURN_SYMBOL, sym_Z), LOC);
+   else           body.append(Token(TOK_RETURN_VOID), LOC);
 
    check_duplicate_symbols();
 
    if (sym_LO)   sym_FUN->set_nc(NC_OPERATOR, this);
-   else               sym_FUN->set_nc(NC_FUNCTION, this);
-
-   // fix program constants so that they don't get deleted.
-   //
-   loop(b, body.size())
-       {
-         Token & t = body[b];
-         if (t.get_ValueType() == TV_VAL)   t.get_apl_val()->set_shared();
-       }
+   else          sym_FUN->set_nc(NC_FUNCTION, this);
 
    if (old_function)
       {
@@ -110,7 +102,7 @@ Function * old_function = sym_FUN->get_function();
         delete old_ufun;
       }
 
-   error_line = -1;   // no error
+   error_line = -1;   // assume no error
 }
 //-----------------------------------------------------------------------------
 UserFunction::~UserFunction()
@@ -133,7 +125,7 @@ UserFunction::eval_()
 
    if (sym_B)   SYNTAX_ERROR;   // not defined niladic
 
-   Workspace::the_workspace->push_SI(this, LOC);
+   Workspace::push_SI(this, LOC);
 
    eval_common();
 
@@ -146,10 +138,10 @@ UserFunction::eval_B(Value_P B)
    Log(LOG_UserFunction__enter_leave)
       {
         CERR << "Function " << get_name() << " calls eval_B("
-             << B << ")" << endl;
+             << Token(TOK_APL_VALUE1, B) << ")" << endl;
       }
 
-   Workspace::the_workspace->push_SI(this, LOC);
+   Workspace::push_SI(this, LOC);
 
    if (sym_LO)    SYNTAX_ERROR;   // defined as operator
 
@@ -168,10 +160,10 @@ UserFunction::eval_XB(Value_P X, Value_P B)
    Log(LOG_UserFunction__enter_leave)
       {
         CERR << "Function " << get_name() << " calls eval_B("
-             << B << ")" << endl;
+             << Token(TOK_APL_VALUE1, B) << ")" << endl;
       }
 
-   Workspace::the_workspace->push_SI(this, LOC);
+   Workspace::push_SI(this, LOC);
 
    if (sym_LO)    SYNTAX_ERROR;   // defined as operator
 
@@ -190,10 +182,11 @@ UserFunction::eval_AB(Value_P A, Value_P B)
    Log(LOG_UserFunction__enter_leave)
       {
         CERR << "Function " << get_name() << " calls eval_AB("
-             << A << ", " << B << ")" << endl;
+             << Token(TOK_APL_VALUE1, A) << ", "
+             << Token(TOK_APL_VALUE1, B) << ")" << endl;
       }
 
-   Workspace::the_workspace->push_SI(this, LOC);
+   Workspace::push_SI(this, LOC);
 
    if (sym_LO)    SYNTAX_ERROR;   // defined as operator
 
@@ -212,10 +205,11 @@ UserFunction::eval_AXB(Value_P A, Value_P X, Value_P B)
    Log(LOG_UserFunction__enter_leave)
       {
         CERR << "Function " << get_name() << " calls eval_AB("
-             << A << ", " << B << ")" << endl;
+             << Token(TOK_APL_VALUE1, A) << ", "
+             << Token(TOK_APL_VALUE1, B) << ")" << endl;
       }
 
-   Workspace::the_workspace->push_SI(this, LOC);
+   Workspace::push_SI(this, LOC);
 
    if (sym_LO)    SYNTAX_ERROR;   // defined as operator
 
@@ -238,7 +232,7 @@ UserFunction::eval_LB(Token & LO, Value_P B)
              << Token(TOK_APL_VALUE1, B) << ")" << endl;
       }
 
-   Workspace::the_workspace->push_SI(this, LOC);
+   Workspace::push_SI(this, LOC);
 
    if (sym_RO)    SYNTAX_ERROR;   // dyadic operator called monadically
 
@@ -264,7 +258,7 @@ UserFunction::eval_LXB(Token & LO, Value_P X, Value_P B)
              << Token(TOK_APL_VALUE1, B) << ")" << endl;
       }
 
-   Workspace::the_workspace->push_SI(this, LOC);
+   Workspace::push_SI(this, LOC);
 
    if (sym_RO)    SYNTAX_ERROR;   // dyadic operator called monadically
 
@@ -290,7 +284,7 @@ UserFunction::eval_ALB(Value_P A, Token & LO, Value_P B)
              << Token(TOK_APL_VALUE1, B) << ")" << endl;
       }
 
-   Workspace::the_workspace->push_SI(this, LOC);
+   Workspace::push_SI(this, LOC);
 
    if (!sym_A)    SYNTAX_ERROR;   // monadic function called dyadically
    if (sym_X)   sym_X->push();
@@ -318,7 +312,7 @@ UserFunction::eval_ALXB(Value_P A, Token & LO, Value_P X, Value_P B)
              << Token(TOK_APL_VALUE1, B) << ")" << endl;
       }
 
-   Workspace::the_workspace->push_SI(this, LOC);
+   Workspace::push_SI(this, LOC);
 
    if (!sym_A)    SYNTAX_ERROR;   // monadic function called dyadically
    if (sym_RO)    SYNTAX_ERROR;   // defined as dyadic operator
@@ -346,7 +340,7 @@ UserFunction::eval_LRB(Token & LO, Token & RO, Value_P B)
              << Token(TOK_APL_VALUE1, B) << ")" << endl;
       }
 
-   Workspace::the_workspace->push_SI(this, LOC);
+   Workspace::push_SI(this, LOC);
 
    if (!sym_RO)    SYNTAX_ERROR;   // not defined as dyadic operator
 
@@ -376,7 +370,7 @@ UserFunction::eval_LRXB(Token & LO, Token & RO, Value_P X, Value_P B)
              << Token(TOK_APL_VALUE1, B) << ")" << endl;
       }
 
-   Workspace::the_workspace->push_SI(this, LOC);
+   Workspace::push_SI(this, LOC);
 
    if (!sym_RO)    SYNTAX_ERROR;   // not defined as dyadic operator
 
@@ -405,7 +399,7 @@ UserFunction::eval_ALRB(Value_P A, Token & LO, Token & RO, Value_P B)
              << Token(TOK_APL_VALUE1, B) << ")" << endl;
       }
 
-   Workspace::the_workspace->push_SI(this, LOC);
+   Workspace::push_SI(this, LOC);
 
    if (!sym_A)    SYNTAX_ERROR;    // defined monadic called dyadic
    if (sym_X)      sym_X->push();
@@ -437,7 +431,7 @@ UserFunction::eval_ALRXB(Value_P A, Token & LO, Token & RO,
              << Token(TOK_APL_VALUE1, B) << ")" << endl;
       }
 
-   Workspace::the_workspace->push_SI(this, LOC);
+   Workspace::push_SI(this, LOC);
 
    if (!sym_A)    SYNTAX_ERROR;    // defined monadic called dyadic
    if (!sym_RO)    SYNTAX_ERROR;   // defined monadic op called dyadic
@@ -489,8 +483,9 @@ UserFunction::pop_local_vars() const
    if (sym_LO)   sym_LO->pop(true);
    if (sym_RO)   sym_RO->pop(true);
 
-   if (sym_Z)   return sym_Z->pop(false);
-   return 0;
+   if (!sym_Z)   return Value_P();
+
+   return sym_Z->pop(false);
 }
 //-----------------------------------------------------------------------------
 void
@@ -501,27 +496,27 @@ UCS_string & message_2 = error.error_message_2;
    if (sym_A)
       {
         Value_P val_A = sym_A->get_value();
-        if (val_A)
+        if (!!val_A)
            {
              PrintContext pctx(PR_APL_FUN);
              PrintBuffer pb(*val_A, pctx);
-             message_2 += UCS_string(pb, 1);
-             message_2 += UNI_ASCII_SPACE;
+             message_2.append(UCS_string(pb, 1));
+             message_2.append(UNI_ASCII_SPACE);
            }
       }
 
    Assert(sym_FUN);
-   message_2 += sym_FUN->get_name();
+   message_2.append(sym_FUN->get_name());
 
    if (sym_B)
       {
         Value_P val_B = sym_B->get_value();
-        if (val_B)
+        if (!!val_B)
            {
-             message_2 += UNI_ASCII_SPACE;
+             message_2.append(UNI_ASCII_SPACE);
              PrintContext pctx(PR_APL_FUN);
              PrintBuffer pb(*val_B, pctx);
-             message_2 += UCS_string(pb, 1);
+             message_2.append(UCS_string(pb, 1));
            }
       }
 
@@ -647,7 +642,7 @@ UCS_string ucs(utf);
    close(in);
 
 int error_line = -1;
-   fun = fix(ucs, error_line, LOC);
+   fun = fix(ucs, error_line, false, LOC);
 }
 //-----------------------------------------------------------------------------
 Function_PC
@@ -659,39 +654,54 @@ UserFunction::pc_for_line(int l) const
 }
 //-----------------------------------------------------------------------------
 UserFunction *
-UserFunction::fix(const UCS_string & text, int & error_line, const char * loc)
+UserFunction::fix(const UCS_string & text, int & error_line,
+                  bool keep_existing,  const char * loc)
 {
    Log(LOG_UserFunction__fix)
-      CERR << "fix pmode=user function:" << endl << text
-           <<  endl << "------------------- fix --" << endl;
+      CERR << "fix pmode=user function:" << endl << text << endl
+           <<  "------------------- fix --" << endl;
 
-   if (Workspace::the_workspace->SI_top())
-      Workspace::the_workspace->SI_top()->set_safe_execution(true);
+   if (Workspace::SI_top())
+      Workspace::SI_top()->set_safe_execution(true);
 
 UserFunction * fun = 0;
    try
       {
-        fun = new UserFunction(text, error_line, loc);
+        fun = new UserFunction(text, error_line, keep_existing, loc);
       }
    catch (Error err)
       {
         err.print(CERR);
-        if (Workspace::the_workspace->SI_top())
-           Workspace::the_workspace->SI_top()->set_safe_execution(false);
+        if (Workspace::SI_top())
+           Workspace::SI_top()->set_safe_execution(false);
         return 0;
       }
    catch (...)
       {
         CERR << __FUNCTION__ << "caught something unexpected at " LOC << endl;
-        if (Workspace::the_workspace->SI_top())
-           Workspace::the_workspace->SI_top()->set_safe_execution(false);
+        if (Workspace::SI_top())
+           Workspace::SI_top()->set_safe_execution(false);
         return 0;
       }
 
-   Log(LOG_UserFunction__fix)   fun->print(CERR);
+   Log(LOG_UserFunction__fix)
+      {
+        CERR << " addr " << (const void *)fun << endl;
+        fun->print(CERR);
+      }
 
-   if (Workspace::the_workspace->SI_top())
-      Workspace::the_workspace->SI_top()->set_safe_execution(false);
+   if (Workspace::SI_top())
+      Workspace::SI_top()->set_safe_execution(false);
+
+   // UserFunction::UserFunction() sets error line to -1 if successful.
+   // We may have created a UserFunction but with an error. In this case
+   // we delete it here.
+   //
+   if (error_line != -1)
+      {
+        delete fun;
+        return 0;
+      }
 
    return fun;
 }
@@ -797,7 +807,7 @@ UserFunction::parse_header_line(UCS_string header_line)
    // add a semicolon as a guaranteed end marker.
    // This to avoids checks of the header token count
    // 
-   header_line += Unicode(';');
+   header_line.append(Unicode(';'));
 
 Token_string tos;
    {
@@ -960,7 +970,7 @@ UCS_string
 UserFunction::get_name_and_line(Function_PC pc) const
 {
 UCS_string ret = sym_FUN->get_name();
-   ret += UNI_ASCII_L_BRACK;
+   ret.append(UNI_ASCII_L_BRACK);
 
    // pc may point to the next token already. If that is the case then
    // we go back one token.
@@ -969,7 +979,7 @@ UCS_string ret = sym_FUN->get_name();
 
 const Function_Line line = get_line(pc);
    ret.append_number(line);
-   ret += UNI_ASCII_R_BRACK;
+   ret.append(UNI_ASCII_R_BRACK);
    return ret;
 }
 //-----------------------------------------------------------------------------
@@ -991,9 +1001,9 @@ UserFunction::canonical(bool with_lines) const
 UCS_string ucs;
    loop(t, text.size())
       {
-        if (with_lines)   ucs += line_prefix(Function_Line(t));
-        ucs += text[t];
-        ucs += UNI_ASCII_LF;
+        if (with_lines)   ucs.append(line_prefix(Function_Line(t)));
+        ucs.append(text[t]);
+        ucs.append(UNI_ASCII_LF);
       }
 
    return ucs;
@@ -1014,15 +1024,15 @@ UCS_string
 UserFunction::line_prefix(Function_Line l)
 {
 UCS_string ucs;
-   ucs += UNI_ASCII_L_BRACK;
-   if (l < 10)   ucs += UNI_ASCII_SPACE;
+   ucs.append(UNI_ASCII_L_BRACK);
+   if (l < 10)   ucs.append(UNI_ASCII_SPACE);
 
 char cc[40];
    snprintf(cc, sizeof(cc), "%d", l);
-   for (const char * s = cc; *s; ++s)   ucs += Unicode(*s);
+   for (const char * s = cc; *s; ++s)   ucs.append(Unicode(*s));
 
-   ucs += UNI_ASCII_R_BRACK;
-   ucs += UNI_ASCII_SPACE;
+   ucs.append(UNI_ASCII_R_BRACK);
+   ucs.append(UNI_ASCII_SPACE);
    return ucs;
 }
 //-----------------------------------------------------------------------------
