@@ -28,69 +28,71 @@ using namespace std;
 #include "Command.hh"
 #include "Input.hh"
 #include "LibPaths.hh"
+#include "main.hh"
 #include "Output.hh"
+#include "Quad_FX.hh"
 #include "Quad_TF.hh"
+#include "TestFiles.hh"
 #include "UserFunction.hh"
 #include "Workspace.hh"
 
-Workspace * Workspace::the_workspace = 0;
+// Workspace::the_workspace is defined in StaticObjects.cc
 
-//-----------------------------------------------------------------------------
-Workspace::Workspace()
-   : WS_name("CLEAR WS"),
-//   prompt("-----> "),
-     prompt("      "),
-     top_SI(0)
-{
-   if (the_workspace == 0)   the_workspace = this;
-}
-//-----------------------------------------------------------------------------
-Workspace::~Workspace()
-{
-   if (the_workspace == this)   the_workspace = 0;
-}
 //-----------------------------------------------------------------------------
 void
 Workspace::push_SI(const Executable * fun, const char * loc)
 {
-   Assert(fun);
+   Assert1(fun);
 
-   top_SI = new StateIndicator(fun, top_SI);
+
+   if (Quad_SYL::si_depth_limit && SI_top() &&
+       Quad_SYL::si_depth_limit <= SI_top()->get_level())
+      {
+        Workspace::more_error() = UCS_string(
+        "the system limit on SI depth (as set in ⎕SYL) was reached\n"
+        "(and to avoid lock-up, the limit in ⎕SYL was automatically cleared).");
+
+        Quad_SYL::si_depth_limit = 0;
+        attention_raised = interrupt_raised = true;
+      }
+
+   the_workspace.top_SI = new StateIndicator(fun, SI_top());
 
    Log(LOG_StateIndicator__push_pop)
       {
-        CERR << "Push  SI[" <<  top_SI->get_level() << "] "
+        CERR << "Push  SI[" <<  SI_top()->get_level() << "] "
              << "pmode=" << fun->get_parse_mode()
              << " exec=" << fun << " "
              << fun->get_name();
 
-        CERR << " new SI is " << (const void *)top_SI << " at " << loc << endl;
+        CERR << " new SI is " << (const void *)SI_top()
+             << " at " << loc << endl;
       }
 }
 //-----------------------------------------------------------------------------
 void
 Workspace::pop_SI(const char * loc)
 {
-   Assert(top_SI);
-   Assert1(top_SI->get_executable());
+   Assert(SI_top());
+const Executable * exec = SI_top()->get_executable();
+   Assert1(exec);
 
-const Executable * exec = top_SI->get_executable();
 
    Log(LOG_StateIndicator__push_pop)
       {
-        CERR << "Pop  SI[" <<  top_SI->get_level() << "] "
+        CERR << "Pop  SI[" <<  SI_top()->get_level() << "] "
              << "pmode=" << exec->get_parse_mode()
              << " exec=" << exec << " ";
 
         if (exec->get_ufun())   CERR << exec->get_ufun()->get_name();
-        else                    CERR << top_SI->get_parse_mode_name();
-        CERR << " " << (const void *)top_SI << " at " << loc << endl;
+        else                    CERR << SI_top()->get_parse_mode_name();
+        CERR << " " << (const void *)SI_top() << " at " << loc << endl;
       }
 
    // remove the top SI
    //
-StateIndicator * del = top_SI;
-   top_SI = top_SI->get_parent();
+StateIndicator * del = SI_top();
+   the_workspace.top_SI = del->get_parent();
    delete del;
 }
 //-----------------------------------------------------------------------------
@@ -106,7 +108,7 @@ Workspace::SI_top_fun()
 }
 //-----------------------------------------------------------------------------
 const StateIndicator *
-Workspace::SI_top_error() const
+Workspace::SI_top_error()
 {
    for (const StateIndicator * si = SI_top(); si; si = si->get_parent())
        {
@@ -127,15 +129,18 @@ Workspace::immediate_execution(bool exit_on_error)
            }
          catch (Error err)
             {
-              CERR << __FUNCTION__ << "() caught APL error "
-                   << HEX(err.error_code) << " ("
-                   << err.error_name(err.error_code) << ")" << endl;
+              if (!err.get_print_loc())
+                 {
+                   if (err.error_code != E_DEFN_ERROR)
+                      {
+                        err.print_em(UERR, LOC);
+                        CERR << __FUNCTION__ << "() caught APL error "
+                             << HEX(err.error_code) << " ("
+                             << err.error_name(err.error_code) << ")" << endl;
 
-              const Error * error = get_error();
-              if (error)   get_error()->print(CERR);
-
-              TestFiles::apl_error(LOC);
-              err.print_em(CERR, LOC);
+                        TestFiles::apl_error(LOC);
+                      }
+                 }
               if (exit_on_error)   return Token(TOK_OFF);
             }
          catch (...)
@@ -153,7 +158,7 @@ Workspace::immediate_execution(bool exit_on_error)
 NamedObject *
 Workspace::lookup_existing_name(const UCS_string & name)
 {
-   if (name[0] == UNI_QUAD_QUAD)   // distinguished name
+   if (Avec::is_quad(name[0]))   // distinguished name
       {
         int len;
         Token tok = get_quad(name, len);
@@ -168,7 +173,8 @@ Workspace::lookup_existing_name(const UCS_string & name)
       }
 
    // user defined variable or function
-   return symbol_table.lookup_existing_name(name);
+   //
+   return the_workspace.symbol_table.lookup_existing_name(name);
 }
 //-----------------------------------------------------------------------------
 Symbol *
@@ -176,7 +182,7 @@ Workspace::lookup_existing_symbol(const UCS_string & symbol_name)
 {
    if (symbol_name.size() == 0)   return 0;
 
-   if (symbol_name[0] == UNI_QUAD_QUAD)   // distinguished name
+   if (Avec::is_quad(symbol_name[0]))   // distinguished name
       {
         int len;
         Token tok = get_quad(symbol_name, len);
@@ -186,13 +192,13 @@ Workspace::lookup_existing_symbol(const UCS_string & symbol_name)
         return tok.get_sym_ptr();
       }
 
-   return symbol_table.lookup_existing_symbol(symbol_name);
+   return the_workspace.symbol_table.lookup_existing_symbol(symbol_name);
 }
 //-----------------------------------------------------------------------------
 Token
 Workspace::get_quad(UCS_string ucs, int & len)
 {
-   if (ucs.size() == 0 || ucs[0] != UNI_QUAD_QUAD)
+   if (ucs.size() == 0 || !Avec::is_quad(ucs[0]))
      {
        len = 0;
        return Token();
@@ -203,7 +209,7 @@ const Unicode av_1 = (ucs.size() > 2) ? ucs[2] : Invalid_Unicode;
 const Unicode av_2 = (ucs.size() > 3) ? ucs[3] : Invalid_Unicode;
 
 #define var(t, l) { len = l + 1; \
-   return Token(TOK_QUAD_ ## t, &v_quad_ ## t); }
+   return Token(TOK_QUAD_ ## t, &the_workspace.v_quad_ ## t); }
 
 #define f0(t, l) { len = l + 1; \
    return Token(TOK_QUAD_ ## t, &Quad_ ## t::fun); }
@@ -303,6 +309,10 @@ const Unicode av_2 = (ucs.size() > 3) ? ucs[3] : Invalid_Unicode;
                   if (av_2 == UNI_ASCII_R)   f1 (SVR, 3)
                   if (av_2 == UNI_ASCII_S)   f1 (SVS, 3)
                 }
+             if (av_1 == UNI_ASCII_Y)
+                {
+                  if (av_2 == UNI_ASCII_L)   var(SYL, 3)
+                }
              break;
 
         case UNI_ASCII_T:
@@ -332,18 +342,18 @@ const Unicode av_2 = (ucs.size() > 3) ? ucs[3] : Invalid_Unicode;
 }
 //-----------------------------------------------------------------------------
 StateIndicator *
-Workspace:: oldest_exec(const Executable * exec) const
+Workspace:: oldest_exec(const Executable * exec)
 {
 StateIndicator * ret = 0;
 
-   for (StateIndicator * si = top_SI; si; si = si->get_parent())
+   for (StateIndicator * si = SI_top(); si; si = si->get_parent())
        if (exec == si->get_executable())   ret = si;   // maybe not yet oldest
 
    return ret;
 }
 //-----------------------------------------------------------------------------
 bool
-Workspace::is_called(const UCS_string & funname) const
+Workspace::is_called(const UCS_string & funname)
 {
    for (const StateIndicator * si = SI_top(); si; si = si->get_parent())
       {
@@ -363,15 +373,15 @@ Workspace::write_OUT(FILE * out, uint64_t & seq, const vector<UCS_string>
    //
    if (objects.size() == 0)   // all objects plus some system variables
       {
-        v_quad_CT.write_OUT(out, seq);
-        v_quad_FC.write_OUT(out, seq);
-        v_quad_IO.write_OUT(out, seq);
-        v_quad_LX.write_OUT(out, seq);
-        v_quad_PP.write_OUT(out, seq);
-        v_quad_PR.write_OUT(out, seq);
-        v_quad_RL.write_OUT(out, seq);
+        get_v_quad_CT().write_OUT(out, seq);
+        get_v_quad_FC().write_OUT(out, seq);
+        get_v_quad_IO().write_OUT(out, seq);
+        get_v_quad_LX().write_OUT(out, seq);
+        get_v_quad_PP().write_OUT(out, seq);
+        get_v_quad_PR().write_OUT(out, seq);
+        get_v_quad_RL().write_OUT(out, seq);
 
-        symbol_table.write_all_symbols(out, seq);
+        get_symbol_table().write_all_symbols(out, seq);
       }
    else                       // only specified objects
       {
@@ -409,56 +419,56 @@ Workspace::write_OUT(FILE * out, uint64_t & seq, const vector<UCS_string>
 }
 //-----------------------------------------------------------------------------
 void
-Workspace::unmark_all_values() const
+Workspace::unmark_all_values()
 {
    // unmark user defined symbols
    //
-   symbol_table.unmark_all_values();
+   the_workspace.symbol_table.unmark_all_values();
 
    // unmark system variables
    //
-#define rw_sv_def(x) v_quad_ ## x.unmark_all_values();
-#define ro_sv_def(x) v_quad_ ## x.unmark_all_values();
+#define rw_sv_def(x) the_workspace.v_quad_ ## x.unmark_all_values();
+#define ro_sv_def(x) the_workspace.v_quad_ ## x.unmark_all_values();
 #include "SystemVariable.def"
-   v_quad_QUAD .unmark_all_values();
-   v_quad_QUOTE.unmark_all_values();
+   the_workspace.v_quad_QUAD .unmark_all_values();
+   the_workspace.v_quad_QUOTE.unmark_all_values();
 
    // unmark token reachable vi SI stack
    //
-   for (StateIndicator * si = top_SI; si; si = si->get_parent())
+   for (StateIndicator * si = SI_top(); si; si = si->get_parent())
        si->unmark_all_values();
 
    // unmark token in (failed) ⎕EX functions
    //
-   loop(f, expunged_functions.size())
-      expunged_functions[f]->unmark_all_values();
+   loop(f, the_workspace.expunged_functions.size())
+      the_workspace.expunged_functions[f]->unmark_all_values();
 }
 //-----------------------------------------------------------------------------
 int
-Workspace::show_owners(ostream & out, Value_P value) const
+Workspace::show_owners(ostream & out, const Value & value)
 {
 int count = 0;
 
    // user defined variabes
    //
-   count += symbol_table.show_owners(out, value);
+   count += the_workspace.symbol_table.show_owners(out, value);
 
    // system variabes
    //
-#define rw_sv_def(x) count += v_quad_ ## x.show_owners(out, value);
-#define ro_sv_def(x) count += v_quad_ ## x.show_owners(out, value);
+#define rw_sv_def(x) count += get_v_quad_ ## x().show_owners(out, value);
+#define ro_sv_def(x) count += get_v_quad_ ## x().show_owners(out, value);
 #include "SystemVariable.def"
-   count += v_quad_QUAD .show_owners(out, value);
-   count += v_quad_QUOTE.show_owners(out, value);
+   count += the_workspace.v_quad_QUAD .show_owners(out, value);
+   count += the_workspace.v_quad_QUOTE.show_owners(out, value);
 
-   for (StateIndicator * si = top_SI; si; si = si->get_parent())
+   for (StateIndicator * si = SI_top(); si; si = si->get_parent())
        count += si->show_owners(out, value);
 
-   loop(f, expunged_functions.size())
+   loop(f, the_workspace.expunged_functions.size())
        {
          char cc[100];
          snprintf(cc, sizeof(cc), "    ⎕EX[%lld] ", f);
-         count += expunged_functions[f]->show_owners(cc, out, value);
+         count += the_workspace.expunged_functions[f]->show_owners(cc, out, value);
        }
 
    return count;
@@ -467,7 +477,7 @@ int count = 0;
 int
 Workspace::cleanup_expunged(ostream & out, bool & erased)
 {
-const int ret = expunged_functions.size();
+const int ret = the_workspace.expunged_functions.size();
 
    if (SI_entry_count() > 0)
       {
@@ -477,10 +487,10 @@ const int ret = expunged_functions.size();
         return ret;
       }
 
-   while(expunged_functions.size())
+   while(the_workspace.expunged_functions.size())
        {
-         const UserFunction * ufun = expunged_functions.back();
-         expunged_functions.pop_back();
+         const UserFunction * ufun = the_workspace.expunged_functions.back();
+         the_workspace.expunged_functions.pop_back();
          out << "finally deleting " << ufun->get_name() << "...";
          delete ufun;
          out << " OK" << endl;
@@ -493,7 +503,7 @@ const int ret = expunged_functions.size();
 void
 Workspace::replace_arg(Value_P old_value, Value_P new_value)
 {
-   for (StateIndicator * si = top_SI; si; si = si->get_parent())
+   for (StateIndicator * si = SI_top(); si; si = si->get_parent())
        {
         if (si->replace_arg(old_value, new_value))   break;
        }
@@ -508,15 +518,15 @@ Workspace::clear_WS(ostream & out, bool silent)
 
    // clear the symbol table
    //
-   symbol_table.clear(out);
+   the_workspace.symbol_table.clear(out);
 
    // clear the )MORE error info
    //
-   more_error.clear();
+   more_error().clear();
 
    // clear the read/write system variables...
    //
-#define rw_sv_def(x) v_quad_ ## x.clear_vs();
+#define rw_sv_def(x) get_v_quad_ ## x().clear_vs();
 #define ro_sv_def(x)
 #include "SystemVariable.def"
 
@@ -525,26 +535,26 @@ Workspace::clear_WS(ostream & out, bool silent)
    //
 // Value::erase_all(out);
 
-#define rw_sv_def(x) new  (&v_quad_ ##x)  Quad_ ##x;
+#define rw_sv_def(x) new  (&get_v_quad_ ##x())  Quad_ ##x;
 #define ro_sv_def(x)
 #include "SystemVariable.def"
 
-   WS_name = UCS_string("CLEAR WS");
+   set_WS_name(UCS_string("CLEAR WS"));
    if (!silent)   out << "CLEAR WS" << endl;
 }
 //-----------------------------------------------------------------------------
 void
 Workspace::clear_SI(ostream & out)
 {
-   while (top_SI)
+   while (SI_top())
       {
-        top_SI->clear(out);
+        SI_top()->clear(out);
         pop_SI(LOC);
       }
 }
 //-----------------------------------------------------------------------------
 void
-Workspace::list_SI(ostream & out, SI_mode mode) const
+Workspace::list_SI(ostream & out, SI_mode mode)
 {
    for (const StateIndicator * si = SI_top(); si; si = si->get_parent())
        si->list(out, mode);
@@ -561,12 +571,12 @@ Workspace::save_WS(ostream & out, vector<UCS_string> & lib_ws)
 
    if (lib_ws.size() == 0)   // no argument: use )WSID value
       {
-         lib_ws.push_back(WS_name);
+         lib_ws.push_back(the_workspace.WS_name);
       }
    else if (lib_ws.size() > 2)   // too many arguments
       {
         out << "BAD COMMAND" << endl;
-        more_error = UCS_string("too many parameters in command )SAVE");
+        more_error() = UCS_string("too many parameters in command )SAVE");
         return;
       }
 
@@ -588,14 +598,14 @@ UTF8_string filename = LibPaths::get_lib_filename(libref, wname, false, "xml");
       {
         // filename does not end with .xml, so we try filename.xml
         //
-        filename += UTF8_string(".xml");
+        filename.append(UTF8_string(".xml"));
       }
 
 
    if (wname.compare(UCS_string("CLEAR WS")) == 0)   // don't save CLEAR WS
       {
         COUT << "NOT SAVED: THIS WS IS " << wname << endl;
-        more_error = UCS_string(
+        more_error() = UCS_string(
         "the workspace was not saved because 'CLEAR WS' is a special \n"
         "workspace name that cannot be saved. Use )WSID <name> first.");
         return;
@@ -604,22 +614,24 @@ UTF8_string filename = LibPaths::get_lib_filename(libref, wname, false, "xml");
    // dont save if workspace names differ and file exists
    //
    {
-     if (access((const char *)filename.c_str(), F_OK) == 0)   // file exists
+     if (access(filename.c_str(), F_OK) == 0)   // file exists
         {
-          if (wname.compare(WS_name) != 0)   // names differ
+          if (wname.compare(the_workspace.WS_name) != 0)   // names differ
              {
                COUT << "NOT SAVED: THIS WS IS " << wname << endl;
-               UCS_string & t4 = more_error;
+               UCS_string & t4 = more_error();
                t4.clear();
-               t4.app("the workspace was not saved because:\n"
-                      "   the workspace name '"); t4 += WS_name;
-               t4.app("' of )WSID\n   does not match the name '");
-               t4  += wname;
-               t4.app("' used in the )SAVE command\n"
+               t4.append_utf8("the workspace was not saved because:\n"
+                      "   the workspace name '");
+               t4.append(the_workspace.WS_name);
+               t4.append_utf8("' of )WSID\n   does not match the name '");
+               t4.append(wname);
+               t4.append_utf8("' used in the )SAVE command\n"
                       "   and the workspace file\n   ");
-               t4.app(filename.c_str());
-               t4.app("\n   already exists. Use )WSID "); t4  += wname;
-               t4.app(" first."); 
+               t4.append_utf8(filename.c_str());
+               t4.append_utf8("\n   already exists. Use )WSID ");
+               t4.append(wname);
+               t4.append_utf8(" first."); 
                return;
              }
         }
@@ -627,21 +639,22 @@ UTF8_string filename = LibPaths::get_lib_filename(libref, wname, false, "xml");
 
    // at this point it is OK to rename and save the workspace
    //
-   WS_name = wname;
+   the_workspace.WS_name = wname;
 
-ofstream outf((const char *)filename.c_str(), ofstream::out);
+ofstream outf(filename.c_str(), ofstream::out);
    if (!outf.is_open())   // open failed
       {
-        CERR << "Unable to )SAVE workspace '" << WS_name << "'." << endl;
+        CERR << "Unable to )SAVE workspace '"
+             << the_workspace.WS_name << "'." << endl;
         return;
       }
 
-XML_Saving_Archive ar(outf, *this);
-   ar << *this;
+XML_Saving_Archive ar(outf);
+   ar.save();
 
    // print time and date to COUT
    {
-     const int offset = v_quad_TZ.get_offset();
+     const int offset = get_v_quad_TZ().get_offset();
      const YMDhmsu time(now());
 const char * tz_sign = (offset < 0) ? "" : "+";
 
@@ -671,7 +684,7 @@ LibRef libref = LIB_NONE;
 UCS_string wname = lib_ws.back();
 UTF8_string filename = LibPaths::get_lib_filename(libref, wname, true, "xml");
 
-XML_Loading_Archive in((const char *)filename.c_str(), *this);
+XML_Loading_Archive in(filename.c_str());
 
    if (!in.is_open())   // open failed: try filename.xml unless already .xml
       {
@@ -683,9 +696,8 @@ XML_Loading_Archive in((const char *)filename.c_str(), *this);
            {
              // filename does not end with .xml, so we try filename.xml
              //
-             filename += UTF8_string(".xml");
-             new (&in) XML_Loading_Archive((const char *)filename.c_str(),
-                                                         *this);
+             filename.append(UTF8_string(".xml"));
+             new (&in) XML_Loading_Archive(filename.c_str());
            }
 
         if (!in.is_open())   // open failed again: give up
@@ -702,7 +714,7 @@ XML_Loading_Archive in((const char *)filename.c_str(), *this);
    // got open file. We assume that from here on everything will be fine.
    // clear current WS and load it from file
    //
-   clear_WS(CERR, true);
+   the_workspace.clear_WS(CERR, true);
    in.read_Workspace();
    return Workspace::get_LX();
 }
@@ -735,7 +747,7 @@ LibRef libref = LIB_NONE;
 UCS_string wname = lib_ws.back();
 UTF8_string filename = LibPaths::get_lib_filename(libref, wname, true, "xml");
 
-XML_Loading_Archive in((const char *)filename.c_str(), *this);
+XML_Loading_Archive in(filename.c_str());
 
    if (!in.is_open())   // open failed: try filename.xml unless already .xml
       {
@@ -747,9 +759,8 @@ XML_Loading_Archive in((const char *)filename.c_str(), *this);
            {
              // filename does not end with .xml, so we try filename.xml
              //
-             filename += UTF8_string(".xml");
-             new (&in) XML_Loading_Archive((const char *)filename.c_str(),
-                                                         *this);
+             filename.append(UTF8_string(".xml"));
+             new (&in) XML_Loading_Archive(filename.c_str());
            }
 
         if (!in.is_open())   // open failed again: give up
@@ -776,7 +787,7 @@ Workspace::wsid(ostream & out, UCS_string arg)
 
    if (arg.size() == 0)   // inquire workspace name
       {
-        out << "IS " << WS_name << endl;
+        out << "IS " << the_workspace.WS_name << endl;
         return;
       }
 
@@ -789,8 +800,15 @@ Workspace::wsid(ostream & out, UCS_string arg)
            }
       }
 
-   out << "WAS " << WS_name << endl;
-   WS_name = arg;
+   out << "WAS " << the_workspace.WS_name << endl;
+   the_workspace.WS_name = arg;
 }
-//-----------------------------------------------------------------------------
+//=============================================================================
+PrintContext::PrintContext()
+: style(Workspace::get_PS()),
+  precision(Workspace::get_PP()),
+  width(Workspace::get_PW())
+{
+}
+//=============================================================================
 
