@@ -27,7 +27,6 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
-#include "../config.h"   // for _WANTED macros
 #include "buildtag.hh"
 #include "Command.hh"
 #include "Common.hh"
@@ -41,8 +40,9 @@
 #include "Quad_SVx.hh"
 #include "TestFiles.hh"
 #include "UserFunction.hh"
-#include "Value.hh"
+#include "ValueHistory.hh"
 #include "Workspace.hh"
+#include "Value.hh"
 
 #ifndef PATH_MAX
 #define PATH_MAX 4096
@@ -51,6 +51,7 @@
    // global flags
    //
 bool silent = false;
+bool emacs_mode = false;
 bool do_not_echo = false;
 
 //-----------------------------------------------------------------------------
@@ -62,10 +63,11 @@ rlimit rl;
    getrlimit(RLIMIT_AS, &rl);
    total_memory = rl.rlim_cur;
 
+   Output::init();
    Avec::init();
    LibPaths::init(argv0);
    Value::init();
-//   PrefixNode::print_all(cerr);
+   VH_entry::init();
 }
 //-----------------------------------------------------------------------------
 /// the opposite of init()
@@ -103,8 +105,16 @@ static struct sigaction new_control_C_action;
 static void
 seg_fault(int)
 {
-   do_Assert(0, "SEGMENTATION FAULT ", 0, 0);
-   CERR << "\n\n====================================================\n";
+   CERR << "\n\n====================================================\n"
+           "SEGMENTATION FAULT" << endl;
+
+   Backtrace::show(__FILE__, __LINE__);
+
+   CERR << "====================================================\n";
+
+   // count errors
+   TestFiles::assert_error();
+
    Command::cmd_OFF(3);
 }
 
@@ -339,6 +349,13 @@ show_configure_options()
 #endif
    << endl <<
 
+#ifdef VALUE_HISTORY_WANTED
+   "    VALUE_HISTORY_WANTED=yes"
+#else
+   "    VALUE_HISTORY_WANTED=no (default)"
+#endif
+   << endl <<
+
 #ifdef VF_TRACING_WANTED
    "    VF_TRACING_WANTED=yes"
 #else
@@ -381,8 +398,10 @@ _("    -l num               turn log facility num (1-%d) ON\n"), LID_MAX-1);
 "    --par proc           use processor parent ID proc (default: no parent)\n"
 "    -w milli             wait milli milliseconds at startup\n"
 "    --noCIN              do not echo input(for scripting)\n"
-"    --noCONT             do not load CONTINUE workspace on startup)\n"
+"    --rawCIN             do not use the readline lib for input\n"
 "    --[no]Color          start with ]XTERM ON [OFF])\n"
+"    --noCONT             do not load CONTINUE workspace on startup)\n"
+"    --emacs              run in emacs mode\n"
 "    --[no]SV             [do not] start APnnn (a shared variable server)\n"
 "    --cfg                show ./configure options used and exit\n"
 "    --gpl                show license (GPL) and exit\n"
@@ -396,6 +415,7 @@ _("    -l num               turn log facility num (1-%d) ON\n"), LID_MAX-1);
 "                         2:   continue (don't exit) after last testcase\n"
 "                         3:   stop testcases after first error (don't exit)\n"
 "                         4:   exit after first error\n"
+"    --TR                 randomize order of testfiles\n"
 "    --TS                 append to (rather than override) summary.log\n"
 "    --                   end of options for %s\n"), prog1);
    CERR << cc << endl;
@@ -442,10 +462,20 @@ char c2[200];
 
 const char * lines[] =
 {
+  "",
+  "   ______ _   __ __  __    ___     ____   __ ",
+  "  / ____// | / // / / /   /   |   / __ \\ / / ",
+  " / / __ /  |/ // / / /   / /| |  / /_/ // /  ",
+  "/ /_/ // /|  // /_/ /   / ___ | / ____// /___",
+  "\\____//_/ |_/ \\____/   /_/  |_|/_/    /_____/",
+
+
+
   "",
   c1,
   "",
   "Copyright (C) 2008-2013  Dr. Jürgen Sauermann",
+  "Banner by FIGlet: www.figlet.org",
   "",
 _("This program comes with ABSOLUTELY NO WARRANTY;"),
   c2,
@@ -544,7 +574,8 @@ struct user_preferences
      do_sv(true),
      daemon(false),
      append_summary(false),
-     wait_ms(0)
+     wait_ms(0),
+     randomize_testfiles(false)
    {}
 
    /// load workspace CONTINUE on start-up
@@ -570,6 +601,9 @@ struct user_preferences
 
    /// wait at start-up
    int wait_ms;
+
+   /// randomize the order of testfiles
+   bool randomize_testfiles;
 };
 //-----------------------------------------------------------------------------
 // read user preference file(s) if present
@@ -654,9 +688,28 @@ int line = 0;
 
          const bool yes_no  = yes || no;
 
-         if (yes_no && !strcasecmp(opt, "Color"))
+         if (!strcasecmp(opt, "Color"))
             {
-              up.do_Color = yes;
+              if (yes_no)   // user said "Yes" or "No"
+                 {
+                   // if the user said "No" then use_curses is not touched.
+                   // if the user said "Yes" (which is an obsolete setting)
+                   // then use_curses is cleared because at the time when
+                   // "Yes" was valid, curses was not yet implemented.
+                   //
+                   up.do_Color = yes;
+                   if (yes)   Output::use_curses = false;
+                 }
+              if (!strcasecmp(arg, "ANSI"))
+                 {
+                   up.do_Color = true;
+                   Output::use_curses = false;
+                 }
+              if (!strcasecmp(arg, "CURSES"))
+                 {
+                   up.do_Color = true;
+                   Output::use_curses = true;
+                 }
             }
          else if (yes_no && !strcasecmp(opt, "Welcome"))
             {
@@ -685,6 +738,11 @@ int line = 0;
               loop(p, count - 1)   Output::color_CERR[p] = (char)(d[p] & 0xFF);
               Output::color_CERR[count - 1] = 0;
             }
+         else if (!strcasecmp(opt, "UERR-SEQUENCE"))
+            {
+              loop(p, count - 1)   Output::color_UERR[p] = (char)(d[p] & 0xFF);
+              Output::color_UERR[count - 1] = 0;
+            }
          else if (!strcasecmp(opt, "RESET-SEQUENCE"))
             {
               loop(p, count - 1)   Output::color_RESET[p] = (char)(d[p] & 0xFF);
@@ -695,6 +753,38 @@ int line = 0;
               loop(p, count - 1)   Output::clear_EOL[p] = (char)(d[p] & 0xFF);
               Output::clear_EOL[count - 1] = 0;
             }
+         else if (!strcasecmp(opt, "CIN-FOREGROUND"))
+            {
+              Output::color_CIN_foreground = atoi(arg);
+            }
+         else if (!strcasecmp(opt, "CIN-BACKGROUND"))
+            {
+              Output::color_CIN_background = atoi(arg);
+            }
+         else if (!strcasecmp(opt, "COUT-FOREGROUND"))
+            {
+              Output::color_COUT_foreground = atoi(arg);
+            }
+         else if (!strcasecmp(opt, "COUT-BACKGROUND"))
+            {
+              Output::color_COUT_background = atoi(arg);
+            }
+         else if (!strcasecmp(opt, "CERR-FOREGROUND"))
+            {
+              Output::color_CERR_foreground = atoi(arg);
+            }
+         else if (!strcasecmp(opt, "CERR-BACKGROUND"))
+            {
+              Output::color_CERR_background = atoi(arg);
+            }
+         else if (!strcasecmp(opt, "UERR-FOREGROUND"))
+            {
+              Output::color_UERR_foreground = atoi(arg);
+            }
+         else if (!strcasecmp(opt, "UERR-BACKGROUND"))
+            {
+              Output::color_UERR_background = atoi(arg);
+            }
          else if (!strncasecmp(opt, "LIBREF-", 7))
             {
               const int lib_ref = atoi(opt + 7);
@@ -729,7 +819,6 @@ user_preferences up;
    read_config_file(true,  up);   // /etc/gnu-apl.d/preferences
    read_config_file(false, up);   // $HOME/.gnu_apl
 
-
    // struct sigaction differs between GNU/Linux and other systems, which
    // causes direct bracket assignment to not compile on some machines.
    //
@@ -746,9 +835,7 @@ user_preferences up;
 
    sigaction(SIGINT,  &new_control_C_action, &old_control_C_action);
    sigaction(SIGUSR1, &new_USR1_action,      &old_USR1_action);
-// sigaction(SIGSEGV, &new_segfault_action,  &old_segfault_action);
-
-Workspace w;
+   sigaction(SIGSEGV, &new_segfault_action,  &old_segfault_action);
 
    // init NLS so that usage() will be translated
    //
@@ -831,10 +918,18 @@ Workspace w;
             {
               do_not_echo = true;
             }
+         else if (!strcmp(opt, "--rawCIN"))
+            {
+              Input::use_readline = false;
+            }
          else if (!strcmp(opt, "--noCONT"))
             {
               up.do_CONT = false;
             }
+         else if (!strcmp(opt, "--emacs"))
+            {
+              emacs_mode = true;
+            }
          else if (!strcmp(opt, "--SV"))
             {
               up.do_sv = true;
@@ -911,6 +1006,10 @@ Workspace w;
                    return 5;
                  }
             }
+         else if (!strcmp(opt, "--TR"))
+            {
+              up.randomize_testfiles = true;
+            }
          else if (!strcmp(opt, "--TS"))
             {
               up.append_summary = true;
@@ -938,6 +1037,45 @@ Workspace w;
             }
        }
 
+   if (emacs_mode)
+      {
+        // CIN = U+F00C0 = UTF8 F3 B0 83 80 ...
+        Output::color_CIN[0] = 0xF3;
+        Output::color_CIN[1] = 0xB0;
+        Output::color_CIN[2] = 0x83;
+        Output::color_CIN[3] = 0x80;
+        Output::color_CIN[4] = 0;
+
+        // COUT = U+F00C1 = UTF8 F3 B0 83 81 ...
+        Output::color_COUT[0] = 0xF3;
+        Output::color_COUT[1] = 0xB0;
+        Output::color_COUT[2] = 0x83;
+        Output::color_COUT[3] = 0x81;
+        Output::color_COUT[4] = 0;
+
+        // CERR = U+F00C2 = UTF8 F3 B0 83 82 ...
+        Output::color_CERR[0] = 0xF3;
+        Output::color_CERR[1] = 0xB0;
+        Output::color_CERR[2] = 0x83;
+        Output::color_CERR[3] = 0x82;
+        Output::color_CERR[4] = 0;
+
+        // UERR = U+F00C3 = UTF8 F3 B0 83 83 ...
+        Output::color_UERR[0] = 0xF3;
+        Output::color_UERR[1] = 0xB0;
+        Output::color_UERR[2] = 0x83;
+        Output::color_UERR[3] = 0x83;
+        Output::color_UERR[4] = 0;
+
+        // no clear_EOL
+        Output::clear_EOL[0] = 0;
+      }
+
+   // init input after reading the command line options, so that the user
+   // has a chance to disable readline
+   //
+   Input::init();
+
    if (up.daemon)
       {
         if (fork())   return 0;   // parent returns
@@ -952,6 +1090,7 @@ Workspace w;
    Log(LOG_argc_argv)   show_argv(argc, argv);
 
    TestFiles::testcase_count = TestFiles::test_file_names.size();
+   if (up.randomize_testfiles)   TestFiles::randomize_files();
 
    if (ProcessorID::init(up.do_sv, up.requested_id, up.requested_par))
       {
@@ -967,7 +1106,7 @@ Workspace w;
          UTF8_string filename =
             LibPaths::get_lib_filename(LIB0, "CONTINUE", true, "xml");
 
-         if (access((const char *)filename.c_str(), F_OK) == 0)
+         if (access(filename.c_str(), F_OK) == 0)
             {
               UCS_string load_cmd(")LOAD CONTINUE");
               Command::process_line(load_cmd);
@@ -976,8 +1115,8 @@ Workspace w;
 
    for (;;)
        {
-         Token t = w.immediate_execution(
-                   TestFiles::test_mode == TestFiles::TM_EXIT_AFTER_ERROR);
+         Token t = Workspace::immediate_execution(
+                       TestFiles::test_mode == TestFiles::TM_EXIT_AFTER_ERROR);
          if (t.get_tag() == TOK_OFF)   Command::cmd_OFF(0);
        }
 
